<!-- js-import.xsl
=============================================================================
Process XML representations of JavaScript

Todo:

  - Ensure that duplicate require and recommend objects are not duplicated
    during import.

  - Modern browsers should be able to handle long lines of JavaScript, but it
    would be nice to be able to trim lines of code w/o putting the entire
    object on a single line. Ideally, lines could also be stripped of non-
    syntactic whitespace.

=============================================================================
-->

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
  <xsl:template name="js-src">
    <xsl:param name="src"/>
    <xsl:param name="use"/>
    <xsl:choose>
      <xsl:when test="$use">
        <xsl:for-each select="$use/obj">
          <xsl:variable name="obj"><xsl:value-of select="."/></xsl:variable>
          <xsl:apply-templates select="document($src)/*/js-var[@name=$obj] | document($src)/*/js-function[@name=$obj]">
            <xsl:with-param name="src" select="$src"/>
            <xsl:with-param name="use-recommended" select="@use-recommended"/>
            <xsl:with-param name="ignore-requirements" select="@ignore-requirements"/>
          </xsl:apply-templates>
        </xsl:for-each>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates select="document($src)/JavaScript"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="JavaScript">
    <xsl:apply-templates select="js-var">
      <xsl:with-param name="ignore-requirements" select="1"/>
    </xsl:apply-templates>
    <xsl:apply-templates select="js-function">
      <xsl:with-param name="ignore-requirements" select="1"/>
    </xsl:apply-templates>
  </xsl:template>

  <xsl:template match="js-var">
    <xsl:param name="ignore-requirements"/>
    <xsl:param name="use-recommended"/>
    <xsl:param name="src"/>
    <xsl:if test="not($ignore-requirements)">
      <xsl:apply-templates select="require">
        <xsl:with-param name="src" select="$src"/>
      </xsl:apply-templates>
    </xsl:if>
    <xsl:variable name="body"><xsl:apply-templates select="body"/></xsl:variable>
    var <xsl:value-of select="@name"/> = <xsl:text disable-output-escaping="yes"><xsl:value-of select="normalize-space(string($body))"/></xsl:text>
    <xsl:if test="$use-recommended">
      <xsl:apply-templates select="recommend">
        <xsl:with-param name="src" select="$src"/>
      </xsl:apply-templates>
    </xsl:if>
  </xsl:template>

  <xsl:template match="js-function">
    <xsl:param name="ignore-requirements"/>
    <xsl:param name="use-recommended"/>
    <xsl:param name="src"/>
    <xsl:if test="not($ignore-requirements)">
      <xsl:apply-templates select="require">
        <xsl:with-param name="src" select="$src"/>
      </xsl:apply-templates>
    </xsl:if>
    <xsl:variable name="body"><xsl:apply-templates select="body"/></xsl:variable>
    function <xsl:value-of select="@name"/>(<xsl:for-each select="args/arg"><xsl:value-of select="@name"/><xsl:if test="position()!=last()">, </xsl:if></xsl:for-each>) {<xsl:text disable-output-escaping="yes"><xsl:value-of select="normalize-space(string($body))"/>}</xsl:text>
    <xsl:if test="$use-recommended">
      <xsl:apply-templates select="recommend">
        <xsl:with-param name="src" select="$src"/>
      </xsl:apply-templates>
    </xsl:if>
  </xsl:template>

  <xsl:template match="js-var/body | js-function/body">
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="JavaScript//comment()">
    <!-- /*<xsl:value-of select="."/>*/ -->
  </xsl:template>

  <xsl:template match="require | recommend">
    <xsl:param name="src"/>
    <xsl:variable name="r" select="."/>
    <xsl:apply-templates select="document($src)/*/js-var[@name=$r] | document($src)/*/js-function[@name=$r]"/>
  </xsl:template>
</xsl:stylesheet>

<!-- Usage
=============================================================================

Example 1 - The XML:

  <JavaScript>
    <js-var name="greeting"><body>'Hey there, '</body></js-var>
    <js-var name="who"><body>'J. User'</body></js-var>

    <js-function name="annoy">
      <require>who</require>
      <args>
        <arg name="w"/>
      </args>
      <body><![CDATA[
        if (!w)
          w = who;
        return alert((greeting ? greeting : 'Hello, ') + w + '!');
      ]]></body>
    </js-function>
  </JavaScript>


Example 2 - Import the "annoy" function (and implicitly the "who" var):

  <xsl:include href="js-import.xsl"/>
  <xsl:template match="/">
   <SCRIPT>
     <xsl:call-template name="js-src">
       <xsl:with-param name="src">js.xml</xsl:with-param>
       <xsl:with-param name="obj">
         <obj>annoy</obj>
       </xsl:with-param>
     </xsl:call-template>
   </SCRIPT>
  </xsl:template>

Example 3 - Output of Example 2:

<SCRIPT>
  who = 'J. User';
  function annoy(w) {if (!w) w = who; return alert((greeting ? greeting : 'Hello, ') + w + '!');}
</SCRIPT>

Example 4 - Import the "greeting" var:

  <xsl:include href="js-import.xsl"/>
  <xsl:template match="/">
   <SCRIPT>
     <xsl:call-template name="js-src">
       <xsl:with-param name="src">js.xml</xsl:with-param>
       <xsl:with-param name="obj">
         <obj>greeting</obj>
       </xsl:with-param>
     </xsl:call-template>
   </SCRIPT>
  </xsl:template>

Example 5 - Import the entire library:

  <xsl:include href="js-import.xsl"/>
  <xsl:template match="/">
   <SCRIPT>
     <xsl:call-template name="js-src">
       <xsl:with-param name="src">js.xml</xsl:with-param>
     </xsl:call-template>
   </SCRIPT>
  </xsl:template>

=============================================================================
-->
