#!/usr/bin/perl -w

#############################################################
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
#############################################################
#
# ConfigDB.pm
#
# Functions to access and manipulate a configuration database.
#
package VMware::ConfigDB;
use strict;
use VMware::Tokenizer;
use Exporter   ();
use VMware::ExtHelpers qw(&Warning &Panic &Debug);
use vars       qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

BEGIN {
    # set the version for version checking
    $VERSION     = 1.00;
    @ISA         = qw(Exporter);
    @EXPORT_OK   = qw();
}

# The file mode to use for a config DB.
my $gDefaultFileMode = 00644;

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my ($dbFile) = @_;

    my $self = {};
    $self->{DB_FILE} = $dbFile;
    $self->{DB} = {};
    $self->{DB_CHANGED} = 0;
    bless($self, $class);
    return($self);
}

sub getDBFile() {
    my $self = shift;
    return($self->{DB_FILE});
}

sub set($$) {
    my $self = shift;
    my ($key, $value) = @_;
    if( defined($self->{DB}->{$key}) && $self->{DB}->{$key} eq $value) {
	return;
    }

    $self->markChanged();
    $self->{DB}->{$key} = $value;
}

sub unset($) {
    my $self = shift;
    my ($key) = @_;
    if( defined($self->{DB}->{$key}) ) {
	$self->markChanged();
	delete($self->{DB}->{$key});
    }
}

sub get($) {
    my $self = shift;
    my ($key) = @_;
    return($self->{DB}->{$key});
}

sub clear() {
    my $self = shift;
    undef(%{$self->{DB}});
    $self->markChanged();
}

sub getKeys() {
    my $self = shift;
    return( sort(keys(%{$self->{DB}})) );
}

# Creates a new DB file
sub createDBFile($) {
    my($file) = @_;
    my $mask = umask();

    umask(00077);

    if( -e $file ) {
	return(1);	
    }

    local *INSTALLDB;
    if( !open(INSTALLDB, '>' . $file) ) {
	umask($mask);
	return(0);
    }
    close(INSTALLDB);
    umask($mask);

    if( !chmod($gDefaultFileMode, $file) ) {
	Warning("Could not change the mode of file $file to " . sprintf("%o", $gDefaultFileMode) . 
		".  Reason: $!.\n");
    }

    return(1);
}

# Load the installer database
sub load {
    my $self = shift;
    local *INSTALLDB;
    my $file = $self->getDBFile();

    # Describes the mapping for escape sequences
    my $EscapeChar = "\\";
    my %EscapeSequenceMapping = ( "\\\\", "\\",
				  '\n', "\n",
				  '\t', "\t",
				  '\"', "\""
				  );
    my $Delimiters = "=";

    # XXX Need locking if multiple instances of ConfigDB are ever used concurrently
    #     - jhu
    if( !open(INSTALLDB, '<' . $file) ) {
	if( createDBFile($file) ) {
	    # XXX Set DB_CHANGED to 0 before returning -jhu
	    return(1);
	}
	Warning('Unable to open the installer database ' . $file . '.  Reason: ' . $!);
	return(0);
    }

    # Set up the tokenizer to understand the dictionary format with escaping
    my $tokenizer = VMware::Tokenizer->new();
    $tokenizer->setEscapeChar($EscapeChar);
    $tokenizer->setEscapeMapping(\%EscapeSequenceMapping);
    $tokenizer->setDelimiters($Delimiters);

    my $lineno = 0;

    # XXX Ought to undef everything here. -jhu

    while (<INSTALLDB>) {
	chomp();      # Remove trailing newlines
	$lineno++;

	# Remove comments. Cannot intersperse comments with quotes in this format
	s/\#.*//;

	# Prepare to tokenize by reseting the count to zero before tokenizing
	$tokenizer->resetCount();    
	my ($rc, $toks) = $tokenizer->tokenize($_);

	if( !$rc ) {
	    Warning('Error in parsing of string of database ' . $file . ' on line ' . $lineno . '.');
	    next;
	}
	my @tokens = @$toks;

	if( $#tokens == 2 && $tokens[1]->token() eq '=' ) {
	    my $key = $tokens[0]->token();
	    my $value = $tokens[2]->token();

	    if( substr($value, 0, 1) eq '"' ) {
		$value = substr($value, 1);
	    }

	    if( substr($value, length($value) - 1, 1) eq '"' ) {
		$value = substr($value, 0, -1);
	    }
	    Debug(3, "got '$key' and '$value'");
	    $self->set($key, $value);
	}
    }
    close(INSTALLDB);

    # We have loaded the db.  Mark it as not having changed.  
    $self->{DB_CHANGED} = 0;
    return(1);
}

# Mark the config DB as changed to force it to commit to disk
sub markChanged() {
    my $self = shift;
    $self->{DB_CHANGED} = 1;
}

# Indicates if the DB has changed and needs to be saved
sub hasChanged() {
    my $self = shift;
    return($self->{DB_CHANGED});
}

# Clear the DB changed flag
sub clearChanged() {
    my $self = shift;
    $self->{DB_CHANGED} = 0;
}

sub save() {
    my $self = shift;
    local *INSTALLDB;
    my $dbfile = $self->getDBFile();

    if( ! -e $dbfile ) {
	createDBFile($dbfile);
    }

    if( !open(INSTALLDB, '>' . $dbfile) ) {
	Warning("Unable to open the db file $dbfile.");
	return(0);
    }

    my $key;
    foreach $key ($self->getKeys()) {
	my $val = $self->{DB}->{$key};
	print INSTALLDB $key . ' = "' . $val . '"' . "\n";
    }
    close(INSTALLDB);

    $self->clearChanged();

    return(1);
}

1;


