#####################################################################
# DevNames.pm
#
# A basic class that contains a history of device names.
#####################################################################

package VMware::HConfig::DevNames;
use strict;
use VMware::ExtHelpers qw(&Warning &Panic &Debug);
use VMware::HConfig::Constant qw($gVmkDeviceType_nic $gVmkDeviceType_hba &c_getDir);
use VMware::HConfig::PCIDevice;
use VMware::ConfigDB;


#####################################################################
# Public Constructor Methods                                        #
#####################################################################

# The data structure used to store the name info is a hash.  The key
# represents the combination of PCI address/route and equivalence to
# which each name is bound. The reason why we use a one-level hash with
# the key representing all the info, is because in the future, we could
# switch from PCI address to PCI routes, or even attach device names to
# device serial numbers instead.  Having one level of hash key seems to
# provide the most flexibility.  Ex.:
# $devnames{"001:04.00.nic"} = "vmnic0";
# $devnames{"004:09:01.aic7xxx"} = "vmhba2";

# Constructs a new DevNames object.
sub new() {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};

    $self->{NUMBERS}    = {};    # used to number new devices
    $self->{NAMES}      = {};    # this is the hash of dev names

    bless($self, $class);
    return($self);
}


#####################################################################
# Public Methods                                                    #
#####################################################################


# HConfig::DevNames::getDeviceName() --
#   Returns the device name associated with a given device location.
#
# Args: ($PCIHash, $equiv) where $PCIHash is the bus:dev:fn PCI addr
#             and $equiv is the device equivalence, eg "nic".
#
# Returns: device name string, or "".
#
sub getDeviceName(@) {
    my $self = shift;
    my $hash = devLocationToHash(@_);

    if (defined($self->{NAMES}{$hash})) {
        return $self->{NAMES}{$hash};
    } else {
        return "";
    }
}


# HConfig::DevNames::setDeviceName() --
#   Changes the device name for a given device location.  Also keeps
#   track of the next vmnicN or vmhbaN number for use in auto-
#   generated device names.
#
# Args: ($PCIHash, $equiv, $newName)
#         where $PCIHash is the bus:dev:fn PCI addr
#             and $equiv is the device equivalence, eg "nic".
#
# Returns: Nothing
#
sub setDeviceName($$$) {
    my $self = shift;
    my($PCIhash,$equiv,$newName) = @_;
    my $hash = devLocationToHash($PCIhash, $equiv);

    $self->{NAMES}{$hash} = $newName;

    # update hash used to generate unique device names
    if ($newName =~ /^(\D+)(\d+)$/ ) {
        if (not(exists $self->{NUMBERS}{$1} and 
                $2 < $self->{NUMBERS}{$1})) {
            $self->{NUMBERS}{$1} = $2 + 1;
        }
    }
}


# HConfig::DevNames::suggestDeviceName() --
#   Given a PCI Device object, use an algorithm to generate a new
#   unique device name, of the form <vmnicN> or <vmhbaN>. 
#   Must be a manageable device, ie network or storage...
#
sub suggestDeviceName($) {
    my $self = shift;
    my $dev = shift;
    my $stem;

    if ($dev->isNetClassDevice()) {
        $stem = $gVmkDeviceType_nic;
    } elsif ($dev->isStorageClassDevice() or
             $dev->isSerialFiberDevice()) {
        $stem = $gVmkDeviceType_hba;
    } else {
        $stem = $dev->getEquiv();
    }

    # Build new device name
    if (!$stem) {
        Panic("No device equivalence exists for device\n" . $dev->toString());
        return undef;
    }

    if (exists $self->{NUMBERS}{$stem} ) {
        return $stem . $self->{NUMBERS}{$stem};
    } else {
        return $stem . "0";
    }
}


# HConfig::DevNames::createCompatibleNames() --
#   Create a device name configuration which preserves the old vmnicN /
#   vmhbaN device names.  The order of assigning names is by module
#   loaded; within each module, by increasing PCI address. This should
#   be done only once, during an upgrade or install, to create device
#   names for the first time.  If this is called when a device name
#   configuration already exists, the existing names will be
#   overwritten and device names won't be preserved.
# 
#   Basically this creates the same names as makedevnames.pl.
#
# Args: ($pciDevList)
#   $pciDevList    - list of PCI devices, as PCIDeviceList object
#
# Returns: the number of names created
#
sub createCompatibleNames($) {
    my $self = shift;
    my $pciDevList = shift;
    my $numCreated = 0;

    # start device naming at vmnic0 and vmhba0
    $self->{NUMBERS}{$gVmkDeviceType_hba} = 0;
    $self->{NUMBERS}{$gVmkDeviceType_nic} = 0;

    # Get list of vmk modules to be loaded
    my $driverList = $pciDevList->buildDriverList();

    # Loop for each module loaded
    my $module;
    foreach $module ($driverList->getDriverModules()) {
        next if $module eq "vmklinux";
        next if $module eq "nfshaper.o";

        # Go through device list, pick out ones owned 
        # by vmkernel or shared, and have matching module name
        my $matchFunc = sub { return $_[0]->getOwner() != VMware::HConfig::PCIDevice::HOST
                                  && defined($_[0]->getDrivers()->[0])
                                  && $_[0]->getDrivers()->[0] eq $module };
        my $matchedDevList = $pciDevList->filterPCIDevices($matchFunc);

        # getPCIDevices returns device list sorted by
        # bus/device/function. We name these devices.
        my $dev;
        foreach $dev ($matchedDevList->getPCIDevices()) {
            my $name = $self->suggestDeviceName($dev);

            $self->setDeviceName($dev->getHash(),
                                 $dev->getEquiv(),
                                 $name);
            $numCreated++;
        }
    }

    return $numCreated;
}


# HConfig::DevNames::deviceNameConfToString() --
#   For each VMKernel-owned/shared PCI device, writes out one line
#   containing the PCI address, equivalence, and device name, returning
#   the whole thing as a string. Missing device names will be generated
#   using suggestDeviceName().  If there are no VMkernel-owned devices,
#   then returns "# no devices found" or similar.  The list is sorted
#   by increasing PCI bus/device/function.
#
# Args: ($pciDevList)
#
# Returns: string, as specified above
#
sub deviceNameConfToString($) {
    my $self = shift;
    my $pciDevList = shift;
    my $s = "";

    # Go through device list, pick out VM-owned or shared devices 
    my $matchFunc = sub { return $_[0]->getOwner() != VMware::HConfig::PCIDevice::HOST
                              && defined($_[0]->getDrivers()->[0]) };
    my $matchedDevList = $pciDevList->filterPCIDevices($matchFunc);

    if ($matchedDevList->count() == 0) {
        Warning("No devices assigned to vmkernel.");
        $s .= "# no devices assigned to vmkernel\n";
    } else {
        # Loop through devices in filtered list, print line for each
        my $dev;
        foreach $dev ($matchedDevList->getPCIDevices()) {
            my $PCIhash = $dev->getHash();
            my $equiv = $dev->getEquiv();
            my $name = $self->getDeviceName($PCIhash, $equiv);

            if (!$name) {
                Warning("No name for device at $PCIhash, creating new name");
                $name = $self->suggestDeviceName($dev);
                $self->setDeviceName($PCIhash, $equiv, $name);
            }

            my $PCIaddr = PCIHashToHash($PCIhash);
            $s .= "$PCIaddr\t$equiv\t$name\n";
        }
    }

    return $s;
}


# function returning device names info in string form, for dump()
sub toString() {
    my $self = shift;
    my $s = "";
    my $stem;
    my $hash;

    foreach $stem (keys %{$self->{NUMBERS}} ) {
        $s .= "Next $stem is $stem" . $self->{NUMBERS}{$stem} . "\n";
    }

    foreach $hash (sort keys %{$self->{NAMES}} ) {
        $s .= "$hash\t" . $self->{NAMES}{$hash} . "\n";
    }

    return $s;
}


# HConfig::DevNames::findNamesInConfigDB() --
#   Finds the list of device names in the config database
#   object. If a label is supplied, looks under
#   devicenames.<label>.*, otherwise just looks under
#   devicenames.* . Writes over previous names, so best 
#   to call this after a new().
#
# Args: ($configDB, $label)
#   $configDB    - Config database object
#   $label       - Boot label, eg "vmnix"
#
# Returns: The number of names found, or 0.
# 
sub findNamesInConfigDB($$) {
    my $self = shift;
    my ($configDB, $label) = @_;
    my $count = 0;

    # Get all the device name entries for a given label
    my $prefix = "devicenames.";
    $prefix .= "$label." if $label;

    my @keys = grep(/^$prefix(\d+):(\d+)\.(\d+)/, $configDB->getKeys());

    # Go through each one and add it to our name list
    my $key;
    foreach $key (sort @keys) {
        if ($key =~ /^$prefix(.+)$/) {
            my ($PCIhash, $equiv) = hashToDevLocation($1);
            my $val = $configDB->get($key);

            $self->setDeviceName($PCIhash, $equiv, $val);
            $count++;
        }
    }

    return $count;
}


# HConfig::DevNames::writeToConfigDB() --
#   Saves the list of device names for a particular label
#   into the database object. Keys added would be, ex 
#   "devicenames.vmnix.<hash-key> = "vmnic0".  This does
#   NOT actually save the modified database to disk.
#
#   If no label is specified then the key would be like,
#   "devicenames.<hash-key> = vmnic0"
#
# Args: ($configDB, $label)
#   $configDB    - Config database object
#   $label       - Boot label, eg "vmnix"
#
# Returns: Nothing
#
sub writeToConfigDB($$) {
    my $self = shift;
    my ($configDB, $label) = @_;
    my %keyUsed;
    my $key;
    my $hash;

    my $prefix = "devicenames.";
    $prefix .= "$label." if $label;

    # Get all the device names that exist in the database
    my @keys = grep(/^$prefix(\d+):(\d+)\.(\d+)/, $configDB->getKeys());
    @keys = sort(@keys);

    foreach $key (@keys) {
	$keyUsed{$key} = 0;
    }

    # Iterate through the list of device names and add the info into the db.
    foreach $hash (sort keys %{$self->{NAMES}} ) {
        $configDB->set($prefix . $hash, $self->{NAMES}{$hash});
        $keyUsed{$prefix . $hash} = 1;
    }

    # Clean up the db and remove entries for db keys that are not
    # there any more.
    foreach $key (keys(%keyUsed)) {
	if( !$keyUsed{$key} ) {
	    $configDB->unset($key);
	}
    }
}


# HConfig::DevNames::getDeviceNameList() --
#   Returns a list of stored device names. Note that no PCI location
#   information is returned.  The list is sorted by increasing PCI
#   address, then by alphabetical Equivalence.
#
# Args: none
#
# Returns: sorted list of device names. Duplicates are possible.
#
sub getDeviceNameList() {
    my $self = shift;
    my @names = map { $self->{NAMES}{$_} } (sort keys %{$self->{NAMES}});

    return @names;
}

 
# HConfig::DevNames::findPCIDeviceByName() --
#   Looks up the PCI Location (PCI Addr., Equiv) for the given device
#   name.
#
# Args: ($devname)  - device name string
#
# Returns: ($PCIaddr, $equiv)  for the named device.
#   If more than one PCI addr/Equiv combo has the same device name,
#   multiple PCI addr, Equiv pairs will be returned as a list.
#
sub findPCIDeviceByName($) {
    my $self = shift;
    my $wantedName = shift;
    my @pairs;
    my @hashes = grep ($self->{NAMES}{$_} eq $wantedName,
                       sort keys %{$self->{NAMES}} );
    my $hash;
    foreach $hash (@hashes) {
        my ($PCIhash, $equiv) = hashToDevLocation($hash);
        @pairs = (@pairs, $PCIhash, $equiv);
    }

    return @pairs;
}


#####################################################################
# Methods not intended for use by user                              #
#####################################################################


#####################################################################
# Private Static Methods (indep. of each object)                    #
#####################################################################

# Convert device location parameters to hash.
# Args: (PCI Address, Equiv).
#  PCI Address is the hash as returned by PCIDevice::hash()
#
# Hash format is "bbb:ss.ff.eee", where
# bbb = PCI Bus #, decimal
# ss  = PCI Slot #, decimal
# ff  = PCI function #
# eee = Equivalence string, eg "nic", etc...
#
# This is a static method to be overloaded by derived classes
# that implement PCI routing or Serial Number-based indexing.
sub devLocationToHash($$) {
    my ($PCIhash, $equiv) = @_;
    my $hash = PCIHashToHash($PCIhash);

    if ($hash ne "") {
        return "$hash.$equiv";
    } else {
        Panic("Illegal PCI location $PCIhash\n");
        return "";
    }

}


# Converts a PCIDevice-style PCI addr hash (bus:slot:fn) to our own hash style
sub PCIHashToHash($) {
    my $PCIhash = shift;
    my ($bus, $device, $function) = VMware::HConfig::PCIDevice::hashToBusDevFunc($PCIhash);

    if (defined($bus) && defined($device) && defined($function)) {
        return sprintf("%03d:%02d.%1d",
                       $bus, $device, $function);
    } else {
        return "";
    }
}


# hashToDevLocation() --
#  converts device name hash to PCI addr, equiv fields
#
# Args: ($hash)
#
# Returns: ($PCIHash, $equiv) where $PCIHash is the bus:dev:fn PCI addr
#             and $equiv is the device equivalence, eg "nic".
#
sub hashToDevLocation($) {
    my $namehash = shift;

    if ($namehash =~ /^(\d+):(\d+)\.(\d+)\.(\w+)/ ) {
        my $PCIhash = VMware::HConfig::PCIDevice::hash($1, $2, $3);
        return ($PCIhash, $4);
    } else {
        Warning("Illegal device name key $namehash");
        return (undef, undef);
    }
}

1;
