#!/usr/bin/perl -w

#############################################################
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
#############################################################
#
# LILOBootImage.pm
# 
# A basic class that contains information representing the lilo.conf
# configuration file.  Information includes a series of options, both
# global and local.

package VMware::HConfig::LILOBootImage;
use strict;

my @VALID_TYPES = ("image", "other");

#####################################################################
# Public Static Methods (indep. of each object)                     #
#####################################################################

sub new($$) {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my ($type, $image) = @_;
    my $self = create($class);

    if( isValidType($type) ) {
	$self->{TYPE} = $type;
    }
    else {
	$self->{TYPE} = $VALID_TYPES[0];  # Set to default
    }

    $self->{IMAGE} = $image;
    $self->{OPTIONS} = "";

    return($self);
}

# Checks if the arg is of a valid type
sub isValidType($) {
    my ($type) = @_;

    my $t;

    foreach $t (@VALID_TYPES) {  
	if($t eq $type) { 
	    return(1);
	}
    }

    return(0);
}

sub isVMnixImage($) {
    my ($img) = @_;
    return($img =~ /vmnix/);
}

sub isIdeRootDev($) {
    my ($root) = @_;
    return($root =~ /^\/dev\/hd/);
}
	

#####################################################################
# Public Methods                                                    #
#####################################################################

# Accessor functions
sub getType() {
    my $self = shift;   
    return( $self->{TYPE} );
}

sub getImage() {
    my $self = shift;   
    return( $self->{IMAGE} );
}

sub getLabel() {
    my $self = shift;   
    return( $self->{LABEL} );
}

sub getInitrd() {
    my $self = shift;   
    return( $self->{INITRD} );
}

sub getOptions() {
    my $self = shift;   
    return( $self->{OPTIONS} );
}

sub getMemSize() {
    my $self = shift;   
    return( $self->{MEMSIZE} );
}

sub getRootDev($) {
    my $self = shift;
    return( $self->{RDEV} );
}

sub getCPCI() {
    my $self = shift;   
    return( $self->{CPCI} );
}

sub getOtherAppend() {
    my $self = shift;   
    return( $self->{OTHER_APPEND} );
}

sub usesVMnixImage() {
    my $self = shift;   
    return( isVMnixImage($self->getImage()) );
}

sub usesIdeRootDev() {
    my $self = shift;   
    return( isIdeRootDev($self->getRootDev()) );
}

sub toString() {
    my ($self) = shift;
    my $s = "";
    $s = $self->getType() . "=" . $self->getImage() . "\n" . 
	"\tlabel=" . $self->getLabel() . "\n";

    # Add the root device if it is a Linux/VMnix OS
    if( $self->getType() eq "image" ) {
	$s .= "\troot=" . $self->getRootDev() . "\n";

	# Add the initrd options if it exists, but mandatory if it does not use 
	# an ide root device
	if( $self->getInitrd() ne "" && -f $self->getInitrd() ) {
	    $s .= "\tinitrd=" . $self->getInitrd() . "\n";
	}
    }

    # Add the remaining options
    $s .= $self->getOptions();

    if( isVMnixImage($self->getImage()) ) {
	$s .= 
	    "\tappend=\"mem=" . $self->getMemSize() . "M console=ttyS0,115200 console=tty0 cpci=" . $self->getCPCI() . 
	    ((length($self->getOtherAppend()) > 0) ? (" " . $self->getOtherAppend) : "") .
		"\"\n";
    }

    # Not exactly correct if there needs to be any escape characters.
    # This assumes that escape sequences are not needed in the append
    # line.  Not guaranteed to be true.
    else {
        my $append = "";
        if (length($self->getMemSize()) > 0) {
           $append .= length($append) > 0 ? " " : "";
           $append .= "mem=" . $self->getMemSize() . "M";
        }
        if (length($self->getOtherAppend()) > 0) {
           $append .= length($append) > 0 ? " " : "";
           $append .= $self->getOtherAppend();
        }

        if (length($append) > 0) {
           $s .= "\tappend=\"" . $append . "\"\n";
        }
    }

    return($s);
}

    

# Mutator functions
sub setType($) {
    my $self = shift;
    $self->{TYPE} = shift;
}

sub setImage($) {
    my $self = shift;
    $self->{IMAGE} = shift;
}

sub setLabel($) {
    my $self = shift;
    $self->{LABEL} = shift;
}

sub setInitrd($) {
    my $self = shift;
    $self->{INITRD} = shift;
}

sub setOptions($) {
    my $self = shift;
    $self->{OPTIONS} = shift;
}

sub setMemSize($) {
    my $self = shift;
    $self->{MEMSIZE} = shift;
}

sub setRootDev($) {
    my $self = shift;
    $self->{RDEV} = shift;
}

sub setCPCI($) {
    my $self = shift;
    $self->{CPCI} = shift;
}

sub setOtherAppend($) {
    my $self = shift;
    $self->{OTHER_APPEND} = shift;
}


#####################################################################
# Methods not intended for use by user                              #
#####################################################################

sub create($) {
    my ($class) = @_;
    my ($self) = {};
    $self->{TYPE} = "image";
    $self->{IMAGE} = "";
    $self->{LABEL} = "";
    $self->{OPTIONS} = "";
    $self->{MEMSIZE} = "";
    $self->{RDEV} = "";
    $self->{CPCI} = "";
    $self->{OTHER_APPEND} = "";
    $self->{INITRD} = "";

    bless($self, $class);        
    return($self);
}

1;
