#!/usr/bin/perl -w

#############################################################
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
#############################################################
#
# MFSFileSystem.pm
#
# A basic class that contains basic information for each file system.
# and provides ways to do things in a file system.


package VMware::HConfig::MFSFileSystemRemote;
use strict;
use VMware::ExtHelpers qw(&Warning &System &Panic &shell_string);
use VMware::HConfig::Constant qw(c_getHelper);
use VMware::HConfig::RawDisk;
use VMware::HConfig::MFSFileSystem;
use VMware::HConfig::SSHostRemote;
use Exporter   ();
use vars       qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

BEGIN {
    # set the version for version checking
    $VERSION     = 1.00;
    @ISA         = qw(Exporter VMware::HConfig::MFSFileSystem);
    @EXPORT_OK   = qw();
}

use vars qw();

#####################################################################
# Public Static Methods (indep. of each object                      #
#####################################################################

# Takes in a RawDisk object and partition which is information about
# where this file system can be found.
sub new($$$$) {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my ($disk, $part, $server, $ctx) = @_;

    if (VMware::HConfig::RawDisk::isLogVmkDiskName($disk)) {
	# Returns successfully on if the RawDisk's name is of the form "scsi#:# and the
	# partition indicated refers to a valid MFS partition
	my $self = $proto->create($disk, $part);

        $self->{REMOTE_SERVER} = $server;
        $self->{VMDB_CTX} = $ctx;

        # Irrelevant to an MFS partition
        #	$self->{MOUNT_POINT} = "";   # Location where FS is mounted

        # Do not keep a back reference to the SSHost object or else we'll
        # have a circular reference and have to explicitly garbage collect.

	bless($self, $class);
	return($self);
    }
    else {
	return(undef);
    }
}

# Inherited from superclass
# sub create($$) {}

######################################################################
## Public Methods                                                    #
######################################################################

## Accessor functions
# Inherited from superclass
# sub getDisk() {}
# sub getPartition() {}
# sub getVmkName() {}
# sub getLogName() {}
# sub getCapacity() {}
# sub getFreeCapacity() {}
# sub getBlockSize() {}
# sub getAccessibility() {}
# sub getVersion() {}
# sub getMinorVersion() {}
# sub getExtents() {}
# sub toString() {}
# sub listFiles() {}


# NOT IMPLEMENTED
# sub setMountPoint() {}
# sub getMountPoint() {}


# Mutator functions
# Export a file from src to dest
#
# NOT IMPLEMENTED
sub exportFile($$) {
    my $self = shift;
    my ($src, $dest) = @_;
    Panic("Not Implemented for Remote Calls");

    if( !c_getHelper('vmfs_export_file') ) {
	Warning("Cannot execute vmfs_export_file.");
	return(0);
    }

    my $command = c_getHelper('vmfs_export_file') . ' ' . shell_string($dest) . ' ' . shell_string($self->getVmkName() . ":$src");

    # We'll keep stderr messages.  They'll either appear on the console or the
    # web server logs
    if(System($command)) {
	# Error
	Warning("Error in MFSFileSystem::exportFile: $!");
	return(0);
    }

    return(1);
}

# Import a file from src to dest
#
# NOT IMPLEMENTED
sub importFile($$) {
    my $self = shift;
    my ($src, $dest) = @_;
    Panic("Not Implemented for Remote Calls");

    if( !c_getHelper('vmfs_import_file') ) {
	Warning("Cannot execute vmfs_import_file.");
	return(0);
    }

    my $command = c_getHelper('vmfs_import_file') . ' ' . shell_string($src) . ' ' . shell_string($self->getVmkName() . ":$dest");

    # We'll keep stderr messages.  They'll either appear on the console or the
    # web server logs
    if(System($command)) {
	# Error
	Warning("Error in MFSFileSystem::importFile: $!");
	return(0);
    }

    return(1);
}

# Remove a file
#
# NOT IMPLEMENTED
sub removeFile($) {
    my $self = shift;
    my ($filename) = @_;
    Panic("Not Implemented for Remote Calls");

    if( !c_getHelper('vmfs_remove_file') ) {
	Warning("Cannot execute vmfs_remove_file.");
	return(0);
    }

    my $command = c_getHelper('vmfs_remove_file') . " " .
        shell_string($self->getPathname($filename));

    # We'll keep stderr messages.  They'll either appear on the console or the
    # web server logs
    if(System($command)) {
	Warning("Error in MFSFileSystem::removeFile: $!");
	return(0);
    }

    return(1);
}

# Create a file
#
# Size is in megabytes
#
# NOT IMPLEMENTED
sub createFile($$) {
    my $self = shift;
    my ($filename, $size) = @_;
    Panic("Not Implemented for Remote Calls");

    if( !c_getHelper('vmfs_create_file') ) {
	Warning("Cannot execute vmfs_create_file.");
	return(0);
    }

    my $command = c_getHelper('vmfs_create_file') . " " . shell_string($size . "M") . ' ' .
	shell_string($self->getVmkName() . ":$filename");

    # We'll keep stderr messages.  They'll either appear on the console or the
    # web server logs
    if(System($command)) {
	Warning("Error in MFSFileSystem::createFile: $!");
	return(0);
    }

    return(1);
}

# XXX The following methods should be implemented to match the semantics in the rest
#     of this module but are not.
# sub changeOwner($$) {}
# sub changeGroup($$) {}
# sub changeOwnerAndGroup($$$) {}

# Sets the VMFS logical name
sub setVMFSLogName($) {
   my $self = shift;
   my ($name) = @_;
   
   my $ctx = $self->getCtx();
   if (!defined($ctx)) {
      Panic("Cannot get VMFS information without a VMDB context.\n");
      return 0;
   }

   my $vmfsPath = $self->getVmfsCmdPath();

   $ctx->BeginTransaction();
   my $cmdPath = $ctx->NewArrayIndex($vmfsPath);
   $ctx->SetString($cmdPath . "op/", "setName");
   $ctx->SetString($cmdPath . "op/setName/in/label/", $name);
   $ctx->EndTransaction(1);
   
   # XXX There has to be a better way
   my ($status, $count) = 
     VMware::HConfig::SSHostRemote::waitForCmdStatus($ctx, $cmdPath, 30);
   if ($status eq "error") {
      my $errMsg = $ctx->GetString($cmdPath . "status/error/errMsg");
      Warning("$errMsg\n");
      $ctx->Unset($cmdPath);
      return 0;
   }

   $ctx->Unset($cmdPath);
   return(1);
}


# Sets VMFS accessibility (private/shared/public)
sub setAccessibility($) {
   my $self = shift;
   my ($accessibility) = @_;
   
   my $ctx = $self->getCtx();
   if (!defined($ctx)) {
      Panic("Cannot get VMFS information without a VMDB context.\n");
      return 0;
   }
   my $vmfsPath = $self->getVmfsCmdPath();
   
   $ctx->BeginTransaction();
   my $cmdPath = $ctx->NewArrayIndex($vmfsPath);
   $ctx->SetString($cmdPath . "op/", "setAccessibility");
   $ctx->SetString($cmdPath . "op/setAccessibility/in/access", $accessibility);
   $ctx->EndTransaction(1);

   # XXX There has to be a better way
   my ($status, $count) = 
     VMware::HConfig::SSHostRemote::waitForCmdStatus($ctx, $cmdPath, 30);
   if ($status eq "error") {
      my $errMsg = $ctx->GetString($cmdPath . "status/error/errMsg");
      Warning("$errMsg\n");
      $ctx->Unset($cmdPath);
      return 0;
   }

   $ctx->Unset($cmdPath);
   return(1);
}


# Attaches an extent
sub attachExtent($) {
   my $self = shift;
   my ($extent) = @_;

   my $in = makeServerdRequest('ESXHostEdit_AttachVMFSExtent');
   $in->setValue('.in.vmfs', $self->getVmkName());
   $in->setValue('.in.extent', $extent);

   my $out = $self->sendServerdRequest($in);

   my ($rc) = getReturnCode($out);
   if (!$rc) {
      ServerdWarning($out);
      return(0);
   }

   $in->dispose();
   $out->dispose();

   return(1);
}

# Convert VMFS-1 to VMFS-2
sub convertToVmfs2($) {
   my $self = shift;

   my $in = makeServerdRequest('ESXHostEdit_ConvertToVMFS2');
   $in->setValue('.in.vmfs', $self->getVmkName());

   my $out = $self->sendServerdRequest($in);

   my ($rc) = getReturnCode($out);
   if (!$rc) {
      ServerdWarning($out);
      return 0;
   }

   $in->dispose();
   $out->dispose();

   return 1;
}

#####################################################################
# Methods not intended for use by user                              #
#####################################################################

#####################################################################
# Methods Used to Support Remote Interface                          #
#####################################################################

sub getRemoteData() {
    my($self) = shift;
    if( defined($self->{DOM}) ) {
	my $doc = $self->{DOM}->getSubTree(".");
	if( !defined($doc) ) {
	    Panic("Did not get a DOMAccess node");
	}
	return($doc);
    } else {
	Panic("No remote data available when getRemoteData was called.");
    }
}

sub setRemoteData($) {
    my($self) = shift;
    my($doc) = @_;
    $self->{DOM} = $doc;
}

sub dumpRemoteData() {
    my($self) = shift;
    print $self->{DOM}->prettyPrint() . "\n";
}

sub getRemoteServer() {
   my($self) = shift;
   return $self->{REMOTE_SERVER};
}


sub getCtx() {
   my $self = shift;
   return $self->{VMDB_CTX};
}


sub getVmfsCmdPath($) {
   my $self = shift;

   my $vmkName = $self->getVmkName();
   my $vmfsCmdPath = "/vmkernel/config/storage/vmfs/#$vmkName/cmd/##/";
   
   return $vmfsCmdPath;
}

#####################################################################
# End of Methods Used to Support Remote Interface                   #
#####################################################################

sub loadFSInfo() {
    my $self = shift;

    my @files;

    my $doc = $self->getRemoteData();

    my $logname = $doc->getValue('.logical_name');
    my $capacity = $doc->getValue('.total_size');
    my $freecap = $doc->getValue('.free_size');
    my $blocksize = $doc->getValue('.block_size');
    my $accessibility = $doc->getValue('.accessibility');
    my $version = $doc->getValue('.version');
    my $minorversion = $doc->getValue('.minorversion');

    $self->{LOG_NAME} = $logname;
    $self->{CAPACITY} = $capacity;
    $self->{FREE_CAPACITY} = $freecap;
    $self->{BLOCK_SIZE} = $blocksize;
    $self->{ACCESSIBILITY} = $accessibility;
    $self->{VERSION} = $version;
    $self->{MINORVERSION} = $minorversion;

    # Get the list of extents.
    $self->{EXTENTS} = [];
    foreach my $extentKey ($doc->listElementNames('.extents')) {
       push(@{$self->{EXTENTS}}, $doc->getValue('.extents.' . $extentKey));
    }

    # Get the list of files.
    my @fileKeys = $doc->listElementNames('.files');
    my $fileKey;

    foreach $fileKey (@fileKeys) {
	my $key_file = '.files.' . $fileKey;

	my $name = $doc->getValue("$key_file.name");
	my $size = $doc->getValue("$key_file.size");
	my $rwx  = $doc->getValue("$key_file.rwx");
	my $uid  = $doc->getValue("$key_file.uid");
	my $gid  = $doc->getValue("$key_file.gid");
	my $type = $doc->getValue("$key_file.type");
	my $lmod = $doc->getValue("$key_file.lmod");

	my @f = ($name, $size, $rwx, $uid, $gid, $type, $lmod);

	push( @files, \@f );
    }

    $self->{FILES} = \@files;
}


#####################################################################
# Serverd Remote Request Support Functions                          #
# XXX These functions should be split into a remote module of its   #
#     own among other things.                                       #
#     Other copies of these functions exist at                      #
#     SSHostRemote.pm, RawDiskRemote.pm, MFSFileSystemRemote.pm     #
#     Index.pm                                                      #
#####################################################################

# The DOMAccess object returned must be explicitly freed by the caller
sub makeServerdRequest($) {
    my($op) = @_;
    my $doc = VMware::DOMAccess->new('exec');
    $doc->setValue('.op', $op);
    return($doc);
}

# Sends a serverd request
sub sendServerdRequest($) {
   my($self) = shift;
   my($in) = @_;
   my $server = $self->getRemoteServer();

   return VMware::HConfig::SSHostRemote::doServerdRequestWithServer($server, $in, undef);
}

# Gets the return code of the response from serverd
sub getReturnCode($) {
    my($out) = @_;
    my $rc = $out->getValue('exec.returncode');
    if( $rc eq '' ) {
	Warning("Null return code received");
	return(0);
    }
    return($rc);
}

# Propagates warnings sent by vmserverd
sub ServerdWarning($) {
    my($out) = @_;
    my $key;

    my @errors = $out->listElementNames('.messages');
    foreach $key (@errors) {
	Warning($out->getValue('.messages.' . $key));
    }
}

#####################################################################
# End of Serverd Remote Request Support Functions                   #
#####################################################################

1;
