#!/usr/bin/perl -w

#############################################################
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
#############################################################
#
# PCIDevice.pm
#
# A basic class that contains information representing a PCI device.
# Information includes bus #, device/slot #, function #, name, irq,
# owner.

package VMware::HConfig::PCIDevice;
use strict;
use VMware::ExtHelpers qw(&Warning &Panic &Debug);

sub HOST { 0 };
sub VM { 1 };
sub SHARED { 2 };

my @VALID_OWNER = qw(HOST VM SHARED);

# Device classes and subclasses as defined in linux/pci.h
my $cPCI_CLASS_NOT_DEFINED =            0x000000;
my $cPCI_CLASS_NOT_DEFINED_VGA = 	0x000100;

my $cPCI_BASE_CLASS_STORAGE =		0x010000;
my $cPCI_CLASS_STORAGE_SCSI =		0x010000;
my $cPCI_CLASS_STORAGE_IDE =		0x010100;
my $cPCI_CLASS_STORAGE_FLOPPY =	        0x010200;
my $cPCI_CLASS_STORAGE_IPI =		0x010300;
my $cPCI_CLASS_STORAGE_RAID =		0x010400;
my $cPCI_CLASS_STORAGE_OTHER =		0x018000;

my $cPCI_BASE_CLASS_NETWORK =		0x020000;
my $cPCI_CLASS_NETWORK_ETHERNET =	0x020000;
my $cPCI_CLASS_NETWORK_TOKEN_RING =	0x020100;
my $cPCI_CLASS_NETWORK_FDDI =		0x020200;
my $cPCI_CLASS_NETWORK_ATM =		0x020300;
my $cPCI_CLASS_NETWORK_OTHER =		0x028000;

my $cPCI_BASE_CLASS_DISPLAY =		0x030000;
my $cPCI_CLASS_DISPLAY_VGA =		0x030000;
my $cPCI_CLASS_DISPLAY_XGA =		0x030100;
my $cPCI_CLASS_DISPLAY_OTHER =		0x038000;

my $cPCI_BASE_CLASS_MULTIMEDIA =	0x040000;
my $cPCI_CLASS_MULTIMEDIA_VIDEO =	0x040000;
my $cPCI_CLASS_MULTIMEDIA_AUDIO =	0x040100;
my $cPCI_CLASS_MULTIMEDIA_OTHER =	0x048000;

my $cPCI_BASE_CLASS_MEMORY =		0x050000;
my $cPCI_CLASS_MEMORY_RAM =		0x050000;
my $cPCI_CLASS_MEMORY_FLASH =		0x050100;
my $cPCI_CLASS_MEMORY_OTHER =		0x058000;

my $cPCI_BASE_CLASS_BRIDGE =		0x060000;
my $cPCI_CLASS_BRIDGE_HOST =		0x060000;
my $cPCI_CLASS_BRIDGE_ISA =		0x060100;
my $cPCI_CLASS_BRIDGE_EISA =		0x060200;
my $cPCI_CLASS_BRIDGE_MC =		0x060300;
my $cPCI_CLASS_BRIDGE_PCI =		0x060400;
my $cPCI_CLASS_BRIDGE_PCMCIA =	        0x060500;
my $cPCI_CLASS_BRIDGE_NUBUS =		0x060600;
my $cPCI_CLASS_BRIDGE_CARDBUS =	        0x060700;
my $cPCI_CLASS_BRIDGE_OTHER =		0x068000;

my $cPCI_BASE_CLASS_COMMUNICATION =	0x070000;
my $cPCI_CLASS_COMMUNICATION_SERIAL =	0x070000;
my $cPCI_CLASS_COMMUNICATION_PARALLEL = 0x070100;
my $cPCI_CLASS_COMMUNICATION_OTHER =	0x078000;

my $cPCI_BASE_CLASS_SYSTEM =		0x080000;
my $cPCI_CLASS_SYSTEM_PIC =		0x080000;
my $cPCI_CLASS_SYSTEM_DMA =		0x080100;
my $cPCI_CLASS_SYSTEM_TIMER =		0x080200;
my $cPCI_CLASS_SYSTEM_RTC =		0x080300;
my $cPCI_CLASS_SYSTEM_OTHER =		0x088000;

my $cPCI_BASE_CLASS_INPUT =		0x090000;
my $cPCI_CLASS_INPUT_KEYBOARD =	        0x090000;
my $cPCI_CLASS_INPUT_PEN =		0x090100;
my $cPCI_CLASS_INPUT_MOUSE =		0x090200;
my $cPCI_CLASS_INPUT_OTHER =		0x098000;

my $cPCI_BASE_CLASS_DOCKING =		0x0a0000;
my $cPCI_CLASS_DOCKING_GENERIC =	0x0a0000;
my $cPCI_CLASS_DOCKING_OTHER =		0x0a0100;

my $cPCI_BASE_CLASS_PROCESSOR =	        0x0b0000;
my $cPCI_CLASS_PROCESSOR_386 =		0x0b0000;
my $cPCI_CLASS_PROCESSOR_486 =		0x0b0100;
my $cPCI_CLASS_PROCESSOR_PENTIUM =	0x0b0200;
my $cPCI_CLASS_PROCESSOR_ALPHA =	0x0b1000;
my $cPCI_CLASS_PROCESSOR_POWERPC =	0x0b2000;
my $cPCI_CLASS_PROCESSOR_CO =		0x0b4000;

my $cPCI_BASE_CLASS_SERIAL =		0x0c0000;
my $cPCI_CLASS_SERIAL_FIREWIRE =	0x0c0000;
my $cPCI_CLASS_SERIAL_ACCESS =		0x0c0100;
my $cPCI_CLASS_SERIAL_SSA =		0x0c0200;
my $cPCI_CLASS_SERIAL_USB =		0x0c0300;
my $cPCI_CLASS_SERIAL_FIBER =		0x0c0400;
my $cPCI_CLASS_SERIAL_SMBUS =           0x0c0500;

my $cPCI_BASE_CLASS_INTELLIGENT =       0x0e0000;
my $cPCI_CLASS_INTELLIGENT_I2O  =       0x0e0000;

my $cPCI_CLASS_HOT_SWAP_CONTROLLER =	0xff0000;

my $cPCI_CLASS_OTHERS =		        0xff0000;

my %gClassName = (
    $cPCI_CLASS_NOT_DEFINED => 'Undefined',
    $cPCI_CLASS_NOT_DEFINED_VGA => 'VGA controller',

    $cPCI_BASE_CLASS_STORAGE => 'Storage controller',
    $cPCI_CLASS_STORAGE_SCSI => 'SCSI storage controller',
    $cPCI_CLASS_STORAGE_IDE => 'IDE storage controller',
    $cPCI_CLASS_STORAGE_FLOPPY => 'Floppy storage controller',
    $cPCI_CLASS_STORAGE_IPI => 'IPI storage controller',
    $cPCI_CLASS_STORAGE_RAID => 'RAID storage controller',
    $cPCI_CLASS_STORAGE_OTHER => 'Other storage controller',

    $cPCI_BASE_CLASS_NETWORK => 'Network controller',
    $cPCI_CLASS_NETWORK_ETHERNET => 'Ethernet controller',
    $cPCI_CLASS_NETWORK_TOKEN_RING => 'Token Ring controller',
    $cPCI_CLASS_NETWORK_FDDI => 'FDDI controller',
    $cPCI_CLASS_NETWORK_ATM => 'ATM controller',
    $cPCI_CLASS_NETWORK_OTHER => 'Other network controller',

    $cPCI_BASE_CLASS_DISPLAY => 'Display controller',
    $cPCI_CLASS_DISPLAY_VGA => 'VGA controller',
    $cPCI_CLASS_DISPLAY_XGA => 'XGA controller',
    $cPCI_CLASS_DISPLAY_OTHER => 'Other display controller',

    $cPCI_BASE_CLASS_MULTIMEDIA => 'Multimedia controller',
    $cPCI_CLASS_MULTIMEDIA_VIDEO => 'Video controller',
    $cPCI_CLASS_MULTIMEDIA_AUDIO => 'Audio controller',
    $cPCI_CLASS_MULTIMEDIA_OTHER => 'Other multimedia controller',

    $cPCI_BASE_CLASS_MEMORY => 'Memory',
    $cPCI_CLASS_MEMORY_RAM => 'RAM memory',
    $cPCI_CLASS_MEMORY_FLASH => 'Flash memory',
    $cPCI_CLASS_MEMORY_OTHER => 'Other memory',

    $cPCI_BASE_CLASS_BRIDGE => 'Bridge',
    $cPCI_CLASS_BRIDGE_HOST => 'Host Bridge',
    $cPCI_CLASS_BRIDGE_ISA => 'ISA Bridge',
    $cPCI_CLASS_BRIDGE_EISA => 'EISA Bridge',
    $cPCI_CLASS_BRIDGE_MC => 'MC Bridge',
    $cPCI_CLASS_BRIDGE_PCI => 'PCI Bridge',
    $cPCI_CLASS_BRIDGE_PCMCIA => 'PCMCIA Bridge',
    $cPCI_CLASS_BRIDGE_NUBUS => 'NUBus Bridge',
    $cPCI_CLASS_BRIDGE_CARDBUS => 'CardBus Bridge',
    $cPCI_CLASS_BRIDGE_OTHER => 'Other bridge',

    $cPCI_BASE_CLASS_COMMUNICATION => 'Commomuncation controller',
    $cPCI_CLASS_COMMUNICATION_SERIAL => 'Serial port controller',
    $cPCI_CLASS_COMMUNICATION_PARALLEL => 'Parallel port controller',
    $cPCI_CLASS_COMMUNICATION_OTHER => 'Other communication controller',

    $cPCI_BASE_CLASS_SYSTEM => 'System controller',
    $cPCI_CLASS_SYSTEM_PIC => 'PIC controller',
    $cPCI_CLASS_SYSTEM_DMA => 'DMA controller',
    $cPCI_CLASS_SYSTEM_TIMER => 'Timer controller',
    $cPCI_CLASS_SYSTEM_RTC => 'RTC controller',
    $cPCI_CLASS_SYSTEM_OTHER => 'Other system controller',

    $cPCI_BASE_CLASS_INPUT => 'Input controller',
    $cPCI_CLASS_INPUT_KEYBOARD => 'Keyboard controller',
    $cPCI_CLASS_INPUT_PEN => 'Pen input controller',
    $cPCI_CLASS_INPUT_MOUSE => 'Mouse controller',
    $cPCI_CLASS_INPUT_OTHER => 'Other input controller',

    $cPCI_BASE_CLASS_DOCKING => 'Docking controller',
    $cPCI_CLASS_DOCKING_GENERIC => 'Generic docking controller',
    $cPCI_CLASS_DOCKING_OTHER => 'Docking controller',

    $cPCI_BASE_CLASS_PROCESSOR => 'Processor',
    $cPCI_CLASS_PROCESSOR_386 => '386 processor',
    $cPCI_CLASS_PROCESSOR_486 => '486 processor',
    $cPCI_CLASS_PROCESSOR_PENTIUM => 'Pentium processor',
    $cPCI_CLASS_PROCESSOR_ALPHA => 'Alpha processor',
    $cPCI_CLASS_PROCESSOR_POWERPC => 'PowerPC processor',
    $cPCI_CLASS_PROCESSOR_CO => 'Processor controller',

    $cPCI_BASE_CLASS_SERIAL => 'Serial controller',
    $cPCI_CLASS_SERIAL_FIREWIRE => 'Firewire controller',
    $cPCI_CLASS_SERIAL_ACCESS => 'Access controller',
    $cPCI_CLASS_SERIAL_SSA => 'SSA controller',
    $cPCI_CLASS_SERIAL_USB => 'USB controller',
    $cPCI_CLASS_SERIAL_FIBER => 'Fiber storage controller',
    $cPCI_CLASS_SERIAL_SMBUS => 'SMBus controller',

    $cPCI_BASE_CLASS_INTELLIGENT => 'Intelligent controller',
    $cPCI_CLASS_INTELLIGENT_I2O  => 'Intelligent Input/Output',

    $cPCI_CLASS_HOT_SWAP_CONTROLLER => 'Hot swap controller',

    $cPCI_CLASS_OTHERS => 'Other PCI controller'
		  );

my %gSerialFiberDrivers = ( "lpfcdd.o"      => 1,
                            "lpfcdd_122.o"  => 1,
                            "qla2200_603.o" => 1,
                            "qla2200_604.o" => 1,
                            "qla2300_603.o" => 1,
                            "qla2300_604.o" => 1 );

# List of drivers supporting I2O (Intelligent I/O) storage devices.
my %gI2ODrivers = ( "megaraid.o"    => 1,
                    "ips.o"         => 1,
                    "dpt_i2o.o"     => 1 );

#####################################################################
# Public Static Methods (indep. of each object)                     #
#####################################################################

# Creates a hash string for each PCI device.
#
# Required arguments are bus #, device #, function #
# Returns a hash string
sub hash($$$) {
    my ($bus, $device, $func) = @_;
    if( !defined($bus) || !defined($device) || !defined($func) ) {
	Panic("Null hash value to PCIDevice::hash()");
    }
    return("$bus:$device:$func");
}

sub hashToBusDevFunc($) {
    my($hash) = @_;
    if( $hash =~ /^(\d+):(\d+):(\d+)$/ ) {
	my($bus, $device, $func) = ($1, $2, $3);
	return($bus, $device, $func);
    }
    return(undef, undef, undef);
}

#####################################################################
# Public Methods                                                    #
#####################################################################

# Constructor Methods
# Make a copy of this device
sub copy() {
    my $self = shift;

    my $new_obj = __PACKAGE__->new($self->getBus(), $self->getDevice(), $self->getFunction());
    $new_obj->setName($self->getName());
    $new_obj->setIrq($self->getIrq());
    $new_obj->setOwner($self->getOwner());
    $new_obj->setOptions($self->getOptions());
    $new_obj->setVendorID($self->getVendorID());
    $new_obj->setDeviceID($self->getDeviceID());
    $new_obj->setClass($self->getClass());
    $new_obj->setSubsysVendorID($self->getSubsysVendorID());
    $new_obj->setSubsysDeviceID($self->getSubsysDeviceID());
    $new_obj->setDrivers($self->getDrivers());
    $new_obj->setDriverNames($self->getDriverNames());
    return($new_obj);
}

# Accessor functions
sub getBus() {
    my $self = shift;
    return( $self->{BUS} );
}

sub getDevice() {
    my $self = shift;
    return( $self->{DEVICE} );
}

sub getFunction() {
    my $self = shift;
    return( $self->{FUNCTION} );
}

sub getName() {
    my $self = shift;
    return( $self->{NAME} );
}

# Returns the class of the device
sub getClass() {
    my $self = shift;
    return( $self->{CLASS} );
}

sub getClassName() {
    my $self = shift;
    if( defined($gClassName{$self->{CLASS}}) ) {
	return( $gClassName{$self->{CLASS}} );
    } else {
	return( "Unknown class" );
    }
}

sub getDrivers() {
    my $self = shift;
    return( $self->{DRIVERS} );
}

sub getDriverNames() {
    my $self = shift;
    return( $self->{DRIVER_NAMES} );
}

# XXX This should eventually replace getDriverNames(). 
# what's in the DRIVER_NAMES field is now a device "equivalence",
# used strictly for device naming purposes.  Also, even if
# more than one driver module is listed, they all map to one
# single equivalence, which is related more to device class.
sub getEquiv() {
    my $self = shift;
    my $driverNames = $self->getDriverNames();

    if (defined($driverNames)) {
        return $driverNames->[0];
    } else {
        return "";
    }
}

# Returns the hash that uniquely identifies this device
sub getHash() {
    my $self = shift;
    return( hash($self->getBus(), $self->getDevice(), $self->getFunction()) );
}

# Gets the IRQ for the device if it has one.
#
# Results:
#     Returns the IRQ number if it has one.  Otherwise should return the
#     undefined value.
sub getIrq() {
    my $self = shift;
    return( $self->{IRQ} );
}

sub getOwner() {
    my $self = shift;
    return( $self->{OWNER} );
}

sub getOwnerString() {
    my $self = shift;
    my $owner = $self->getOwner();

    if( $owner == HOST ) { return("host"); }
    elsif( $owner == VM ) { return("VM"); }
    elsif( $owner == SHARED ) { return("shared"); }

    return("unknown");
}

sub getOptions() {
    my $self = shift;
    return( $self->{OPTIONS} );
}

sub getVendorID() {
    my $self = shift;
    return( $self->{VENDOR} );
}

sub getDeviceID() {
    my $self = shift;
    return( $self->{DEVID} );
}

sub isNetClassDevice() {
    my $self = shift;
    my $devClass = ($self->getClass() & 0x00ff0000);  # Get the high 8 bits

    if( $devClass == $cPCI_BASE_CLASS_NETWORK ) {
	return(1);
    } else {
	return(0);
    }
}

sub isNetEthernetDevice() {
    my $self = shift;
    my $devSubclass = ($self->getClass() & 0x00ffff00);  # Get the high 16 bits

    if( $devSubclass == $cPCI_CLASS_NETWORK_ETHERNET ) {
	return(1);
    } else {
	return(0);
    }
}

sub isStorageClassDevice() {
    my $self = shift;
    my $devClass = ($self->getClass() & 0x00ff0000);  # Get the high 8 bits

    if( $devClass == $cPCI_BASE_CLASS_STORAGE ) {
	return(1);
    } else {
        return $self->isI2OStorageDevice();
    }
}

sub isI2OStorageDevice() {
    my $self = shift;
    my $devSubclass = ($self->getClass() & 0x00ffff00);  # Get the high 16 bits

    if( $devSubclass == $cPCI_CLASS_INTELLIGENT_I2O ) {
        for (@{$self->getDrivers()}) {
            if ($gI2ODrivers{$_}) {
                return 1;
            }
        }
    }

    return 0;
}

sub isStorageSCSIDevice() {
    my $self = shift;
    my $devSubclass = ($self->getClass() & 0x00ffff00);  # Get the high 16 bits

    if( $devSubclass == $cPCI_CLASS_STORAGE_SCSI ) {
	return(1);
    } else {
	return(0);
    }
}

sub isStorageRAIDDevice() {
    my $self = shift;
    my $devSubclass = ($self->getClass() & 0x00ffff00);  # Get the high 16 bits

    if( $devSubclass == $cPCI_CLASS_STORAGE_RAID ) {
	return(1);
    } else {
	return(0);
    }
}

sub isSerialFiberDevice() {
   my $self = shift;
   my $devSubclass = ($self->getClass() & 0x00ffff00);  # Get the high 16 bits

   if ($devSubclass == $cPCI_CLASS_SERIAL_FIBER) {
      return 1;
   } else {
      for (@{$self->getDrivers()}) {
         if ($gSerialFiberDrivers{$_}) {
            return 1;
         }
      }
   }

   return 0;
}

sub getSubsysVendorID($) {
    my $self = shift;
    return($self->{SUBSYS_VENDOR});
}

sub getSubsysDeviceID($) {
    my $self = shift;
    return($self->{SUBSYS_DEVID});
}

# Returns whether these PCI Devices are equal
sub equals($) {
    my $self = shift;
    my ($dev) = @_;

    return( $self->getHash() eq $dev->getHash() &&
	    $self->getDeviceID() == $dev->getDeviceID() &&
	    $self->getVendorID() == $dev->getVendorID() &&
	    $self->getSubsysVendorID == $dev->getSubsysVendorID() &&
	    $self->getSubsysDeviceID == $dev->getSubsysDeviceID() &&
	    $self->getClass() == $dev->getClass() );
}

# Returns whether we can manage this device
sub isVMwareManageable() {
    my $self = shift;

    my @drivers = @{ $self->getDrivers() };
    return( $#drivers > -1 );
}

# Returns whether this device can be shared and managed by us
sub isVMwareSharable($) {
    my ($self) = shift;

    return( $self->isVMwareManageable() && !$self->isNetEthernetDevice() );
}


# Mutator functions
sub setName($) {
    my $self = shift;
    $self->{NAME} = shift;
}

# If unsetting the IRQ, should pass in the undef value
sub setIrq($) {
    my $self = shift;
    $self->{IRQ} = shift;
}

sub setOwner($) {
    my $self = shift;
    my $owner = shift;

    # Owner must be one of the possible valid owners
    if( $owner < 0 || $owner > $#VALID_OWNER ) {
	return;
    }

    if( $owner == HOST ) {
	Debug(2, "Setting owner of device " . $self->getHash() . " to HOST.");
    } elsif( $owner == VM ) {
	Debug(2, "Setting owner of device " . $self->getHash() . " to VM.");
    } else {
	Debug(2, "Setting owner of device " . $self->getHash() . " to SHARED.");
    }
    $self->{OWNER} = $owner;
}

# Set the owner using the string name.
sub setOwnerString($) {
    my $self = shift;
    my ($ownerString) = @_;

    if( $ownerString eq "host" ) {
	$self->setOwner(HOST);
    } elsif( $ownerString eq "VM" ) {
	$self->setOwner(VM);
    } elsif( $ownerString eq "shared" ) {
	$self->setOwner(SHARED);
    } else {
	Warning('Invalid owner string "' . $ownerString . '" specified.');
    }
}

sub setOptions($) {
    my $self = shift;
    $self->{OPTIONS} = shift;
}

sub setVendorID($) {
    my $self = shift;
    $self->{VENDOR} = shift;
}

sub setDeviceID($) {
    my $self = shift;
    $self->{DEVID} = shift;
}

sub setClass($) {
    my $self = shift;
    $self->{CLASS} = shift;
}

sub setDrivers($) {
    my $self = shift;
    $self->{DRIVERS} = shift;
}

sub setDriverNames($) {
    my $self = shift;
    $self->{DRIVER_NAMES} = shift;
}

sub setSubsysVendorID($) {
    my $self = shift;
    $self->{SUBSYS_VENDOR} = shift;
}

sub setSubsysDeviceID($) {
    my $self = shift;
    $self->{SUBSYS_DEVID} = shift;
}

# Returns device information in a string form
sub toString() {
    my $self = shift;

    my $vendor = sprintf("%04x", $self->getVendorID());
    my $devid = sprintf("%04x", $self->getDeviceID());
    my $name = $self->getName();
    my $class = sprintf("%06x", $self->getClass());
    my $subsys_vendor = sprintf("%04x", $self->getSubsysVendorID());
    my $subsys_devid = sprintf("%04x", $self->getSubsysDeviceID());

    my $s =
	("Bus " . $self->getBus() . "," .
	 " device " . $self->getDevice() . "," .
	 " function " . $self->getFunction() .
	 " class " . $class .
	 "\n" .
	 "Name: " . $name . "\n" .
	 "Owner: " . $self->getOwnerString() .
	 (defined($self->getIrq()) ? (", IRQ " . $self->getIrq()) : "") . "\n" .
	"Vendor: " . $vendor .
	 " Device ID: " . $devid . "\n" .
	 (hex($subsys_vendor) != 0 ?
		  ("Subsystem: " . $subsys_vendor . ':' . $subsys_devid . "\n") :
		  "")
	 );

    my @drivers = @ { $self->getDrivers() };
    my @driverNames = @ { $self->getDriverNames() };
    if( $#drivers > -1 ) {
        $s .= "Drivers:";

	my $i;
	for($i = 0; $i <= $#drivers; $i++ ) {
	    $s .= " " . $drivers[$i] . " (" . $driverNames[$i] . ") ";
	}

	$s .= "\n";
    }

    return($s);
}


#####################################################################
# Methods not intended for use by user                              #
#####################################################################

# Constructor for a PCI device
sub new($$$) {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my ($bus, $device, $function) = @_;
    my $self = {};

    $self->{BUS} = $bus;
    $self->{DEVICE} = $device;
    $self->{FUNCTION} = $function;

    $self->{NAME} = "Unknown Device";
    $self->{IRQ} = undef;
    $self->{OWNER} = HOST;      # Defaults to host
    $self->{OPTIONS} = "";      # Driver options for device

    $self->{VENDOR} = 0;        # Vendor id of device
    $self->{DEVID} = 0;         # Device id given by vendor

    $self->{CLASS} = 0x000000;  # Class and subclass of the device
    $self->{DRIVERS} = [];      # List of possible drivers
    $self->{DRIVER_NAMES} = []; # List of names for drivers

    $self->{SUBSYS_VENDOR} = 0;  # Subsystem vendor id
    $self->{SUBSYS_DEVID} = 0;   # Subsystem device id

    # Needed to bind the data structure $self to this package so
    # Subroutines will then act as the methods for the data structure.
    bless($self);

    return($self);
}

#####################################################################
# PCIDeviceList.pm
#
# A basic class that contains a list of PCI devices.
#####################################################################
package VMware::HConfig::PCIDeviceList;
use strict;
use VMware::HConfig::PCIDevice;
use VMware::ExtHelpers qw(&Warning &Panic &Debug);
use VMware::HConfig::Constant qw(@gRequiredDrivers @gOptionalDrivers &c_getDir);
use VMware::Tokenizer();

# Constructs a new PCIDeviceList object
sub new() {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = {};
    bless($self, $class);
    return($self);
}

# Constructs a copy of the PCIDeviceList object
sub copy() {
    my $self = shift;
    my $new_obj = __PACKAGE__->new();
    my $dev;
    foreach $dev ($self->getPCIDevices()) {
	$new_obj->addPCIDevice($dev->copy());
    }
    return($new_obj);
}

# Adds a pci device to the list
sub addPCIDevice($) {
    my $self = shift;
    my $pcidev = shift;
    $self->{ $pcidev->getHash() } = $pcidev;
}

# Returns a sorted array of all the pci devices in the list.
sub getPCIDevices() {
    my($self) = shift;
    my @pci_devices = sort byBusDeviceFunc values(%$self);
    return(@pci_devices);
}

# Returns the number of PCI devices in the list
sub count() {
    my($self) = shift;
    my @count = keys(%$self);
    return($#count + 1);
}

# Filters and creates a new PCIDeviceList consisting of the PCI
# devices that match the filter.
#
# The filter takes in a PCIDevice object as an argument and returns
# true if it matches or false if it is to be left out.
sub filterPCIDevices($) {
    my($self) = shift;
    my($filter) = @_;

    my @pciDevs = $self->getPCIDevices();
    my $newPCIDeviceList = $self->new();

    my $dev;
    foreach $dev (@pciDevs) {
	if( &{ $filter }($dev) ) {
	    $newPCIDeviceList->addPCIDevice($dev);
	}
    }
    return($newPCIDeviceList);
}

# Gets the PCIDevice information based on the current running
# state of the machine.
sub getPCIDeviceByHash($) {
    my($self) = shift;
    my($hash) = @_;

    my @devs = grep( $hash eq $_->getHash(), $self->getPCIDevices());

    # There should be only one if at all
    if( $#devs != -1 ) {
	return($devs[0]);
    }
    else {
	return(undef);
    }
}

# Gets the PCIDevice information based on the current running
# state of the machine.
#
# Returns an array of devices that match the bus:device description
sub getPCIDeviceByBusDev($$) {
    my($self) = shift;
    my($bus, $dev) = @_;

    my @devs = grep( $bus eq $_->getBus() && $dev eq $_->getDevice(), $self->getPCIDevices());

    return(@devs);
}

# Gets the PCIDevice information based on the current running
# state of the machine.
#
# Returns an array of devices that match the bus:device:function description
sub getPCIDeviceByBusDevFunc($$$) {
    my($self) = shift;
    my($bus, $dev, $func) = @_;

    my @devs = grep( $bus eq $_->getBus() && 
		     $dev eq $_->getDevice() && 
		     $func eq $_->getFunction(), 
		     $self->getPCIDevices());
    
    # There should be only one if at all
    return scalar(@devs) ? $devs[0] : undef;
}

# Passed into the sort function to determine which device $a or $b is
# to come first in the list.
sub byBusDeviceFunc {
    if( ($a->getBus() <=> $b->getBus()) ) {
	return($a->getBus() <=> $b->getBus());
    }
    elsif( ($a->getDevice() <=> $b->getDevice()) ) {
	return($a->getDevice() <=> $b->getDevice());
    }
    else {
	return($a->getFunction() <=> $b->getFunction());
    }
}

# Gets just the unique PCI cards in the list
sub getUniqPCIDevice(@) {
    my $self = shift;
    my @pcidevs = $self->getPCIDevices();
    my @newdevs;

    while( $#pcidevs > -1 ) {
	my $dev = shift(@pcidevs);
        if ($dev->isVMwareSharable()) {
           # Sharable device: all functions for a given bus/device
           #                  are treated as part of the same device
           @pcidevs = grep( $dev->getDevice() ne $_->getDevice() ||
                            $dev->getBus() ne $_->getBus(), @pcidevs );
        } else {
           # Non-sharable device: each bus/device/function is a unique device
        }

	push(@newdevs, $dev);
    }

    return(@newdevs);
}

# Gets the list of matching PCI cards
sub getMatchingPCIDevice(@) {
   my $self = shift;
   my $dev = shift;

   my @pcidevs = $self->getPCIDevices();
   my @newdevs;

   # Sharable devices are allocated by bus, 
   # non-sharable devices are allocated by function
   if ($dev->isVMwareSharable()) {
      # Sharable device: all functions for a given bus/device
      #                  are treated as part of the same device
      @newdevs = grep( $dev->getDevice() eq $_->getDevice() &&
                       $dev->getBus() eq $_->getBus(), @pcidevs );
   } else {
      # Non-sharable device: each function is a unique device
      @newdevs = ($dev);
   }
   return @newdevs;
}

# Determines whether the PCI List settings are identical
sub equals($) {
    my $self = shift;
    my ($PCIDeviceList) = (@_);

    if( $PCIDeviceList->count() != $self->count() ) {
	return(0);
    }

    my @list1 = $self->getPCIDevices();
    my @list2 = $PCIDeviceList->getPCIDevices();

    my $i;
    for( $i = 0; $i <= $#list1; $i++ ) {
	if( ! $list1[$i]->equals($list2[$i]) ) {
	    return(0);
	}
    }

    return(1);
}

# Builds a human-readable string containing the PCI information
sub toString() {
    my($self) = shift;
    my $s = "";
    my $dev;
    foreach $dev ($self->getPCIDevices()) {
	$s .= $dev->toString() . "\n";
    }

    if( $s eq "" ) {
	$s = "No PCI Devices found!\n";
    }

    return($s);
}

# Make a cpci line based on the configuration of the devices in this list
# Builds a CPCI line of the form "bus0:device0[.function0],d1[.f1],...dn[.fn];bus1:...;...
# from the PCIDevices that are currently held by the object.
#
# Rules for CPCI line:
#
# 1. A blank CPCI setting means all devices go to host.
# 2. Any device that is listed must have its bus and device number
#    passed to the host even it it shares the bus and device with
#    a device allocated to the VMs.
# 3. The '-' character is added if NO device on the bus or device
#    generates interrupts.  If one device generates interrupts,
#    the '-' cannot be used.
# 4. A shared device must be known by the host and so it must be
#    included on the cpci line.
# 5. If a bus is not mentioned, the host is given all the devices
#    on that bus.
# 6. If the bus is mentioned but no devices listed, all devices
#    are given to the VMs.
# 7. If the bus and device are mentioned but no functions listed,
#    all functions are given to the host.
sub makeCPCILine {
    my($self) = shift;
    my $s = "";
    my $VM = VMware::HConfig::PCIDevice::VM;

    # Devices come sorted Bus:Device:Function
    my @devs = $self->getPCIDevices();
    my @busDevs;
    my @deviceDevs;

    while (@devs) {
        my $bus = $devs[0]->getBus();
        @busDevs = grep { $_->getBus() == $bus } @devs;
        @devs = grep { $_->getBus() != $bus } @devs;


        $s .= "$bus:";

        if ((grep { $_->getOwner() != $VM } @busDevs) == 0) {
            # "B:" -- all devices on this bus go to the VMkernel
            $s .= "";
        } elsif ((grep { $_->getOwner() == $VM } @busDevs) == 0) {
            # "B:*" -- all devices on this bus go to the host
            $s .= "*";
        } else {
            # "B:D[.F], ... ,D[.F]" -- comma-separated list of device[.function]
            my @devFuncList = ();
            while (@busDevs) {
                my $device = $busDevs[0]->getDevice();
                @deviceDevs = grep { $_->getDevice() == $device } @busDevs;
                @busDevs = grep { $_->getDevice() != $device } @busDevs;

                if ((grep { $_->getOwner() == $VM } @deviceDevs) == 0) {
                    # "B:D" -- all functions on this device go to the host
                    push @devFuncList, "$device";
                } else {
                    # "B:D.F" -- this device/function goes to the host
                    while (@deviceDevs) {
                        my $dev = shift(@deviceDevs);
                        my $function = $dev->getFunction();
                        if ($dev->getOwner() != $VM) {
                            push @devFuncList, "$device.$function";
                        }
                    }
                }
            }
            $s .= join(",", @devFuncList);
        }

        $s .= ";";
    }

    return($s);
}

# Sets the devices to a given state based on the CPCI line.
# The "inverse" function of the makeCPCILine to a limited extent.
#
# Notes:
# 1. Ignore the - character. Not necessary in this case.
# 2. If more than one device uses the same bus and device number,
#    we'll just allocate it to the host even if previously only
#    some of the devices were allocated to the host.  We have no
#    way to remember the previous setting unless we create a
#    separate host configuration file.
# 3. Algorithm: (needed to match semantics of CPCI)
#    - Set all the devices as being owned by the host.
#    - When a bus is listed, set all devices on the bus as owned by
#      the VMs.
#    - If the bus and device is listed, give it to the host.
sub fromCPCILine($) {
    my($self) = shift;
    my($cpci) = @_;
    my $rc = 1;

    Debug(1, "Entered with line '$cpci'.");

    my @devs = $self->getPCIDevices();

    # We can safely ignore any whitespace and '-' in the CPCI string
    $cpci =~ s/[\s-]//g;

    # Initialization: Devices on unlisted buses are managed by the host
    my $dev;
    foreach $dev (@devs) {
        $dev->setOwner(VMware::HConfig::PCIDevice::HOST);
    }

    # Split the CPCI line by bus
    my @busSpecs = split(";", $cpci);
    foreach my $busSpec (@busSpecs) {
        if ($busSpec eq "") {
            next;
        } elsif ($busSpec =~ /^(\w+):([\w.,*]*)$/) {
            my $bus = $1;
            my @busDevs = grep { $_->getBus() == $bus } @devs;

            # Initialization: unlisted devices on this bus are managed by the VMkernel
            foreach my $dev (@busDevs) {
                $dev->setOwner(VMware::HConfig::PCIDevice::VM);
            }

            if ($2 eq "") {
                # "B:" -- all devices on this bus are managed by the VMkernel
            } elsif ($2 eq "*") {
                # "B:*" -- all devices on this bus are managed by the host
                foreach my $dev (@busDevs) {
                    $dev->setOwner(VMware::HConfig::PCIDevice::HOST);
                }
            } else {
                # "B:D[.F], ... ,D[.F]" -- comma-separated list of device[.function]
                my @devSpecs = split(",", $2);
                foreach my $devSpec (@devSpecs) {
                    my @deviceDevs;

                    if ($devSpec eq "") {
                        # "" -- unnecessary comma, ignore
                        @deviceDevs = ();
                    } elsif ($devSpec =~ /^(\w+)$/) {
                        # "D" -- match device with any function
                        my $device = $1;
                        @deviceDevs = grep { $_->getDevice() == $device } @busDevs;
                    } elsif ($devSpec =~ /^(\w+)\.(\w+)$/) {
                        # "D.F" -- match device and function
                        my $device = $1;
                        my $function = $2;
                        @deviceDevs = grep { $_->getDevice() == $device && 
                                             $_->getFunction() == $function } @busDevs;
                    } else {
                        Warning("Invalid CPCI entry '$devSpec' for bus '$bus'.");
                        $rc = 0;
                    }

                    # Assign all matching devices to the console
                    foreach my $dev (@deviceDevs) {
                        $dev->setOwner(VMware::HConfig::PCIDevice::HOST);
                    }
                }
            }
        } else {
            Warning("Invalid CPCI entry '$busSpec'.");
            $rc = 0;
        }
    }

    return $rc;
}

# Builds an array of strings that lists the Console OS drivers that should be
# loaded, specifically to support shared devices
sub buildVmnixDriverList() {
   my $self = shift;
   my $sharedFunc = sub { return $_[0]->getOwner() == VMware::HConfig::PCIDevice::SHARED
                             && defined($_[0]->getDrivers()->[0]) };
   my $pcidevices = $self->filterPCIDevices($sharedFunc);
   my @devs = $pcidevices->getPCIDevices();
   my @modules = map { $_->getDrivers()->[0] } @devs;

   # Remove duplicates and strip off .o at end of the name because
   # vmkernel drivers have a .o suffix, but Linux drivers do not so we
   # have to filter out the suffix.
   #
   # XXX In the past, it has not been good to assume that the Linux drivers
   # share the same basename as the vmkernel drivers, but it is true at the
   # time of the writing of this code. -jhu
   my @vmnixModules;
   foreach my $m (@modules) {
      $m =~ s/\.o$//;
      if (!grep($_ eq $m, @vmnixModules)) {
         push(@vmnixModules, $m);
      }
   }
   return(@vmnixModules);
}

# Builds the list of drivers that are used by this list of PCI devices
sub buildDriverList() {
    my $self = shift;
    my $driverList = VMware::HConfig::PCIDriverList->new();
    my $dev;

    # First add the required drivers
    my $driver;
    foreach $driver (@gRequiredDrivers) {
	my $module = $driver->{MODULE};
	my $type = $driver->{TYPE};
	$driverList->addDriver($module, $type);
    }

    # Second add the optional drivers subject to certain tests.  For now,
    # the only test is if the driver exists.
    foreach $driver (@gOptionalDrivers) {
	my $module = $driver->{MODULE};
	my $type = $driver->{TYPE};
	if( c_getDir('vmkmodule') ) {
	    if( -f c_getDir('vmkmodule') . '/' . $module ) {
		$driverList->addDriver($module, $type);
	    }
	}
    }

    # Only get one instance per bus/device.  We don't have function
    # granularity in our allocation.
    my @devs = $self->getPCIDevices();

    # Get only devices owned by the VMkernel and have a driver and name to load
    my @vmDevs = grep { $_->getOwner() == VMware::HConfig::PCIDevice::VM &&
			defined( $_->getDrivers()->[0]) &&
			defined( $_->getDriverNames()->[0]) } @devs;

    my @sharedDevs = grep { $_->getOwner() == VMware::HConfig::PCIDevice::SHARED &&
			    defined( $_->getDrivers()->[0] ) &&
			    defined( $_->getDriverNames()->[0] ) } @devs;

    # Go through the vm drivers
    foreach $dev (@vmDevs) {
	my $module = $dev->getDrivers()->[0];
	my $type = $dev->getDriverNames()->[0];
	$driverList->addDriver($module, $type);
    }

    # Go through the shared drivers
    foreach $dev (@sharedDevs) {
	my $module = $dev->getDrivers()->[0];
	my $type = $dev->getDriverNames()->[0];
	my $sharedStr = $dev->getBus() . ':' . $dev->getDevice();
	$driverList->addDriver($module, $type);
	$driverList->addDriverSharedInfo($module, $sharedStr);
    }

    # Set driver options for VM and shared devices.
    # List is stored in order of increasing bus/dev/func, which is required
    # to ensure options are passed to the module loader in the correct order.
    foreach $dev (@devs) {
        next if ($dev->getOwner() == VMware::HConfig::PCIDevice::HOST);
        next if (!defined($dev->getDrivers()->[0]));
        next if (!defined($dev->getDriverNames()->[0]));
        # XXX rbosch: Currently assumes that the first listed driver
        # will load the device; if there are multiple drivers, need to
        # check which one loads first and pass the options to that one.
	my $module = $dev->getDrivers()->[0];
        my $options = $dev->getOptions();
        $driverList->addDriverOptions($module, $options);
    }

    return($driverList);
}


#####################################################################
# PCIDriverList.pm
#
# A basic class that contains information about drivers to be loaded.
#####################################################################

package VMware::HConfig::PCIDriverList;
use strict;
use VMware::ExtHelpers qw(Warning Panic);

# Constructs a new PCIDriverList object.  A PCIDriverList object
# is an array of hashes.  Each hash contains information about a
# modules to be loaded.  The keys to each has are
#
# MODULE: the name of the module.  ie. aic7xxx.o
# TYPE: the type of the module. ie. scsi, eth
# SHARED: reference to an array of extra parameters used for sharing
sub new() {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my $self = [];
    bless($self, $class);
    return($self);
}

# Gets all the driver modules that are in the list
sub getDriverModules() {
    my $self = shift;
    my @modules = map( $_->{MODULE}, @$self );
    return(@modules);
}

# This function should not be called externally
sub getDriverInternal($) {
    my $self = shift;
    my ($module) = @_;
    my @driver = grep( $_->{MODULE} eq $module, @$self );
    if( $#driver == 0 ) {
	return($driver[0]);
    } elsif( $#driver > 0 ) {
	Warning("Duplicate driver found in the driver list.  Rep invariant violated.");
	return($driver[0]);
    }
    return(undef);
}


# Gets the type of the driver given the modules name
sub getDriverType($) {
    my $self = shift;
    my ($module) = @_;
    my $driver = $self->getDriverInternal($module);
    if( defined($driver) ) {
	return($driver->{TYPE});
    }
    return('');
}

# Gets an array corresponding to the shared arguments
# for the driver
sub getDriverSharedInfo($) {
    my $self = shift;
    my ($module) = @_;
    my $driver = $self->getDriverInternal($module);
    if( defined($driver) ) {
	return(@{ $driver->{SHARED} });
    }
    my @array;
    return(@array);
}

# Gets an option string for configuring the devices
# using this driver, suitable for appending to the
# vmkload_mod command
sub getDriverOptions($) {
    my $self = shift;
    my ($module) = @_;
    my $driver = $self->getDriverInternal($module);
    my @optionsList = ();
    if( defined($driver) ) {
        foreach my $name (keys %{$driver->{OPTIONS}}) {
            next if ($name eq '');
            my $options = join(',', @{$driver->{OPTIONS}->{$name}});
            push @optionsList, "$name=$options";
        }
    }
    return join(' ', @optionsList);
}

# Adds a driver and its name to the list.  Only adds
# a driver if it is unique.
sub addDriver($$) {
    my $self = shift;
    my ($module, $type) = @_;
    my $driver = $self->getDriverInternal($module);
    if( !defined($driver) ) {
        my $options = { '' => [] };
	my $driver = { TYPE => $type,
		       MODULE => $module,
		       SHARED => [],
                       OPTIONS => $options };
	push( @$self, $driver);
    }
}

# Adds information regarding the sharing of this driver.
# This info is really just an argument to be supplied to
# the loader command
sub addDriverSharedInfo($$) {
    my $self = shift;
    my ($module, $shared) = @_;
    my $driver = $self->getDriverInternal($module);
    if( defined($driver) ) {
	# Check to see if the infomration is here already
	if( !grep($shared eq $_, @{ $driver->{SHARED} }) ) {
	    push( @{ $driver->{SHARED} }, $shared );
	}
    }
}

# Adds driver configuration options for a particular device.
# For each option name, store a list of settings for the devices.
sub addDriverOptions($$) {
    my $self = shift;
    my ($module, $options) = @_;
    my $driver = $self->getDriverInternal($module);
    if( defined($driver) ) {
        # Device options are specified as "key1=value1 key2=value2..."
        # Convert into option list for all devices using this driver
        # Store as a hash from option name to list of values
        foreach my $option (split(' ', $options)) {
            if ($option =~ /^(\S+)=(\S+)$/) {
                if (!defined($driver->{OPTIONS}->{$1})) {
                    # First time we're seeing this option with this driver;
                    # prepend blank list entries for preceding devices
                    @{$driver->{OPTIONS}->{$1}} = @{$driver->{OPTIONS}->{''}};
                }
                push @{$driver->{OPTIONS}->{$1}}, $2;
            }
        }
        push @{$driver->{OPTIONS}->{''}}, '';
    }
}

# Generates a string representation
sub toString() {
    my $self = shift;
    return( $self->toVmkLoadmodString() );
}


# Generates a string format that is accepted by vmkload mod
sub toVmkLoadmodString() {
    my $self = shift;
    my $module;
    my $s  = '';
    foreach $module ($self->getDriverModules()) {
	my $type = $self->getDriverType($module);
	my @sharedInfo = $self->getDriverSharedInfo($module);

	$s .= $module . ' ' . $type;

        if (@sharedInfo) {
           $s .= ' -d ' . join(',', @sharedInfo);
        }

        my $options = $self->getDriverOptions($module);
        if ($options ne '') {
            $s .= " $options";
        }

	$s .= "\n";
    }
    return($s);
}

1;
