#!/usr/bin/perl -w

#############################################################
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
#############################################################
#
# RawDiskRemote.pm
# 
# A class that is used to gather and manipulate information about the
# raw disks on the computer.
# 
package VMware::HConfig::RawDiskRemote;
use strict;
use VMware::HConfig::Constant qw(&c_getFile &c_getDir &c_getHelper $gVmkDumpPartType
 				 &CACHE_NEVER &CACHE_ONCE &CACHE_CALLBACK $gRawDisk_CacheDefaults );
use VMware::ExtHelpers qw(&Warning &System &internal_basename &Debug &Panic $gDEBUG &shell_string);
use VMware::HConfig::RawDisk;
use VMware::HConfig::Serialize;
use VMware::HConfig::SSHostRemote;

use vars       qw($VERSION @ISA);

BEGIN {
    # set the version for version checking
    $VERSION     = 1.00;
    @ISA         = qw(VMware::HConfig::RawDisk);
}

use vars qw();

#####################################################################
# Public Static Methods (indep. of each object)                     #
#####################################################################

sub new($$) {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my($vmklogicName, $server) = @_;
    my $self = $proto->create($vmklogicName);

    $self->{CACHE_OPTIONS} = $gRawDisk_CacheDefaults;

    $self->{REMOTE_SERVER} = $server;

    # Do not keep a back reference to the SSHost object or else we'll
    # have a circular reference and have to explicitly garbage collect.

    bless($self, $class);

    return($self);
}

# Inherited from super class
# sub create($) {

# Not implemented for this class
# sub lookupPartType($) {
# sub isPartExtended($) {
# sub isPartVmware($) {
# sub isPartVmkDump($) {
# sub printMap() {
# sub isDevName($) {
# sub isValidGeometry($$$) {
# sub isPhysicalPart($) {
# sub isLogicalPart($) {
# sub fromChsToLba($$$$$$) {
# sub fromLbaToChs($$$$) {
# sub isLogVmnixDiskName($) {
# sub isLogVmkDiskName($) {
# sub makeLogVmkDiskName($$) {
# sub unmakeLogVmkDiskName($) {
# sub isLogVmkPartName($) {
# sub makeLogVmkPartName($$) {
# sub unmakeLogVmkPartName($) {

######################################################################
## Public Methods                                                    #
######################################################################

sub getRemoteData() {
    my($self) = shift;
    if( defined($self->{DOM}) ) {
	my $doc = $self->{DOM}->getSubTree(".");
	if( !defined($doc) ) {
	    Panic("Did not get a DOMAccess node");
	}
	return($doc);
    } else {
	Panic("No remote data available when getRemoteData was called.");
    }
}

sub setRemoteData($) {
    my($self) = shift;
    my($doc) = @_;
    $self->{DOM} = $doc;
}

sub dumpRemoteData() {
    my($self) = shift;
    print $self->{DOM}->prettyPrint() . "\n";
}

sub getRemoteServer() {
   my($self) = shift;
   return $self->{REMOTE_SERVER};
}

# Accessor functions

# Inherited from super class
# sub getVmkName() {
# sub getVmnixDevName() {
# sub getVmnixPartSeparator() {


# Inherited from super class
# sub getVmkAdapterName() {
# sub getVmnixAdapterName() {
# sub getTarget() {
# sub getLun() {
# sub getGeometry() {
# sub getGeoCylinder() {
# sub getGeoHead() {
# sub getGeoSector() {
# sub getTotalSectors() {
# sub getPartitionTable() {
# sub getChsToLba($$$) {
# sub getLbaToChs($) {
# sub getKBytesToCyl($) {
# sub getCylToKBytes($) {
# sub hasVmnixDevName() {
# sub getPartNums() {
# sub getPart($) {
# sub getPartBestDevName($) {
# sub getPartStartCyl($) {
# sub getPartEndCyl($) {
# sub getPartType($) {
# sub getPartSectors($) {
# sub getPartTypeName($) {


# Mutator functions

# Inherited from super class
#sub setVmnixDevName($) {
#sub setVmnixPartSeparator($) {
#sub createPartitionTable() {

# Loads the partition table
#sub loadPartitionTable() {
#    my($self) = shift;
#    my $partTable = $self->createPartitionTable();
#    if( defined($partTable) ) {
#	$self->{PARTITION_TABLE} = $self->findPartitionTable($partTable, $self->getVmkName(), 
#							     $self->getVmnixDevName());
#    } else {
#	Warning("Could not load partition table.");
#    }
#}

# Inherited from super class
# sub toString() {

#####################################################################
# Multipathing functions
#####################################################################
# Inherited from super class
# sub getMultipathPolicy() {
# sub getMultipathPaths() {
# sub getMultipathActive() {
# sub getMultipathPrefer() {
# sub getMultipathInfo() {

sub setMultipathPolicy($) {
   my $self = shift;
   my $policy = shift;

   return $self->setMultipathInternal("policy", $policy, "ESXHostEdit_SetMultipathPolicy");
}

sub setMultipathActive($) {
   my $self = shift;
   my $path = shift;

   return $self->setMultipathInternal("path", $path, "ESXHostEdit_SetMultipathActive");
}

sub setMultipathPrefer($) {
   my $self = shift;
   my $path = shift;

   return $self->setMultipathInternal("path", $path, "ESXHostEdit_SetMultipathPrefer");
}

sub enableMultipathPath($) {
   my $self = shift;
   my $path = shift;

   return $self->setMultipathInternal("path", $path, "ESXHostEdit_EnableMultipathPath");
}

sub disableMultipathPath($) {
   my $self = shift;
   my $path = shift;

   return $self->setMultipathInternal("path", $path, "ESXHostEdit_DisableMultipathPath");
}


######################################################################
## Caching Support Functions                                         #
######################################################################

# Inherited from super class
# sub getCacheOptions() {
# sub getCacheOption($) {
# sub isCachedNever($) {

#####################################################################
# End of Caching Support Functions                                  #
#####################################################################

#####################################################################
# Disk Modification Methods.   These methods modify a disk.         #
#####################################################################

# Warning, creates a partition.  Use with caution.
# Deprecated.
sub createPartition($$$) {
    my $self = shift;
    my ($start_cyl, $num_cyl, $part_id) = @_;
    Panic("Not Implemented for Remote Calls and Deprecated");
}


# Inherited from super class
# sub checkPartInfo($) {


# Warning, creates a list of partitions.  Use with caution.  Format
# for function, an array of hashes where each array has the keys:
# NUM, START_CYL, END_CYL, and TYPE
#
# Returns 0 on failure, 1 on success.
sub makePartitions($) {
    my $self = shift;
    my($partTable) = @_;

    my $rc;

    if( !defined($partTable) ) {
	Warning("Attempting to make partitions with a null partition table.");
	return(0);
    }

    my $in = makeServerdRequest('ESXHostEdit_MakePartitions');
    $in->setValue('.in.diskname', $self->getVmkName());

    my $partTable_node = VMware::DOMAccess->new('partitionTable');
    if( !VMware::HConfig::Serialize::fromPartitionTableToXML($partTable_node, $partTable) ) {
	$in->dispose();
	$partTable_node->dispose();
	Warning("Could not serialize partition object.");
	return(0);
    }

    $in->addSubTree('.in', $partTable_node);
    $partTable_node->dispose();

    my $out = $self->sendServerdRequest($in);
    $rc = getReturnCode($out);

    if( !$rc ) {
	ServerdWarning($out);
	Warning("Failed to make partitions.");
    }

    $in->dispose();
    $out->dispose();

    if( $rc ) {
	# Only load if the serverd request succeeds.  This will reduce the number
        # of remote calls.

	# XXX This does not actual reload the partition table in the remote version.
	#     We must go back to the server and get new remote data.
	$self->loadPartitionTable();
    }

    return($rc);
}


# Not implemented for this class
# sub dumpPartitionInfo($) {


# Inherited from super class
# sub deleteAllPartitions() {


# Creates a new MFS partition on a partition.
sub createNewMFS($$$) {
    my $self = shift;
    my ($partition, $blocksize, $version) = @_;
    my ($rc);

    my ($name) = $self->getVmkName();

    if( $name eq "" ) {
	Warning("Attempting to create an VMFS on a device with null device name.");
	return(0);
    }

    if( !VMware::HConfig::RawDisk::isLogVmkDiskName($name) ) {
	Warning("Must create an VMFS on a device using a logical scsi name. " . 
		"Name that was used is '$name'.");
	return(0);
    }

    my $partname = VMware::HConfig::RawDisk::makeLogVmkPartName($name, $partition);
    if( !defined($partname) ) {
	Warning("Invalid partition name $partname.");
	return(0);
    }

    my $in = makeServerdRequest('ESXHostEdit_CreateVMFS');
    $in->setValue('.in.partition', $partname);
    $in->setValue('.in.blocksize', $blocksize);
    $in->setValue('.in.version', $version);
    my $out = $self->sendServerdRequest($in);
    $rc = getReturnCode($out);

    if( !$rc ) {
	ServerdWarning($out);
	Warning("Failed to create new VMFS.");
    }

    $in->dispose();
    $out->dispose();

    if( $rc ) {
	# Only load if the serverd request succeeds.  This will reduce the number
        # of remote calls.

	# XXX This does not actual reload the partition table in the remote version.
	#     We must go back to the server and get new remote data.
	$self->loadPartitionTable();
    }

    return($rc);
}


# Warning, changes the type of a partition.  Use with caution.
# id should be an integer
sub setPartitionID($$) {
    my $self = shift;
    my ($partition, $id_int) = @_;
    my ($rc);

    my ($name) = $self->getVmkName();

    if( $name eq "" ) {
	Warning("Attempting to change partition id on a disk with a null disk name.");
	return(0);
    }

    if( !VMware::HConfig::RawDisk::isLogVmkDiskName($name) ) {
	Warning("Must change a partition id on a device using a logical scsi name. " . 
		"Name that was used is '$name'.");
	return(0);
    }

    my $partname = VMware::HConfig::RawDisk::makeLogVmkPartName($name, $partition);
    if( !defined($partname) ) {
	Warning("Invalid partition name $partname.");
	return(0);
    }

    my $in = makeServerdRequest('ESXHostEdit_ChangePartitionType');
    $in->setValue('.in.partition', $partname);
    $in->setValue('.in.type', sprintf('%x', $id_int));
    my $out = $self->sendServerdRequest($in);
    $rc = getReturnCode($out);

    if( !$rc ) {
	ServerdWarning($out);
	Warning("Failed to change partition from of $partname to " . sprintf('%x', $id_int) . "\n");
    }

    $in->dispose();
    $out->dispose();

    if( $rc ) {
	# Only load if the serverd request succeeds.  This will reduce the number
        # of remote calls.

	# XXX This does not actual reload the partition table in the remote version.
	#     We must go back to the server and get new remote data.
	$self->loadPartitionTable();
    }

    return($rc);
}

#####################################################################
# End of Disk Modification Methods.   These methods modify a disk.  #
#####################################################################

#####################################################################
# Methods not intended for use by user                              #
#####################################################################

# Not implemented for this class
# sub findGeometry($) {


# Find the partition information by consulting the remote data.
#
# On success, return an array containing the partition information.
# If no partitions found, return an empty array.
# 
# Definition of the partition fields: 
#
# return->[n]->{PARTNUM}: Number of the partition
# return->[n]->{START_CYL}: Logical start cylinder number of the partition
# return->[n]->{END_CYL}: Logical end cylinder number of the partition
# return->[n]->{SECTORS}: Size of partition in sectors.  (not exact)
# return->[n]->{TYPE}: Integer corresponding to the type of the partition.
# return->[n]->{MFS}: Flag indicating if the partition has the MFS file system.
sub findPartitionTable($$$) {
    my $self = shift;
    my ($partTable, $name, $devName) = @_;

    Debug(1, "Entered");

    # Make a serverd request to get the partition data


    my $doc = $self->getRemoteData();
    $partTable = findPartitionsByRemoteData($partTable, $doc);
    return($partTable);
}

# Find the partition information by consulting the remote data.
# On success, return an array containing the partition information.
# If an error occurs, return undef.  If no partitions found, return
# an empty array.
# 
# Definition of the partition fields: see the comments for findPartition()
sub findPartitionsByRemoteData($$) {
    my ($partTable, $doc) = @_;

    Debug(1, "Entered");

    my $node = $doc->getSubTree('.partitionTable');
    if( !defined($node) ) {
	return(undef);
    }
    $partTable = VMware::HConfig::Serialize::toPartitionTableFromXML($node, $partTable);
    $node->dispose();
    return($partTable);
}


# Not implemented for this class
# sub findPartitionsByPartTool($$$) {
# sub isPartDOSCompatible($) {
# sub lookupVsdDevName($$) {
# sub checkMFS($$) {
# sub s_system($$) {

sub rescanVMFS () {
   Panic("Not Implemented for Remote Call.");
}

# Returns multipath info for a disk.  The example below shows such the
# structure of the info.
# 
# { POLICY => 'fixed',
#   PATHS => [ { PATH => 'vmhba0:0:1', STATUS => 'on'} ,
#              { PATH => 'vmhba1:0:1', STATUS => 'on'} ],
#   ACTIVE => 'vmhba0:0:1',
#   PREFER => 'vmhba0;0:1' };
#
sub findMultipathInfo($$$) {
   my $self = shift;

   Debug(1, "Entered");

   my $info = { POLICY => undef,
                PATHS => [],
                ACTIVE => undef,
                PREFER => undef };

   my $name = $self->getVmkName();

   my $in = makeServerdRequest('ESXHostInfo_GetVmkMultipathing');
   $in->setValue('.in.disk', $name);
   my $out = $self->sendServerdRequest($in);
   my $rc = getReturnCode($out);

   if (!$rc) {
      ServerdWarning($out);
      Warning("Failed to load multipathing data for disk $name.");
      return undef;
   }

   $info->{POLICY} = $out->getValue("exec.out.multipath.policy");
   $info->{ACTIVE} = $out->getValue("exec.out.multipath.active");
   $info->{PREFER} = $out->getValue("exec.out.multipath.prefer");

   $info->{PATHS} = [];
   foreach my $path ($out->listElementNames('exec.out.multipath.paths')) {
      my $key = "exec.out.multipath.paths." . $path;
      my $path = $out->getValue($key . ".name");
      my $status = $out->getValue($key . ".status");

      push(@{$info->{PATHS}}, { PATH => $path, STATUS => $status });
   }

   $in->dispose();
   $out->dispose();

   return($info);
}

sub setMultipathInternal {
   my $self = shift;
   my $key = shift;
   my $path = shift;
   my $cmd = shift;

   my $name = $self->getVmkName();
   my $in = makeServerdRequest($cmd);
   $in->setValue('.in.disk', $name);
   $in->setValue('.in.' . $key, $path);
   my $out = $self->sendServerdRequest($in);
   my $rc = getReturnCode($out);

   if (!$rc) {
      ServerdWarning($out);
      return 0;
   }

   # Invalidate multipath info when it gets changed.
   $self->{MULTIPATH_INFO} = undef;

   return 1;
}

#####################################################################
# Serverd Remote Request Support Functions                          #
# XXX These functions should be split into a remote module of its   #
#     own among other things.                                       #
#     Other copies of these functions exist at                      #
#     SSHostRemote.pm, RawDiskRemote.pm, MFSFileSystem.pm           #
#     Index.pm                                                      #
#####################################################################

# The DOMAccess object returned must be explicitly freed by the caller
sub makeServerdRequest($) {
    my($op) = @_;
    my $doc = VMware::DOMAccess->new('exec');
    $doc->setValue('.op', $op);
    return($doc);
}

# Sends a serverd request
sub sendServerdRequest($) {
   my($self) = shift;
   my($in) = @_;
   my $server = $self->getRemoteServer();

   return VMware::HConfig::SSHostRemote::doServerdRequestWithServer($server, $in, undef);
}

# Gets the return code of the response from serverd
sub getReturnCode($) {
    my($out) = @_;
    my $rc = $out->getValue('exec.returncode');
    if( $rc eq '' ) {
	Warning("Null return code received");
	return(0);
    }
    return($rc);
}

# Propagates warnings sent by vmserverd
sub ServerdWarning($) {
    my($out) = @_;
    my $key;

    my @errors = $out->listElementNames('.messages');
    foreach $key (@errors) {
	Warning($out->getValue('.messages.' . $key));
    }
}

#####################################################################
# End of Serverd Remote Request Support Functions                   #
#####################################################################

1;
