 #!/usr/bin/perl -w

#############################################################
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
#############################################################
#
# SCSIAdapter.pm
# 
# A class that is used to gather and manipulate information about the
# SCSI disks available to the VM's on the computer.

package VMware::HConfig::SCSIAdapter;
use strict;
use VMware::ExtHelpers qw(&Warning);
use VMware::HConfig::PCIDevice;
use VMware::HConfig::RawDisk;
use VMware::HConfig::Constant qw($gVmkDeviceType_hba @gVmnixHostAdapterNames);

#####################################################################
# Public Static Methods (indep. of each object)                     #
#####################################################################

sub new($) {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my($name) = @_;
    my($self) = {};
    
    $self->{VMK_LOGIC_NAME} = $name;  # Logical name of the adapter.  From our point of view 
                                      # it looks like a SCSI. ie. scsi0, scsi1
    $self->{VMNIX_LOGIC_NAME} = "";   # The logical name of the SCSI/RAID adapter as seen 
                                      # by Linux. ie. scsi0, id0, rd0
                                      # This will be same as the logical name in the vmkernel for
    $self->{PCI_DEVICE} = undef;      # The PCI to which this adapter belongs
    $self->{DISKS} = [];              # The disk objects that the adapter owns

    $self->{DEVICES} = {};            # The non-disk devices that the adapter hosts
    $self->{WWPN} = "";               # The WWPN for the adapter if it is on a SAN
    $self->{SANDEVICES} = [];

    bless($self, $class);
    return($self);
}

# Returns true if the name is in "logical" vmkernel SCSI format
sub isLogVmnixSCSIName($) {
    my($name) = @_;

    if( grep($name =~ /^$_[0-9]+$/, @gVmnixHostAdapterNames) ) {
	return(1);
    }
    return(0);
}

# Returns true if the name is in "logical" vmkernel SCSI format
sub isLogVmkSCSIName($) {
    my($name) = @_;

    return( $name =~ /^$gVmkDeviceType_hba[0-9]+/ );
}

#####################################################################
# Public Methods                                                    #
#####################################################################

# Accessor functions
sub getVmkName() {
    my($self) = shift;
    return($self->{VMK_LOGIC_NAME});
}

sub isShared() {
    my($self) = shift;
    if( defined($self->getPCIDevice()) ) {
	return($self->getPCIDevice()->getOwner() == VMware::HConfig::PCIDevice::SHARED);
    } else {
	return(0);
    }
}

sub getPCIDevice() {
    my($self) = shift;
    return($self->{PCI_DEVICE});
}

sub getDisks() {
    my($self) = shift;
    return($self->{DISKS});
}

sub toString() {
    my($self) = shift;
    my $s = "";

    my $shared = "shared";
    if( !$self->isShared() ) {
	$shared = "NOT $shared";
    }

    my $devinfo = "";
    if( defined($self->getPCIDevice()) ) {
	$devinfo = " pci device=" . $self->getPCIDevice()->getHash() . 
	    " (" . $self->getPCIDevice()->getName() . ")";
    }
    if ($self->getWWPN() ne "") {
        $devinfo .= " wwpn=" . $self->getWWPN();
    }

    $s .= "Vmk $gVmkDeviceType_hba Adapter " . $self->getVmkName() . $devinfo . 
        " is " . $shared . 
	" and has the following disks, CDROM, Tape drives, and other devices:\n";

    my $d;
    foreach $d (@{$self->getDisks()}) {
	$s .= $d->toString();
    }

    my $type;
    foreach $type ($self->getNonDiskDeviceTypes()) {
	my @devices = $self->getNonDiskDevicesByType($type);

	if( $#devices > -1 ) {
	    $s .= "$type device(s): " . join(', ', @devices) . "\n";
	}
    }
    
    $s .= "Has the following San Devices\n";
    my $sanDevice;
    foreach $sanDevice (@{$self->getSanDevices()}) {
       my $target;
       $s .= " { ";
       for $target ( keys %$sanDevice ) {
          $s .= "$target=$sanDevice->{$target} ";
       }
       $s .= " } \n";
    }
    return($s);
}


# Mutator functions
sub setPCIDevice($) {
    my($self) = shift;
    $self->{PCI_DEVICE} = shift;
}

sub setShared() {
    my($self) = shift;
    if( defined($self->getPCIDevice()) ) {
	$self->getPCIDevice()->setOwner(VMware::HConfig::PCIDevice::SHARED);
    }
}

sub addDisk(@) {
    my($self) = shift;
    my (@disks) = @_;

    if( $#disks > -1 ) {
	my @newdisks = @{ $self->{DISKS} };

	push(@newdisks, @disks);
	$self->{DISKS} = \@newdisks;
    }
}

sub removeDisk($) {
    my($self) = shift;
    my($disk) = @_;
    my @disks = @{ $self->{DISKS} };
    my @newdisks;

    while( $#disks > -1 ) {
	my $d = shift(@disks);
	if( $disk != $d ) {
	    push(@newdisks, $d);
	}
    }

    $self->{DISKS} = \@newdisks;
}

sub setDisk(@) {
    my($self) = shift;
    my (@disks) = @_;

    $self->{DISKS} = \@disks;
}

# XXX This is really a set operation, not an add. -jhu
sub addNonDiskDevice($$) {
    my($self) = shift;
    my ($vmkname, $type) = @_;
    $self->{DEVICES}->{$vmkname} = $type;
}

sub removeNonDiskDevice($) {
    my($self) = shift;
    my ($vmkname) = @_;
    $self->{DEVICES}->{$vmkname} = undef;
}

# Gets all the non-disk devices
# Reference to hash where key is the name of the disk device and value is
# the type.
sub getNonDiskDevices() {
    my($self) = shift;
    return($self->{DEVICES});
}

# Gets all the non-disk devices by type
sub getNonDiskDevicesByType($) {
    my($self) = shift;
    my($type) = @_;

    my @devices;
    my $dev;
    foreach $dev (keys(%{ $self->{DEVICES} }) ) {
	if( $type eq $self->{DEVICES}->{$dev} ) {
	    push(@devices, $dev);
	}
    }
    return(@devices);
}

# Gets all the non-disk device types
sub getNonDiskDeviceTypes() {
    my($self) = shift;
    my @typeList;

    my $type;
    foreach $type (values(%{ $self->{DEVICES} }) ) {
	if( !grep( $_ eq $type, @typeList ) ) {
	    push(@typeList, $type);
	}
    }
    return(@typeList);
}

# Get world wide port name for this adapter
sub getWWPN() {
   my($self) = shift;
   return $self->{WWPN};
}

# Set world wide port name for this adapter
sub setWWPN() {
   my($self) = shift;
   my ($wwpn) = @_;
   $self->{WWPN} = $wwpn;
}

sub getSanDevices() {
   my($self) = shift;
   return $self->{SANDEVICES};
}

sub setSanDevices(@) {
   my($self)        = shift;
   my(@sanDevices) = @_;
   $self->{SANDEVICES} = \@sanDevices;
}

#####################################################################
# Methods not intended for use by user                              #
#####################################################################

1;
