# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::IIS::SessionGate;
# File: VMware/IIS/SessionGate.pm
# ---------------------------------------------------------------------------
#  1. Get/Set login and session cookies.
#  2. If the current client has not been authenticated, squirrel away the
#     original URI and request parameters; then redirect the request to the
#     authentication URI. (Redirection is probably -different- than simply
#     handing the request over.)
#  3. Determine which Perl module should be fielding the current request.
#  4. Hand the request over for processing; return results to the client.
#
#  
# ---------------------------------------------------------------------------

use strict;

use VMware::IIS::Request;
use VMware::Management::Util qw( trace
                                 hash_cookies
                                 session_is_valid_non_vmdb
                                 validate_session_non_vmdb );
use VMware::Management::Session;
use MIME::Base64 ();
use URI::Escape ();
use VMware::Control::Profiler;

my $r;
my %params;
my %cookies;
my $p = VMware::Control::Profiler::getInstance("mui");


# ---------------------------------------------------------------------------
#  handler
# ---------------------------------------------------------------------------
sub handler {
   my $stub = $p->profile();
   trace("\n\nSessionGate handler()\n");

   %params = VMware::Management::Util::hashQueryString("?$ENV{QUERY_STRING}");
   foreach (sort keys %params) {
      trace("params{$_} = $params{$_}");
   }

   $r = VMware::IIS::Request->new();
   %cookies = hash_cookies( );
   
   my $data = direct_request( );
   my $header = $r->get_http_header( );

   #trace("end SessionGate\nheader\n'$header'\ndata\n'$data'");

   return ($header, $data);
}

# ---------------------------------------------------------------------------
#  direct_request
# ---------------------------------------------------------------------------
sub direct_request {
  my $stub = $p->profile();
  my $s;
  my %h;

  $r->header_out( "Last-Modified", return_last_modified( ) );
  
  $r->header_out( "Expires", return_last_modified( ) );
  $r->header_out( "Date", return_last_modified( ) );
  
  $r->header_out( "Keep-Alive", "timeout=15, max=91" );
  $r->header_out( "Connection", "Keep-Alive, Keep-Alive" );
  #$r->header_out( "Transfer-Encoding", "chunked, chunked" );
  
  # If the client is authenticated and if the current URI is the login URI,
  # forward the client to the default URI; if the current URI is something
  # other than the login URI, direct the client's request appropriately. If
  # the client is not authenticated, ask to see some credentials.

  if ( VMware::Management::Session::session_is_valid( $cookies{ "vmware.mui.sid" } ) ) {
    unless ($params{"auto-refresh"}) {
      $r->headers_out->add( "Set-Cookie", VMware::Management::Session::validate_session( $cookies{ "vmware.mui.sid" } ) );
    }
    if ( $ENV{ "SCRIPT_NAME" } eq $ENV{ "vmware_LOGIN_URI" } ) {
      forward_request( $ENV{ "vmware_DEFAULT_URI" } );
    } else {
      ( $s, %h ) = call_handler( );
    }
  } else {
    if ( $ENV{ "SCRIPT_NAME" } eq $ENV{ "vmware_LOGIN_URI" } ) {
      ( $s, %h ) = call_handler( );
    } else {
      force_login( );
    }
  }

  foreach my $k ( keys( %h ) ) {
    my $l = $k;
    $l =~ tr/A-Z/a-z/;
    if ( $l eq "content-type" ) {
      $r->content_type( $h{ $k } );
    } else {
      if ( $l eq "location" ) {
        forward_request( $h{ $k } );
      }
      if ( $l =~ /cookie/ ) {
        if (defined($h{$k})) {
           $r->headers_out->add( "Set-Cookie" => $h{ $k } );
           trace( "$k is ".$h{ $k }, "debug" );
        }
        next;
      }
      $l = $h{ $k };
      $l =~ tr/A-Z/a-z/;
      if ( $l eq "no-cache" ) {
        #todo - implement no cache
        #$r->no_cache( 1 );
      }
      $r->header_out( "$k", $h{ $k } );
    }
  }

  $r->headers_out->add( "Set-Cookie" => "vmware.mui.test=1;path=/;" );  

  return $s;
}

# ---------------------------------------------------------------------------
#  call_handler
# ---------------------------------------------------------------------------
sub call_handler {
  my $stub = $p->profile();
  no strict 'refs';
  
  my $handler = $ENV{ "vmware_HANDLER" };
  eval( "require $handler" );
  if( $@ ) {
      trace( "Failed to load handler.\nReason:\n$@" );
      return("Error: Failed to load handler.\n\nReason: $@\n", undef);
  }
  $handler .= "::handler";
  trace( "$handler( ) called.", "debug" );

  return &$handler( \%params );
}

# ---------------------------------------------------------------------------
#  force_login
# ---------------------------------------------------------------------------
sub force_login {
  my $stub = $p->profile();
  my $url = $ENV{ "vmware_LOGIN_URI" };
  my $uri = $url."?n=";
  my $next = $ENV{ "SCRIPT_NAME" }."?";

  foreach my $key ( keys( %params ) ) {
    unless ( !$params{ $key } ) {
      $next .= "$key=".$params{ $key }."&";
    }
  }
  chop( $next );

  forward_request($uri.URI::Escape::uri_escape(MIME::Base64::encode_base64($next, "")));

  $r->header_out( "Location" => $url);
}

# ---------------------------------------------------------------------------
#  forward_request
# ---------------------------------------------------------------------------
sub forward_request {
    my $uri = shift( );
    my $url;

    if ( !$ENV{ "HTTP_HOST" } ) {
	   $url = $ENV{URL_PREFIX}.$ENV{SERVER_NAME}.($ENV{SERVER_PORT} && 
	        $ENV{SERVER_PORT} ne "80" ? ":".$ENV{SERVER_PORT} : "").$uri;
    } else {
       $url = $ENV{URL_PREFIX}.$ENV{HTTP_HOST}.$uri;
    }

    $r->header_out( "Location" => $url);
}

# ---------------------------------------------------------------------------
#  return_last_modified
# ---------------------------------------------------------------------------
sub return_last_modified {
  return scalar( gmtime( time( ) ) );
}

# ---------------------------------------------------------------------------
#  env_to_string
# ---------------------------------------------------------------------------
sub env_to_string {
  my $s = $r->uri . "\n";
  foreach my $key ( keys( %params ) ) {
    $s .= "$key = ".$params{ $key }."\n";
  }
  foreach my $key ( keys( %ENV ) ) {
    $s.= "$key = ".$ENV{ $key }."\n";
  }
  return $s;
}

1;
