#!/usr/bin/perl -w

#############################################################
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
#############################################################
#
# LocationDB.pm
#
# Functions to access and manipulate the locations database.
#
package VMware::LocationDB;
use strict;
use Exporter   ();
use vars       qw($VERSION @ISA @EXPORT @EXPORT_OK %EXPORT_TAGS);

BEGIN {
    # set the version for version checking
    $VERSION     = 1.00;
    @ISA         = qw(Exporter);
    @EXPORT_OK = qw(%gDBAnswer 
		    %gDBFile 
		    %gDBDir 
		    $gInstallerMainDB
		    &db_load 
		    &db_append 
		    &db_add_file 
		    &db_remove_file 
		    &db_file_in 
		    &db_file_ts 
		    &db_add_dir 
		    &db_dir_in
		    &db_remove_dir 
		    &db_remove_answer 
		    &db_add_answer 
		    &db_get_answer
		    &db_try_get_answer
		    &db_save
		    &db_iterate
		    &db_iterate_file
		    &db_iterate_dir
		    );
}

use vars      qw(%gDBAnswer %gDBFile %gDBDir $gInstallerMainDB);

# START OF THE SECOND LIBRARY FUNCTIONS
# Exported global variables
%gDBAnswer = ();
%gDBFile = ();
%gDBDir = ();
$gInstallerMainDB = "/etc/vmware/locations";

# Load the installer database
sub db_load {
  undef %gDBAnswer;
  undef %gDBFile;
  undef %gDBDir;

  open(INSTALLDB, '<' . $gInstallerMainDB) or error('Unable to open the installer database ' . $gInstallerMainDB . ' in read-mode.' . "\n\n");
  while (<INSTALLDB>) {
    if (/^answer (\S+) (\S+)/) {
      $gDBAnswer{$1} = $2;
    } elsif (/^answer (\S+)/) {
      $gDBAnswer{$1} = '';
    } elsif (/^remove_answer (\S+)/) {
      delete $gDBAnswer{$1};
    } elsif (/^file (\S+) (\S+)/) {
      $gDBFile{$1} = $2;
    } elsif (/^file (\S+)/) {
      $gDBFile{$1} = 0;
    } elsif (/^remove_file (\S+)/) {
      delete $gDBFile{$1};
    } elsif (/^directory (\S+)/) {
      $gDBDir{$1} = '';
    } elsif (/^remove_directory (\S+)/) {
      delete $gDBDir{$1};
    }
  }
  close(INSTALLDB);
}

# Open the database on disk in append mode
sub db_append {
  if (not open(INSTALLDB, '>>' . $gInstallerMainDB)) {
    error('Unable to open the installer database ' . $gInstallerMainDB . ' in append-mode.' . "\n\n");
  }
  # Force a flush after every write operation.
  # See 'Programming Perl', p. 110
  select((select(INSTALLDB), $| = 1)[0]);
}

# Add a file to the tar installer database
# flags:
#  0x1 write time stamp
sub db_add_file {
  my $file = shift;
  my $flags = shift;

  if ($flags & 0x1) {
    my @statbuf;

    @statbuf = stat($file);
    if (not (defined($statbuf[9]))) {
      error('Unable to get the last modification timestamp of the destination file ' . $file . '.' . "\n\n");
    }

    $gDBFile{$file} = $statbuf[9];
    print INSTALLDB 'file ' . $file . ' ' . $statbuf[9] . "\n";
  } else {
    $gDBFile{$file} = 0;
    print INSTALLDB 'file ' . $file . "\n";
  }
}

# Remove a file from the tar installer database
sub db_remove_file {
  my $file = shift;

  print INSTALLDB 'remove_file ' . $file . "\n";
  delete $gDBFile{$file};
}

# Determine if a file belongs to the tar installer database
sub db_file_in {
  my $file = shift;

  return defined($gDBFile{$file});
}

# Return the timestamp of an installed file
sub db_file_ts {
  my $file = shift;

  return $gDBFile{$file};
}

# Add a directory to the tar installer database
sub db_add_dir {
  my $dir = shift;

  $gDBDir{$dir} = '';
  print INSTALLDB 'directory ' . $dir . "\n";
}

# Determine if a directory belongs to the tar installer database
sub db_file_in {
  my $dir = shift;

  return defined($gDBDir{$dir});
}

# Remove a directory from the tar installer database
sub db_remove_dir {
  my $dir = shift;

  delete $gDBDir{$dir};
  print INSTALLDB 'remove_directory ' . $dir . "\n";
}

# Remove an answer from the tar installer database
sub db_remove_answer {
  my $id = shift;

  if (defined($gDBAnswer{$id})) {
    print INSTALLDB 'remove_answer ' . $id . "\n";
    delete $gDBAnswer{$id};
  }
}

# Add an answer to the tar installer database
sub db_add_answer {
  my $id = shift;
  my $value = shift;

  db_remove_answer($id);
  $gDBAnswer{$id} = $value;
  print INSTALLDB 'answer ' . $id . ' ' . $value . "\n";
}

# Retrieve an answer that must be present in the database
sub db_get_answer {
  my $id = shift;

  if (not defined($gDBAnswer{$id})) {
    error('Unable to find the answer ' . $id . ' in the installer database (' . $gInstallerMainDB . '). You may want to re-install VMware.' . "\n\n");
  }

  return $gDBAnswer{$id};
}

# Try to retrieve an answer that may be in the database.  Return null string if it does not exist.
sub db_try_get_answer {
  my $id = shift;
    
  if (not defined($gDBAnswer{$id})) {
      return("");
  }
  return $gDBAnswer{$id};
}
# Save the tar installer database
sub db_save {
  close(INSTALLDB);
}

# Iterate through all answers in a database in memory, calling <func> with
# id/value pairs and the remaining arguments to this function
sub db_iterate($) {
  my $func = shift;

  my $key;
  foreach $key (sort(keys(%gDBAnswer))) {
    my $val = db_get_answer($key);
    &$func($key, $val);
  }
}

# Iterate through all files in a database in memory, calling <func> with
# the name if the file and the timestamp if it exists
sub db_iterate_file($) {
  my $func = shift;

  my $file;
  foreach $file (sort(keys(%gDBFile))) {
    my $ts = $gDBFile{$file};
    if( defined($ts) && $ts == 0 ) {
      $ts = undef;
    }
    &$func($file, $ts);
  }
}

# Iterate through directories in a database in memory, calling <func> with
# the name if the directory and a null value
sub db_iterate_dir($) {
  my $func = shift;

  my $dir;
  foreach $dir (sort(keys(%gDBDir))) {
    my $val = $gDBFile{$dir};
    &$func($dir, $val);
  }
}

# Print an error message and exit
sub error {
  my $msg = shift;

  print STDERR wrap($msg . 'Execution aborted.' . "\n\n", 0);
  exit 1;
}
# END OF THE SECOND LIBRARY FUNCTIONS
