# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::Operation;
# File: VMware/Management/Operation.pm
# ---------------------------------------------------------------------------
# Handle control operations such as power-on, suspend, etc.
# ---------------------------------------------------------------------------

use strict;

use VMware::Control '1.00';
use VMware::Control::Server ();
use VMware::Control::VM ();
use VMware::Control::Profiler;
use VMware::Management::Util;
use VMware::Management::MultipartCache;

use CGI (); 
use XML::Sablotron ();
use XML::DOM ();
use URI ();
use URI::Escape ();

my %poweroff  = ( "action"      => "stop",
                  "ttl.pending" => "Power-Off: Pending",
                  "msg.pending" => "Powering the virtual machine off.",
                  "ttl.success" => "Power-Off: Success",
                  "msg.success" => "The virtual machine has been powered off.",
                  "ttl.error"   => "Power-Off: Error" );

my %suspend   = ( "action"      => "suspend_to_disk",
                  "ttl.pending" => "Suspend: Pending",
                  "msg.pending" => "Suspending the virtual machine.",
                  "ttl.success" => "Suspend: Success",
                  "msg.success" => "The virtual machine has been suspended.",
                  "ttl.error"   => "Suspend: Error" );

my %resume    = ( "action"      => "start",
                  "ttl.pending" => "Resume: Pending",
                  "msg.pending" => "Resuming the virtual machine.",
                  "ttl.success" => "Resume: Success",
                  "msg.success" => "The virtual machine has been resumed.",
                  "ttl.error"   => "Resume: Error" );

my %poweron   = ( "action"      => "start",
                  "ttl.pending" => "Power-On: Pending",
                  "msg.pending" => "Powering the virtual machine on.",
                  "ttl.success" => "Power-On: Success",
                  "msg.success" => "The virtual machine has been powered on.",
                  "ttl.error"   => "Power-On: Error" );

my %reset     = ( "action"      => "reset",
                  "ttl.pending" => "Reset: Pending",
                  "msg.pending" => "Resetting the virtual machine.",
                  "ttl.success" => "Reset: Success",
                  "msg.success" => "The virtual machine has been reset.",
                  "ttl.error"   => "Reset: Error" );

my %answer    = ( "action"      => "answer",
                  "ttl.pending" => "Reset: Pending",
                  "msg.pending" => "Resetting the virtual machine.",
                  "ttl.success" => "Reset: Success",
                  "msg.success" => "The virtual machine has been reset.",
                  "ttl.error"   => "Reset: Error" );

my %set       = ( "action"      => "set",
                  "ttl.pending" => "Set: Pending",
                  "msg.pending" => "Setting configuration parameter(s).",
                  "ttl.success" => "Set: Success",
                  "msg.success" => "The configuration parameter(s) have been set.",
                  "ttl.error"   => "Set: Error" );

my %operation = ( "poweroff" => \%poweroff,
                  "suspend"  => \%suspend,
                  "resume"   => \%resume,
                  "poweron"  => \%poweron,
                  "reset"    => \%reset,
                  "answer"   => \%answer,
                  "set"      => \%set );

my $q;
my %credentials;
my $p = VMware::Control::Profiler::getInstance("mui");

sub handler {
  my $stub = $p->profile();
  $q = new CGI( shift( ) );
  %credentials = hash_credentials( );
  my $op = $operation{ $ENV{ "OPERATION" } };
  my ( $server, $server_ok, $vm, $vm_ok, $err, $errstr );
  
  $server = &VMware::Control::Server::new( undef, 0, $credentials{ "l" }, $credentials{ "m" } );
  ( $server_ok, $err, $errstr ) = connect_ctrl_obj( $server );
  unless( $server_ok ) {
    # $server is not connected, so why call cleanup( )?
    return emit_xml( undef, undef, $err, $errstr );
  }

  my $cfg = $q->param( "c" );
  $cfg ||= $q->param( "vm" );
  $vm = VMware::Control::VM::new( $server, $cfg );
  ( $vm_ok, $err, $errstr ) = connect_ctrl_obj( $vm );
  unless( $vm_ok ) {
    cleanup( undef, $server ); # $server is ok if we get here.
    return emit_xml( undef, undef, $err, $errstr );
  }
  
  # If force is set, do the operation synchronously since it should be quick.
  my $action = $$op{ "action" };
  if ( ($$op{ "action" } eq "stop" || $$op{ "action" } eq "reset") && !$q->param("f") ) {
    $action = "request_$action";
  }
  trace( "Calling \$vm->$action.", "debug" );
  my $op_ok;
  if ($$op{"action"} eq "set") {
    my @names = $q->url_param();
    for my $name (@names) {
      next if ($name eq "c" || $name eq "f");
      
      $op_ok = $vm->set("$name", $q->url_param($name));
      if (! $op_ok) {
        last;
      }

      # Save everytime to ensure that the config jives with what has been set
      # thus far.
      $op_ok = $vm->save_config();
      if (! $op_ok) {
        last;
      }
    }
  } elsif ( $$op{ "action" } ne "answer" ) { 
    $op_ok = $vm->$action( $q->param( "f" ) or undef );
    # XXX Power on, resume and suspend should probably work regardless of the
    # availaibility of the appropriate tools version (especially while the
    # guest is booting). This, however, masks the fact that scripts failed to
    # run in those cases where users might expect them to (e.g., because they
    # believe that the tools are up and capable of running scripts).
    if (! $op_ok && ! $q->param("f") &&
        ($$op{"action"} eq 'start' || $$op{"action"} eq 'suspend_to_disk')) {
      my ( $err, $errstr ) = $vm->get_last_error( );
      if ($err == VMware::Control::VM_E_BADSTATE) {
        trace("Caught error: $err $errstr. Retrying $$op{action}.", "debug");
        $op_ok = $vm->$action(1);
      }
    }
  } else {
    $op_ok = $vm->answer_question( $q->param( "question-id" ), $q->param( "answer-id" ) );
  }
  unless ( $op_ok ) {
    ( $err, $errstr ) = $vm->get_last_error( );
    trace( $err, "error" );
    trace( $errstr, "error" );
    return emit_xml( $server, $vm, $err, $errstr );
  }

  return emit_xml( $server, $vm );
}

sub cleanup {
  my $stub = $p->profile();
  my ( $vm, $server ) = @_;
  if ( $vm ) {
    $vm->disconnect( );
  }

  if ( $server ) {
    $server->disconnect( );
  }
}

sub emit_xml {
  my $stub = $p->profile();
  my ( $server, $vm, $err, $errstr ) = @_;
  my $cfg = $q->param( "c" );
  $cfg ||= $q->param( "vm" );

  if ( defined( $err ) && $err != VMware::Control::VM_E_NEEDINPUT ) {
    my $parser = new XML::DOM::Parser;
    my $doc = $parser->parse( "<server></server>" );
    my $root = $doc->getDocumentElement( );
    my $processor = new XML::Sablotron();
    my $xsl = $ENV{ "style_DEFAULT" };
    my $xmlfile = &VMware::Management::Util::tmpdir() . "operation.xml";
    my $data;

    append_error( $doc, $root, $err, $errstr );

    append_host( $doc, $root );
    append_agent( $doc, $root );
    append_user( $doc, $root, $credentials{ "l" } );
    append_build( $doc, $root );
    append_product( $doc, $root, $server );

    if ( $vm ) {
      append_vm( $doc, $root, $cfg, $server );
    }

    $data = $doc->toString( );

    if ( $ENV{ "vmware_DEBUG_LEVEL" } > 2 ) {
      string_to_file( $data, $xmlfile, 1 );
    }

    cleanup( $vm, $server );
    $doc->dispose( );

    $processor->SetBase( "file://" . $ENV{ "vmware_BASEHREF" } );
    $p->start("XML::Sablotron::RunProcessor");
    $processor->RunProcessor( "arg:/template", "arg:/data", "arg:/result",
                              undef,
                              [ "template", file_to_string( $xsl ), "data", $data ] );
    $p->end();

    return ( $processor->GetResultArg("result"),
             "Pragma"        => "no-cache",
             "Cache-Control" => "no-cache, must-revalidate",
             "Content-Type"  => "text/html; charset=iso-8859-1" );
  } elsif ( defined( $err ) && $err == VMware::Control::VM_E_NEEDINPUT ) {
    cleanup( $vm, $server );
    return ( "",
             "Location" => "/eventlog?c=".URI::Escape::uri_escape( $cfg ) );
  } else {
    cleanup( $vm, $server );

    # Attempt to direct the client back to the previous page when the operation
    # is successful.

    # Pick a default direction.
    my $u = "/overview";

    if ( $ENV{ "HTTP_REFERER" } ) {
      # Grab the relative path, including the search portion from the referer.
      $u = URI->new( $ENV{ "HTTP_REFERER" } );
      my $h = $u->authority( );
      # Make sure that the refering host is the same as the m-ui host.
      if ( $h !~ /$ENV{ "SERVER_NAME" }/ ) {
        $u = "/overview";
      } else {
        $u = $u->path_query( );
      }

      # Make sure that the referer is not an operation - we don't want to loop
      # indefinitely.
      foreach my $k ( keys( %operation ) ) {
        $k = "/".$k;
        if ( $u =~ /^$k/ ) {
          $u = "/overview";
        }
      }

    }
    if($^O eq "MSWin32") {
      return ( qq(<head><META HTTP-EQUIV="Refresh" Content=0;URL="$u"></head>),
               "Content-type" => "text/html");
    } else {
      return ( "",
               "Location" => $u );
    }
  }
}

1;
