# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::RC;
# File: VMware/Management/RC.pm
# Currently, the script handles Citrix, MSTS, and VNC clients on Windows.
# -- mgu

use strict;
use IO::Socket;
use CGI;
use VMware::Control::Server;
use VMware::Control::VM;
use VMware::Control::Profiler;
use VMware::Management::Util qw( trace
                                 hash_credentials
                                 connect_ctrl_obj );

my $p = VMware::Control::Profiler::getInstance("mui");

### Some global constants

# Error messages
my $conn_err_msg = "Error connecting to server: "; 
my $invalid_url = "Malformed URL";

# Links
my $citrix_d_page = "http://www.citrix.com/download/ica_clients.asp";
my $msts_d_page = "http://www.microsoft.com/WINDOWS2000/downloads/recommended/TSAC/tsac.asp";

# Keys to be checked (for the CGI object)
# 'width' and 'height' excluded because they are not needed by all handlers 
# and they are checked in get_width_height().
my @param_keys = ('os', 'browser', 'version', 'service', 'vm');


sub handler {
    my $stub = $p->profile();
  trace($ENV{QUERY_STRING}, "debug");
  my $q = new CGI;
  my ($vm, $ip, $error_message); 

  ### vnc_mime and vnc_reg 
  # 1) Bypass general error checking
  #    (Error checking implemented in their own handlers)
  # 2) By pass communication with the vm
  #    (since it is irrelevant and may not be available)
  #
  # XXX Note: The front end does not work with it yet.
  #
  if($q->param('service') eq 'vnc_mime') {
    return vnc_mime_handler($q); 
  } elsif($q->param('service') eq 'vnc_reg') {
    return vnc_reg_handler($q);
  }
    
  unless(verify_url($q)) {
    return error_handler($invalid_url);
  }

  # error_message used by both get_vm and get_ip
  ($vm, $error_message) = get_vm($q);
  if(!$error_message) {
    ($ip, $error_message) = get_ip($vm);
    if($vm) { $vm->disconnect(); } # if: just to be safe
  } 

  if($error_message) {
    return error_handler($error_message);
  }

  my $service = $q->param("service");
  if($service eq "citrix") { 
    return citrix_handler($q, $ip); 
  } elsif($service eq "msts") { 
    return msts_handler($q, $ip); 

  # vnc has a number of handlers:
  # vnc:       handles the normal case: returns the configuration file
  # vnc_mime:  handles cases in which the mime type is not configured.
  # vnc_reg:   returns a customized .reg file to the end user for importing.
  } elsif($service eq "vnc") { 
    return vnc_handler($q, $ip); 
  #
  # These two requests are handled at the beginning of the subroutine 
  #
  #} elsif($service eq "vnc_mime") {
  #  return vnc_mime_handler($q, $ip);
  #} elsif($service eq "vnc_reg") {
  #  return vnc_reg_handler($q, $ip);

  # PC Anywhere not supported for now
  #} elsif($service eq "pcanywhere") { 
  #  return pcanywhere_handler($q, $ip); 
  } else {
    my $err_str = "Service '" . $service . "' not supported";
    return error_handler($err_str);
  }
}

sub get_vm {
  my $q = shift;
  my %credentials = VMware::Management::Util::hash_credentials( );

  my ( $cfg, $server, $server_ok, $vm, $vm_ok, $err, $errstr );

  $server = &VMware::Control::Server::new( undef, 0, $credentials{ "l" }, $credentials{ "m" } );
  ( $server_ok, $err, $errstr ) = connect_ctrl_obj( $server );
  unless ( $server_ok ) {
    return ( undef, "$err: $errstr" );
  }

  $cfg = $q->param( "vm" );
  $vm = VMware::Control::VM::new( $server, $cfg );
  unless($vm) {
    if($server) { $server->disconnect(); } # if: just to be safe
    return ( undef, "Error connecting to the virtual machine" );
  }

  ( $vm_ok, $err, $errstr ) = connect_ctrl_obj( $vm );
  unless( $vm_ok ) {
    if($server) { $server->disconnect(); } # if: just to be safe
    return ( undef, "$err: $errstr" );
  }
  return ( $vm, undef );
}

# if ok, return (ip   , undef)
# else , return (undef, error_message) 
sub get_ip {
  my $vm = shift;
  my $ip = $vm->get("Config.guestinfo.ip");
  if($vm->get("Status.power") eq "off") {
    return (undef, "The virtual machine is powered off.");
  } elsif($ip !~ /^[\d\.]+$/ || $ip eq '0.0.0.0' || $ip eq '127.0.0.1') {
    return (undef, "IP address of the virtual machine is not available.");
  } else {
    return ($ip, undef);
  }
}

sub get_width_height {
  my $q = shift;
  my ($width, $height) = ($q->param("width") || 640, $q->param("height") || 480);
  if(($width == 640  && $height == 480)  ||
     ($width == 800  && $height == 600)  ||
     ($width == 1024 && $height == 768)  ||
     ($width == 1280 && $height == 1024) ||
     ($width == 1600 && $height == 1200)) {
    return ($width, $height, undef);
  } else { 
    return (undef, undef, "Resolution $width X $height not supported.");
  }
} 
 
# This is the only place where we put codes for all browsers together
# to save some hassle.  Note that  VBScript will be executed first.
sub error_html {
  my $error_message = shift;
  return <<EOF; 
      <body>
        <br>
        <center><font face=\"arial,helvetica\" size=2>$error_message</font></center>
        <script language="vbscript">
        <!--
          MsgBox "$error_message", 0, "VMware"
          Window.Close
        -->
        </script>

        <script language="javascript">
        <!--
          alert("$error_message");
          window.close();
        // -->
        </script>
      </body>
EOF
}

sub pack_html {
  my $html = shift;
  return($html,
         "Content-Type" => "text/html",
         "Cache-Control" => "must-revalidate");
}

sub verify_url {
  my $q = shift;

  for my $key (@param_keys) {
    unless($q->param($key)) {
      return undef;
    }
  }
  return 1;
}

# not used
sub verify_extension {
  my ($q, $ext) = @_;
  my $url = $q->self_url;
  return $url =~ /^[^\?]+\/\w+\.$ext\?/;
}


# citrix_handler --------------------------------------------------------------
#
# also, some helper functions to handle each browser type
#

sub citrix_handler {
  my ($q, $ip) = @_;
  my ($browser, $version) = ($q->param('browser'), $q->param('version'));
  my ($width, $height, $error_message) = get_width_height($q);

  if($error_message) {
    return error_handler($error_message);
  }

  my $html;

  if($browser eq 'ie') {
    $html = citrix_ie_html($q, $ip, $width, $height);
  } elsif($browser eq 'netscape') {
      $html = citrix_netscape_html($q, $ip, $width, $height);
  } else {
    $html = error_html("Browser [$browser] not supported.");
  }

  return pack_html($html);
}

sub citrix_ie_html {
  my ($q, $ip, $width, $height) = @_;

  return <<EOF;
      <html>
      <head>
        <script language="vbscript">
        <!-- 
          Sub ErrorHandler
            MsgBox "You need to install Citrix ICA Client.", 0, "VMware"
            Window.Open "$citrix_d_page", "citrix_download_window", "toolbar=1,location=1,directories=1,status=1,menubar=1,scrollbars=1,resizable=1", 600, 400
            Window.Close
          End Sub

          Sub ICO1_OnConnectFailed
            MsgBox "$conn_err_msg$ip", 0, "VMware"
            Window.Close
          End Sub

          Sub ICO1_OnDisconnect
            Window.Close
          End Sub
        -->
        </script>
      </head>
      <body topmargin=0 leftmargin=0 rightmargin=0 bottommargin=0 marginwidth=0 marginheight=0>
        <object classid='clsid:238f6f83-b8b4-11cf-8771-00a024541ee3'
                id=ICO1
                width=$width
                height=$height>
          <param name=Address value=$ip>
          <param name=Border value=0>
          <param name=DesiredHRes value=$width>
          <param name=DesiredVRes value=$height>
          <param name=Launch value=0>
          <param name=Start value=1>
          <param name=UserName value=Administrator>
        </object>
        <script language="vbscript">
        <!--
          On Error Resume Next
          If ICO1.UserName = Null Then
            ErrorHandler
          End If
          ICO1.onerror = GetRef("ErrorHandler")
        --> 
        </script>
      </body>
      </html>
EOF
}

sub citrix_netscape_html {
  my ($q, $ip, $width, $height) = @_;

  return <<EOF;
      <html>
      <head>
        <script language="javascript">
        <!--
          function startUp() {
            if(!navigator.mimeTypes["application/x-ica"]) {
              alert("You need to install Citrix ICA Client.");
              window.open("$citrix_d_page", "citrix_download_window", "toolbar=1,location=1,directories=1,status=1,menubar=1,scrollbars=1,resizable=1,width=600,height=400");
              window.close();
            } else { 
              //var ICO;
              if(document.getElementById) {
                //ICO = document.getElementById("ICO1");
              } else if(document.ICO1) {
                document.ICO1.Startup();
                //ICO = document.ICO1;
              }
            }
          }

          function ICO1_OnConnectFailed() {
            alert("$conn_err_msg$ip");
            window.close();
          }
    
          function ICO1_OnDisconnect() {
            window.close();
          }
        // -->
        </script>
      </head>
      <body topmargin=0 leftmargin=0 rightmargin=0 bottommargin=0 marginwidth=0 marginheight=0 onload="startUp()">
              <embed id=ICO1
                     name=ICO1
                     width=$width
                     height=$height
                     type=application/x-ica
                     Address=$ip
                     Border=0
                     DesiredHRes=$width
                     DesiredVRes=$height
                     Launch=0
                     Start=1
                     UserName=Administrator>
      </body>
      </html>
EOF
}

# msts_handler --------------------------------------------------------------
#

sub msts_handler {
  my ($q, $ip) = @_;
  my ($width, $height, $error_message) = get_width_height($q);
  if($error_message) {
    return error_handler($error_message);
  }

  my $html = <<EOF;
      <html>
      <body topmargin=0 leftmargin=0 rightmargin=0 bottommargin=0>
      <script language="javascript">
      <!--
        if(navigator.appName != "Microsoft Internet Explorer") {
          alert("Terminal Services Advance Client only runs in Internet Explorer");
          window.close();
        }
      // -->
      </script>

      <!-- From this point on, we use vbscript -->

      <object language="vbscript"
              id="MsTsc"
              classid="CLSID:a41a4187-5a86-4e26-b40a-856f9035d9cb"
              codebase="mstscax.cab#version=5,0,2221,1"
              width=$width
              height=$height
      >
      </object>

      <script language="vbscript">
      <!--
        Sub ErrorHandler
          MsgBox "You need to install Terminal Services Advanced Client (with default settings).", 0, "VMware"
          Window.Open "$msts_d_page", "tsac_download_window", "toolbar=1,location=1,directories=1,status=1,menubar=1,scrollbars=1,resizable=1", 600, 400
          Window.Close
        End Sub
        MsTsc.onerror = GetRef("ErrorHandler")

        ' Suppress default error handling
        On Error Resume Next
        MsTsc.Server = "$ip"
        If MsTsc.Server = Null Then
          ErrorHandler
        End If
        MsTsc.UserName = "Administrator"
        MsTsc.Connect

        ' Handler for connection failure
        Sub MsTsc_OnDisconnected(disconnectCode)
          If Not disconnectCode = 2 Then
            MsgBox "$conn_err_msg$ip", 0, "VMware"
          End If
          Window.Close
        End Sub
      -->
      </script>
      </body>
      </html>
EOF

  return pack_html($html);
}

# vnc_handler --------------------------------------------------------------
#

sub vnc_handler {
  my ($q, $ip) = @_;
  my $os = $q->param('os');
  
  if($os ne 'win') {
    return error_handler("Operating system [$os] not supported.");
  }

  # always have a valid ip:port to avoid error messages on the client side
  my $port = vnc_get_server_port($ip) || 5900;

  my $url = $q->self_url;
  $url =~ s/^([^\?]+)\/\w+\.\w+\?/$1\/vmvnc\.html\?/;
  $url =~ s/service=(\w+)/service=vnc_mime/;

  my $config = <<EOF;
# VMware
[connection]
host=$ip
port=$port

EOF

  return($config,
         "Content-Type" => "application/vmvnc; name=\"index.vmvnc\"",
         "Cache-Control" => "must-revalidate");
}

sub vnc_mime_handler {
  my $q = shift;
  my $os = $q->param('os');
  my ($browser, $version) = ($q->param('browser'), $q->param('version'));

  if($os ne 'win') {
    return error_handler("Operating system [$os] not supported.");
  }

  my $html;

  if($browser eq 'ie') {
    $html = vnc_mime_ie_html($q);
  } elsif($browser eq 'netscape') {
      $html = vnc_mime_netscape_html($q);
  } else {
    $html = error_html("Browser [$browser] not supported.");
  }
 
  return pack_html($html);
}

sub vnc_mime_ie_html {
  my $q = shift;

  my $url = $q->self_url;
  $url =~ s/^([^\?]+)\/\w+\.\w+\?/$1\/vmvnc\.reg\?/;
  unless($1) { # just to be safe
    return error_html($invalid_url);
  }
  $url =~ s/service=(\w+)/service=vnc_reg/;
  unless($1) { # just to be safe
    return error_html($invalid_url);
  }

  # Use filefield() to do escaping
  my $filefield = $q->filefield(-name => 'exe',
                                -size => 40,
                                -maxlength => 124
                               );

  my $html = <<EOF;
      <html>
      <head>
      <script language="vbscript">
      <!--
        Sub Redirect
          If document.f1.exe.value <> "" Then
            'MsgBox "$url" & "&exe=" + Escape(Document.f1.exe.value)
            Location.HRef = "$url" & "&exe=" + Escape(Document.f1.exe.value)
          End If
        End Sub

        Sub OnloadHandler
          MsgBox "To set up your browser to handle VNC connections to virtual machines, please specify where the VNC Viewer executable (vncviewer.exe) is located and open the file vmvnc.reg that will be downloaded right afterwards." & vbNewLine & "(By default, the viewer is installed at C:\\Program Files\\RealVNC\\vncviewer.exe)", 0, "VMware"
          Document.f1.exe.click
          Redirect()
        End Sub
      -->
      </script>
      </head>
      <body onload="OnloadHandler()">
        <form method=post name=f1>
          <input type=file name=exe size=40 maxlength=124>
          <input type=button value="Download vmvnc.reg" onclick="Redirect()">
          <font face=\"arial,helvetica\" size=2>
          <br>
          </font><br>
        </form>
      </body>
      </html>
EOF
}

sub vnc_mime_netscape_html {
  my $q = shift;

  return <<EOF;
      <html>
      <body>
      <p>
      To set up your browser for VNC connections to virtual machines,
      add (or edit) the following new (or existing) MIME type to your 
      Netscape browser:
            
      <pre>
      Description of type: VMWare VNC Launcher
      File Extension:      vmvnc
      MIME type:           application/vmvnc
      Application to use:  &#34;&#60;path to vncviewer.exe*&#62;&#34;&nbsp;&#34;-config&#34;&nbsp;&#34;&#37;1&#34;

      * The default location is C:&#92;Program Files&#92;RealVNC&#92;vncviewer.exe
      </pre>

      <p>
      To add (or edit) the MIME type, go to Edit->Preferences->(Helper) Applications, and click on "New Type..." (or "Edit...")

      <p>
      For more documentation, <a href="/vmware/doc/en/vmvnc.html">click here</a>.
      </body>
      </html>
EOF
}

sub vnc_reg_handler {
  my $q = shift;
  my ($browser, $version) = ($q->param("browser"), $q->param("version"));

  # First, make sure it's IE
  unless($browser eq 'ie') { # shouldn't happen under normal op
    return error_handler('Browser [$browser] not supported for this operation.');
  }

  # Lastly, we need to make sure the right exe is specified
  my $exe = $q->param("exe");
  # We might want to only check for .exe (rather than vncviewer.exe) since
  # the user might rename the executable.
  # For now, we try to match vncviewer.exe
  if($exe !~ /vncviewer\.exe/) {
    return error_handler("This is not a valid path to vncviewer.exe: [$exe]");
  }
  $exe =~ s/\\/\\\\/g;

  my $reg = <<EOF;
Windows Registry Editor Version 5.00

[HKEY_CLASSES_ROOT\\.vmvnc]
"Content-Type"="application/vmvnc"
@="VMware.RC.VNC.Config"

[HKEY_CLASSES_ROOT\\VMware.RC.VNC.Config]
@="VMware VNC Launcher"

[HKEY_CLASSES_ROOT\\VMware.RC.VNC.Config\\shell]
@="Open"

[HKEY_CLASSES_ROOT\\VMware.RC.VNC.Config\\shell\\Open]
@="&Open"

[HKEY_CLASSES_ROOT\\VMware.RC.VNC.Config\\shell\\Open\\command]
@="\\"$exe\\" \\"-config\\" \\"%1\\""

[HKEY_CLASSES_ROOT\\MIME\\Database\\Content Type\\application/vmvnc]
"Extension"=".vmvnc"

EOF
  return($reg,
         "Cache-Control" => "must-revalidate");
}

sub vnc_get_server_port {
  my $ip = shift;
  my ($sock, $port, $msg);

  # XXX Hack: Find out on which port the service is running by polling.
  # It does not cover cases where the server decides to use ports outside
  # the range.  The better approach is to build service detection into 
  # serverd so that MUI can get the right info with $vm->get():
  # 
  #   $vm->get("tools.rc.vncserver"); 
  #
  # -- mgu
  #

  for $port (5900..5999) { # 5900 is for VNCWin
    $sock = new IO::Socket::INET(PeerAddr => $ip,
                                 PeerPort => $port,
                                 Proto => 'tcp',
                                 );
    next unless $sock;
    $msg = "";
    $sock->recv($msg, 12);
    $sock->close;
    if($msg =~ /^RFB \d\d\d\.\d\d\d/) { # the 1st message of the handshake.
      return $port;
    }
  }
  return undef;
}


#sub pcanywhere {
#}

sub error_handler {
  my ($error_message) = @_;
  my $html = error_html($error_message);
  return pack_html($html);
}

1;
