#!/usr/bin/perl -w

#############################################################
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
#############################################################
#
# HStatus.pm
# 
# A script that displays in HTML the type of data that you have.
package VMware::Management::SS::HStatus;

use strict;
use VMware::HConfig::PCIDevice;
use VMware::HConfig::SSHostRemote;
use VMware::HConfig::Constant qw($gVmkDeviceType_hba);
use VMware::ExtHelpers qw(&p_number $gDEBUG &getWarnings &clearWarnings $gPanicCallback);

use VMware::Management::Util qw( trace
                                 file_to_string
                                 string_to_file
                                 hash_credentials );
use VMware::Management::Session;

use VMware::Control::Profiler;

use Carp qw(confess);
$SIG{__DIE__} = \&confess;
use CGI;

use VMware::Management::SS::ESXConfig;
my $htmlTitle = "Host Status";

#$gDEBUG = 2;
my $p = VMware::Control::Profiler::getInstance("mui");

sub PanicCallbackHTML(@) {
    my (@error) = @_;
    if ( $error[0] ) {
       print toHTMLPanic($htmlTitle, $error[0]);
    } else {
       print toHTMLPanic($htmlTitle, "");
    }
}

sub toVMnixModulesHTML($$) {
    my $stub = $p->profile();
    my ($cgi, $host) = @_;
    my $s = "";

    $s .= 
        ("<hr>\n" .
         "<a name=\"vmnixMod\"></a>\n" .
         "\n" .
         "<h3>Console Modules</h3>\n"
         );

    $s .= toHTMLTableHeader("Module", "Loaded", "Size", "Used", "Aliases");

    my ($vmnixMods) = $host->getVMnixModules();
    my $vmMod;

    foreach $vmMod (@$vmnixMods) {
	my $modname = $vmMod->{MODULE};
        my $modloaded = $vmMod->{LOADED};
	my $modsize = $vmMod->{SIZE};
	my $modused = $vmMod->{USED};
        my $modaliases = $vmMod->{ALIASES};

        $s .= toHTMLTableRow($modname,
                             [ $modloaded ? "Yes" : "No", "align='center'"],
                             [ p_number($modsize), "align='right'"], 
                             [ $modused, "align='center'"],
                             @$modaliases ? join(", ", @$modaliases) : "&nbsp;");
    }

    $s .= toHTMLTableFooter();
    return($s);
}

sub toVMKernelModulesHTML($$) {
    my $stub = $p->profile();
    my ($cgi, $host) = @_;
    my $s = "";

    $s .= 
	("<hr>\n" .
	 "<a name=\"vmkernelMod\"></a>\n" .
	 "\n" .
	 "<h3>VMkernel Modules Loaded</h3>\n"
         );

    $s .= toHTMLTableHeader("ID", "Module", "Size");

    my ($vmkernelMods) = $host->getVMKernelModules();
    my $vmkMod;

    foreach $vmkMod (@$vmkernelMods) {
       $s .= toHTMLTableRow($vmkMod->{ID},
                            $vmkMod->{MODULE},
                            p_number($vmkMod->{SIZE}));
    }
    $s .= toHTMLTableFooter();

    return($s);
}

sub toDirMapHTML($$) {
    my $stub = $p->profile();
    my ($cgi, $host) = @_;
    my $s = "";

    $s .= 
	("<hr>\n" .
	 "<a name=\"dirMap\"></a>\n" .
	 "\n" .
	 "<h3>Mounted Directories</h3>\n");
    
    $s .= toHTMLTableHeader("Mount Point", "Device", "Type");

    my (%dirMap) = %{ $host->getDirMapping() };
    my $mt_point;

    my @keys = sort { $a cmp $b } keys(%dirMap);

    foreach $mt_point (@keys) {
	my $device = $dirMap{$mt_point}->[0];
	my $type = $dirMap{$mt_point}->[1];

	$s .= toHTMLTableRow($mt_point, $device, $type);
    }
    $s .= toHTMLTableFooter();

    return($s);
}

sub toPCIDevicesHTML($$) {
    my $stub = $p->profile();
    my ($cgi, $host) = @_;;
    my $s = "";

    $s .= 
	("<hr>\n" .
	 "<a name=\"pciDevices\"></a>\n" .
	 "\n" .
	 "<h3>PCI Devices Found</h3>\n" .
	 "\n"
	 );

    my(@devs) = $host->getPCIDeviceList('')->getPCIDevices();
    if( $#devs == -1 ) {
	$s .= "<p class='txt'><i>No devices found.</i></p>\n";
    } else {
	$s .= toHTMLTableHeader("Bus", "Device", "Function", "Class", 
                                "Name", "Owner", "IRQ", "Driver");

	my $dev;
	foreach $dev (@devs) {
	    my $bus = $dev->getBus();
	    my $device = $dev->getDevice();
	    my $function = $dev->getFunction();
	    my $class = sprintf("0x%06x", $dev->getClass());
	    my $name = $dev->getClassName() . ': ' . $dev->getName();
	    my $owner = $dev->getOwnerString();
	    my $irq = (defined($dev->getIrq()) ? ("IRQ " . $dev->getIrq()) : "&nbsp");
	    my $vendor = sprintf("0x%04x", $dev->getVendorID());
	    my $devid = sprintf("0x%04x", $dev->getDeviceID());

            my $driver = "";
	    my @drivers = @ { $dev->getDrivers() };
	    my @driverNames = @ { $dev->getDriverNames() };
	    my $i;
	    if( $#drivers == -1 ) {
		$driver = "&nbsp";
	    } else {
		for($i = 0; $i <= $#drivers; $i++ ) {
		    if($i > 0) {
			$driver .= ", ";
		    }
		    $driver .= $drivers[$i] . " (" . $driverNames[$i] . ")";
		}
	    }

	    $s .= toHTMLTableRow($bus, $device, $function, $class,
                                 $name, $owner, $irq, $driver);
	    
	}

        $s .= toHTMLTableFooter();
    } # if( $#devs == -1 )

    return($s);
}

sub toSCSIAdapterHTML($$) {
    my $stub = $p->profile();
    my ($cgi, $host) = @_;
    my $s = "";

    if( !$host->isVMnix() ) {
	return($s);
    }

    $s .= 
	("<a name=\"SCSIAdapter\"></a>\n" .
	 "<hr>\n" .
	 "\n" .
	 "<h3>Host Bus Adapters Available (Virtual Machines only)</h3>\n" .
	 "\n"
	 );

    my @adapters = @{ $host->getVMSCSIAdapters() };
    if( $#adapters == -1 ) {
	$s .= "<p class='txt'><i>No $gVmkDeviceType_hba adapters found!</i></p>\n";
	return($s);
    } 
    my $a;
    foreach $a (@adapters) {
	my $shared = "NOT shared.";    
	if( $a->isShared() ) {
	    $shared = "shared with the Console Operating System.";
	}
	
	my $devinfo = "";
	if( defined($a->getPCIDevice()) ) {
	    $devinfo = "Host Bus Adapter maps to PCI device <b>" . 
		$a->getPCIDevice()->getHash() . 
		"</b> (" . $a->getPCIDevice()->getName() . ")";
	}
        my $wwpninfo = "";
        if ($a->getWWPN() ne "") {
            $wwpninfo = "  WWPN is " . $a->getWWPN() . "\n";
        }
         
        my $sanDeviceInfo = " SAN Devices on this adapter are";
        my $sanDevice;
        foreach $sanDevice (@{$a->getSanDevices()}) {
           my $target;
           $sanDeviceInfo .= " { ";
           for $target ( keys %$sanDevice ) {
              $sanDeviceInfo .= "$target=$sanDevice->{$target} ";
           }
           $sanDeviceInfo .= " }";
        }
	
	my $vmkname = $a->getVmkName();
	$s .= 
	    ("<p class='txt'>Host Bus Adapter <b>$vmkname</b> is $shared</p>\n" . 
	     "<p class='txt'>$devinfo</p>\n" .
	     "<p class='txt'>$wwpninfo</p>\n" .
             "<p class='txt'>$sanDeviceInfo</p>\n".
	     "<p class='txt'>It has the following disks, CDROM, Tape drives, and other devices:</p>\n\n"
	     );

	my @allDisks = @{ $a->getDisks() };

	if( $#allDisks == -1 ) {
	    $s .= "<i>No disks found.</i>\n";
	} else {
	    my $d;
	    foreach $d (@allDisks) {
		$s .= toDiskHTML($d);
		$s .= 
		    ("\n" .
		     "\n"
		     );
	    }
	}

	my $type;
	foreach $type ($a->getNonDiskDeviceTypes()) {
	    my @devices = $a->getNonDiskDevicesByType($type);
	    
	    if( $#devices > -1 ) {
		$s .= toNonDiskHTML($type, \@devices);
	    }
	}
    }
    
    return($s);
}

sub toDiskHTML($) {
    my $stub = $p->profile();
    my ($disk) = @_;
    
    my $s = "";
    
    my $name = $disk->getVmkName();
    my $geo_cyl = p_number($disk->getGeoCylinder());
    my $geo_head = p_number($disk->getGeoHead());
    my $geo_sect = p_number($disk->getGeoSector());
    my $mbSize = p_number($disk->getTotalSectors() / 2 / 1024);

    # Print the disk name
    if( $disk->hasVmnixDevName() ) {
	my $vmnix_name = $disk->getVmnixDevName();
	$s .=
	    ("      <h4>Disk $name ($vmnix_name)</h4>\n" .
	     "\n");
    } else {
	$s .=
	    ("      <h4>Disk $name</h4>\n" .
	     "\n");
    }
    $s .= qq(<p class='txt'>
             $mbSize MB total size ($geo_cyl cylinders, $geo_head heads, $geo_sect sectors)
             </p>);

    
    my @partnums = $disk->getPartNums();
    if( $#partnums > -1 ) {
	$s .= toHTMLTableHeader("Partition", "Type", "Start Cylinder",
                                "End Cylinder", "Size (MB)", "System");
	
	# Print out information about each partition.
	my $part;
	foreach $part (@partnums) {
	    my $part_name = "$name:$part";
	    my $part_type = sprintf("%02x", $disk->getPartType($part));
	    my $part_scyl = $disk->getPartStartCyl($part);
	    my $part_ecyl = $disk->getPartEndCyl($part);
	    my $part_sect = $disk->getPartSectors($part);
	    my $part_mb_size = sprintf("%d", $part_sect / 2 / 1024);
	    my $part_type_name = $disk->getPartTypeName($part);
	    
	    $s .= toHTMLTableRow($part_name, 
                                 [ "0x$part_type", "align='center'" ],
                                 [ p_number($part_scyl), "align='right'" ],
                                 [ p_number($part_ecyl), "align='right'" ],
                                 [ p_number($part_mb_size) . " MB", "align='right'" ],
                                 $part_type_name);
	} # foreach $part (@partnums)

        $s .= toHTMLTableFooter();
    } else {
	$s .= "      <p class='txt'><i>No partitions found</i></p>\n";
    }

    my $policy = $disk->getMultipathPolicy();
    my $active = $disk->getMultipathActive();
    my $prefer = $disk->getMultipathPrefer();
    my @paths = $disk->getMultipathPaths();

    $s .= "      <p class='txt'>Multipathing information: policy is $policy</p>\n";

    $s .= toHTMLTableHeader("Path", "Status", "Active", "Preferred");

    foreach my $p (@paths) {
       my $path = $p->{PATH};
       my $status = $p->{STATUS};
       my $isActive = $active eq $path ? "yes" : "no";
       my $isPrefer = $prefer eq $path ? "yes" : "no";

       $s .= toHTMLTableRow($path, 
                            [ "$status", "align='center'" ],
                            [ "$isActive", "align='center'" ],
                            [ "$isPrefer", "align='center'" ]);
    }

    $s .= toHTMLTableFooter();

    return($s);
}


sub toNonDiskHTML($$) {
    my $stub = $p->profile();
    my ($type, $devices) = @_;
    
    my $s = "";
    
    $s .=
	("      <h4>$type Device(s):</h4>\n" .
	 "      <ul>\n"
	 );

    my $dev;
    foreach $dev (@$devices) {
	$s .= "        <li class='txt'>$dev</li>\n";
    }

    $s .= "      </ul>\n\n";
    return($s);
}
		    

sub toNetAdapterHTML($$) {
    my $stub = $p->profile();
    my ($cgi, $host) = @_;
    my $s = "";

    if( !$host->isVMnix() ) {
	return($s);
    }

    $s .= 
	("<a name=\"Net\"></a>\n" .
	 "<hr>\n" .
	 "\n" .
	 "<h3>Network Adapters Available (Virtual Machines only)</h3>\n" .
	 "\n"
	 );

    my @adapters = @{ $host->getVMNetAdapters() };
    if( $#adapters == -1 ) {
	$s .= "<p class='txt'><i>No Network adapters found!</i></p>\n";
	return($s);
    } 

    $s .= toHTMLTableHeader("Adapter", "PCI Info", "Speed", "Clustering",
                            "NIC Team", "MAC address");

    my $a;
    foreach $a (@adapters) {
       my ($bus, $dev, $func) = $host->getVMNetAdapterPCIInfo($a);
       my $clustering = $host->getVMNetAdapterClustering($a);
       my $speed = $host->getVMNetAdapterSpeed($a) || "Link down";
       my $nicteam = $host->getVMNetAdapterNICTeam($a);
       my @macs = @{ $host->getVMNetAdapterMAC($a) };

       $s .= toHTMLTableRow($a,
                            [ "$bus:$dev:$func", "align='center'" ],
                            [ $speed, "align='center'" ],
                            [ $clustering, "align='center'" ],
                            [ $nicteam, "align='center'" ],
                            join(", ", @macs));
    }

    $s .= toHTMLTableFooter();

    return($s);
}

sub toNetBondAdapterHTML($$) {
    my $stub = $p->profile();
    my ($cgi, $host) = @_;
    my $s = "";

    if( !$host->isVMnix() ) {
	return($s);
    }

    $s .= 
	("<a name=\"NetBond\"></a>\n" .
	 "<hr>\n" .
	 "\n" .
	 "<h3>Network Bond Adapters Available (for NIC Teaming)</h3>\n" .
	 "\n"
	 );

    my @adapters = @{ $host->getVMNetBondAdapters() };
    if( $#adapters == -1 ) {
	$s .= "<p class='txt'><i>No Network Bond adapters found!</i></p>\n";
	return($s);
    } 

    $s .= toHTMLTableHeader("Bond Adapter", "Slaves", "MAC address");

    my $a;
    foreach $a (@adapters) {
       my @slaves = @{ $host->getVMNetBondAdapterSlaves($a) };
       my @macs = @{ $host->getVMNetBondAdapterMAC($a) };

       $s .= toHTMLTableRow($a,
                            join(", ", @slaves),
                            join(", ", @macs));
    }

    $s .= toHTMLTableFooter();

    return($s);
}

sub toNetTcpipHTML($$) {
    my $stub = $p->profile();
    my ($cgi, $host) = @_;
    my $s = "";
    my %tcpipInfo;

    if( !$host->isVMnix() ) {
	return($s);
    }

    $s .= 
	("<a name=\"NetTcpip\"></a>\n" .
	 "<hr>\n" .
	 "\n" .
	 "<h3>Network TCPIP Adapters Available (for migration)</h3>\n" .
	 "\n"
	 );

    $s .= "<h4>Configured TCPIP Adapters</h4>\n\n";
    %tcpipInfo = %{ $host->getVMNetTcpipCfg() };
    if (scalar(keys(%tcpipInfo)) <= 0) {
       $s .= "<p class='txt'><i>No Configured Network TCPIP adapters found!</i></p>\n";
    } else {
       $s .= toHTMLTableHeader("TCPIP Adapter", "IP Address", "Gateway Address", "Interface");

       my $name;
       foreach $name (sort(keys(%tcpipInfo))) {
          my $ipAddr = $tcpipInfo{$name}->{IPADDR};
          my $gatewayAddr = $tcpipInfo{$name}->{GATEWAYADDR};
          my $nicName = $tcpipInfo{$name}->{NICNAME};
          
          $gatewayAddr = $gatewayAddr ? $gatewayAddr : "None";
          
          $s .= toHTMLTableRow($name, $ipAddr, $gatewayAddr, $nicName);
       }

       $s .= toHTMLTableFooter();
    }

    $s .= "<h4>Actual TCPIP Adapters</h4>\n\n";
    %tcpipInfo = %{ $host->getVMNetTcpip() };
    if (scalar(keys(%tcpipInfo)) <= 0) {
       $s .= "<p class='txt'><i>No Network TCPIP adapters found!</i></p>\n";
    } else {
       $s .= toHTMLTableHeader("TCPIP Adapter", "IP Address", "Gateway Address", "Interface");

       my $name;
       foreach $name (sort(keys(%tcpipInfo))) {
          my $ipAddr = $tcpipInfo{$name}->{IPADDR};
          my $gatewayAddr = $tcpipInfo{$name}->{GATEWAYADDR};
          my $nicName = $tcpipInfo{$name}->{NICNAME};
          
          $gatewayAddr = $gatewayAddr ? $gatewayAddr : "N/A";
          
          $s .= toHTMLTableRow($name, $ipAddr, $gatewayAddr, $nicName);
       }

       $s .= toHTMLTableFooter();
    }

    my $sync = $host->isVMNetTcpipSync();
    $s .= ("<p class='txt'><i>TCPIP configured and actual settings are " .
           ($sync ? "" : "NOT ") . "synchronized.</i></p>\n");

    return($s);
}

sub toMFSFilesystemHTML($$) {
    my $stub = $p->profile();
    my ($cgi, $host) = @_;
    my $s = "";

    if( !$host->isVMnix() ) {
	return($s);
    }

    $s .= 
	("<a name=\"MFS\"></a>\n" .
	 "<hr>\n" .
	 "\n" .
	 "<h3>VMFS File Systems Available (Virtual Machines only)</h3>\n" .
	 "\n"
	 );


    my @allMfs = @{ $host->getAllMFS() };
    if( $#allMfs == -1 ) {
	$s .= "<p class='txt'><i>No VMFS Filesystems found!</i></p>\n";
	return($s);
    }

    my $fs;
    foreach $fs (@allMfs) {
	my $partname = $fs->getVmkName();
	my $logname = $fs->getLogName();
	my $capacity = p_number($fs->getCapacity());
	my $freecap = p_number($fs->getFreeCapacity());
        my $version = $fs->getVersion();
        my $minorversion = $fs->getMinorVersion();
        my @extents = @{$fs->getExtents()};

	$s .= 
	    ("<p class='txt'>VMFS on partition <b>$partname</b>" .
             (($logname ne "") ? " with logical name <b>$logname</b>\n" : "\n") .
	     "is $capacity MB ($freecap MB free).  Version VMFS-$version.$minorversion.</p>\n");

        $s .= "<p class='txt'>Extents:</p>\n<ul>\n";
        foreach my $extent (@extents) {
           $s .= "  <li class='txt'>$extent\n";
        }
        $s .= "</ul>\n";

        $s .= "<p class='txt'>File list: </p>\n";
	$s .= toHTMLTableHeader("File Name", "Size",
                                #"Type"
                                );

	my $fileEntry;
	foreach $fileEntry (@{$fs->listFiles()}) {
	    my $name = $fileEntry->[0];
	    my $size = p_number($fileEntry->[1]);
	    my $type = $fileEntry->[2];
	    $s .= toHTMLTableRow($name, [ "$size MB", "align='right'" ],
                                 # $type,
                                 );
	}

	$s .= toHTMLTableFooter();
    }
    
    return($s);
}


sub toVmkDumpPartitionHTML($$) {
    my $stub = $p->profile();
    my ($cgi, $host) = @_;
    my $s = "";

    $s .= 
	("<a name=\"VmkDumpPart\"></a>\n" .
	 "<hr>\n" .
	 "\n" .
	 "<h3>Core Dump Partitions:</h3>\n" .
	 "<ul>\n" .
	 "\n"
	 );


    my @allpart = @{ $host->getVmkDumpPartitions() };
    if( $#allpart == -1 ) {
	$s .= "<p class='txt'><i>No core dump partitions found!</i></p>\n";
    }

    my $partname;
    foreach $partname (@allpart) {
	if( $partname =~ /^(scsi\d+:\d+):(\d+)$/ ) {
	    my $diskname = $1;
	    my $partnum = $2;
	    my $disk = $host->getVMSCSIDiskByName($diskname);
	    my $partDevName = $disk->getPartBestDevName($partnum);
	    $s .= "<li class='txt'>$partname ($partDevName)\n";
	} else {
	    $s .= "<li class='txt'>$partname\n";
	}
    }

    $s .= "</ul>\n";
    
    return($s);
}

sub toRunningWorldsHTML($$) {
    my $stub = $p->profile();
    my ($cgi, $host) = @_;
    my $s = "";

    $s .= 
	("<a name=\"RunningWorlds\"></a>\n" .
	 "<hr>\n" .
	 "\n" .
	 "<h3>Running VMM Worlds:</h3>\n" .
	 "<ul>\n" .
	 "\n"
	 );

    my @worlds = @{ $host->getRunningVmmWorlds() };
    if( $#worlds == -1 ) {
	$s .= "<p class='txt'><i>No running vmm worlds.</i></p>\n";
    }

    my $world;
    foreach $world (@worlds) {
	$s .= "<li class='txt'>$world\n";
    }

    $s .= "</ul>\n";
    
    return($s);
}

sub toStatusHTML($$) {
    my $stub = $p->profile();
    my($cgi, $host) = @_;
    my $s = "";

    my $url_self = $cgi->url(-relative=>1);

    my $vmnix_status = "not running";
    if( $host->isVMnix() ) {
	$vmnix_status = "running";
    }

    my $vmnixMod_status = "not loaded";
    if( $host->isVMnixModule() ) {
	$vmnixMod_status = "loaded";
    }

    my $vmkernel_status = "not loaded";
    if( $host->isVMKernel() ) {
	$vmkernel_status = "loaded";
    }

    my $kernel_version = $host->getKernelVersion();

    my ($licenseCanRun, $message) = $host->checkCanRunLicense();
    my $manual = $host->isManualConfig();

    # Perform a check for duplicate VMFS names
    $host->checkDuplicateVmfsNames($host->getAllMFS());

    $s =
	("<p class='txt'>The following types of system information is available:</p>\n" .
	 "<ul>\n" .
	 "<li class='txt'><a href=\"#Kernel\">Kernel Information</a>\n" .
	 "<li class='txt'><a href=\"#CPU\">CPU Information</a>\n" .
	 "<li class='txt'><a href=\"#Memory\">Memory Information</a>\n" .
	 "<li class='txt'><a href=\"#vmnixMod\">Console Modules</a>\n" .
	 "<li class='txt'><a href=\"#vmkernelMod\">VMkernel Modules</a>\n" .
	 "<li class='txt'><a href=\"#dirMap\">Mounted Directories</a>\n" .
	 "<li class='txt'><a href=\"#pciDevices\">PCI Devices</a>\n" .
	 "<li class='txt'><a href=\"#SCSIAdapter\">Host Bus Adapters</a>\n" .
	 "<li class='txt'><a href=\"#Net\">Network Adapters</a>\n" .
	 "<li class='txt'><a href=\"#NetBond\">Network Bond Adapters</a>\n" .
	 "<li class='txt'><a href=\"#NetTcpip\">TCPIP Adapters</a>\n" .
	 "<li class='txt'><a href=\"#MFS\">VMFS File system info</a>\n" .
	 "<li class='txt'><a href=\"#VmkDumpPart\">Core Dump Partition</a>\n" .
	 "<li class='txt'><a href=\"#RunningWorlds\">Running VMM Worlds</a>\n" .
	 "</ul>\n" .
	 "\n" .
	 "<hr>\n" .
	 "\n" .
	 "<a name=\"Kernel\"></a>\n" .
	 "<h3>Kernel Information</h3>\n" .
	 "<ol>\n" . 
	 "  <li class='txt'>VMnix Kernel is <b>$vmnix_status</b>\n" .
	 "  <li class='txt'>VMnix Module is <b>$vmnixMod_status</b>\n" .
	 "  <li class='txt'>VMkernel is <b>$vmkernel_status</b>\n" .
	 "</ol>\n"
	 );

    $s .= "<p class='txt'>Have a valid product license: <b>" . ($licenseCanRun == 1 ? "Yes" : "No") . "</b></p>\n";
    $s .= "<p class='txt'>Modules are loaded manually: <b>" . ($manual == 1 ? "Yes" : "No") . "</b></p>\n";

    # CPU information
    $s .=
	("<a name=\"CPU\"></a>\n" .
	 "<hr>\n" .
	 "\n" .
	 "<h3>CPU Information</h3>\n"
	 );

    my $numCPU = $host->getNumCPUs();
    $s .= "<p class='txt'>Number of CPUs used by vmkernel: <b>$numCPU</b></p>\n";

    # Memory information
    $s .= 
	("<a name=\"Memory\"></a>\n" .
	 "<hr>\n" .
	 "\n" .
	 "<h3>Memory Information</h3>\n"
	 );

    $s .= toHTMLTableHeader();
    
    my $totalMem = sprintf("%d", $host->getTotalMemSize() / 1024 );
    $s .= toHTMLTableRow("<b>Total Memory</b>", 
                         [ p_number($totalMem) . " MB", "align='right'"] );

    if( $host->isVMnix() ) {
	my $vmnixMem = sprintf("%d", $host->getVmnixMemSize() / 1024 );
        $s .= toHTMLTableRow("<b>Total VMnix Memory</b>", 
                             [ p_number($vmnixMem) . " MB", "align='right'"] );

	my $vmkMem = sprintf("%d", $host->getVmkMemSize() / 1024 );
	$s .= toHTMLTableRow("<b>Total VMKernel Memory</b>", 
                             [ p_number($vmkMem) . " MB", "align='right'" ]);
        
	my $vmNewVMAvailMem = sprintf("%d", $host->getVMAvailMemSize() / 1024 );
	$s .= toHTMLTableRow("<b>Memory Available to New Virtual Machines</b>",
                             [ p_number($vmNewVMAvailMem) . " MB", "align='right'" ]);
    }

    $s .= toHTMLTableFooter();

    $s .= toVMnixModulesHTML($cgi, $host);
    $s .= toVMKernelModulesHTML($cgi, $host);
    $s .= toDirMapHTML($cgi, $host);
    $s .= toPCIDevicesHTML($cgi, $host);
    $s .= toSCSIAdapterHTML($cgi, $host);
    $s .= toNetAdapterHTML($cgi, $host);
    $s .= toNetBondAdapterHTML($cgi, $host);
    $s .= toNetTcpipHTML($cgi, $host);
    $s .= toMFSFilesystemHTML($cgi, $host);
    $s .= toVmkDumpPartitionHTML($cgi, $host);
    $s .= toRunningWorldsHTML($cgi, $host);

    $s .= 
	("</td>\n" .
	 "</tr>\n" .
	 "</table>\n" .
	 "</body>\n" .
	 "</html>\n"
	 );

    return($s);
}

sub toNewMUIHeader($) {
   my $title = shift;
   return <<END_HTML_HEADER;
<html>
<head>
  <title>$title</title>
  <link rel="stylesheet"
        href="/vmware/themes/default/style.css"
        type="text/css">
  <style type="text/css">
    a:hover { background-color:yellow; }
  </style>
  <script src="/vmware/src/init.js" type="text/javascript"></script>
  <script src="/vmware/src/xuaLib.js" type="text/javascript"></script>
  <script type="text/javascript">
    <!--
      function aggressiveUpdate() {
        window.setTimeout('window.location.reload( )', 1000 * 60);
      }
      function openHelp(ref,w,h) {
         window.open(ref,
                     "vmware_mui_help",
                     "location,menubar,resizable,scrollbars,status,toolbar," +
                     "width=" + w + ",height=" + h);
         aggressiveUpdate();
      }
    // -->
  </script>
</head>
<body bgcolor="#ffffff" topmargin="6" leftmargin="6" marginwidth="6" marginheight="6"
      link="#3366cc" vlink="#663300">
<nobr><small><small>
<script>document.write("Last Updated: " + new Date().toString());</script>
</SMALL></small></nobr>
END_HTML_HEADER
}

sub handler {
    my $stub = $p->profile();
    # Delete the PATH to get by the tainted security requirements of Perl
    delete($ENV{'PATH'}); 

    umask 022;   # Create files that are not writable by group

    &clearWarnings();
    
    my $cgi = new CGI( shift( ) );
    my %credentials = hash_credentials( );
    
    my $user = $credentials{ "l" };
    my $password = $credentials{ "m" };

    my $ctx = &VMware::Management::Session::get_temp_context();
    if (!defined($ctx)) {
       return (&VMware::Management::Util::error_page("Could not get temporary context.",
                                                     __PACKAGE__, __LINE__));
    }
    
    my $mui = $cgi->param("mui"); # will be set as 'new' if the new MUI is being used
    my $s;
    # get HTML Header Content
    if ($mui eq "new") {
       $s = toNewMUIHeader($htmlTitle);
    } else {
       $s = toHTMLHeader($htmlTitle, $user) . toHTMLPath(\@htmlPath, $htmlTitle);
    }

    my $host = VMware::HConfig::SSHostRemote->new(undef, 0, $user, $password, $ctx);

    if(!defined($host)) {
       $s .= toHTMLHostFailure($htmlTitle);
       goto footer;
    }
    
    $s .=
        toHTMLSystemInfo($host) . 
        toHTMLWarnings() .
        toStatusHTML($cgi, $host);

footer:
    if (!($mui eq "new")) {
      $s .= toHTMLFooter();
    }
    return ($s, %MimeHeaders);
}

1;
