# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::SS::NetworkConnections;
# File: VMware/Management/SS/NetworkConnections.pm

use strict;

use VMware::HConfig::SSHostRemote ();
use VMware::HConfig::Constant qw( @gNICSettings
                                  %gNICConfigurationOptions
                                  @gNICClusteringDisabled );
use VMware::Management::SS::ESXConfig;
use VMware::Management::Util ();
use VMware::Management::VmdbUtil ();

use CGI();
my $q;


# ---------------------------------------------------------------------------

sub handler {
   $q = new CGI(shift());

   my %credentials = VMware::Management::Util::hash_credentials();
   my $user = $credentials{"l"};
   my $pass = $credentials{"m"};
   my $ctx = &VMware::Management::Session::get_temp_context();
   if (!defined($ctx)) {
      return (&VMware::Management::Util::error_page("Could not get temporary context.",
                                                    __PACKAGE__, __LINE__));
   }
   my $host = VMware::HConfig::SSHostRemote->new(undef, 0, $user, $pass, $ctx);

   if (! defined($host)) {
      my $data = error("Could not create remote host object.");
      return html($data);
   }

   if (! $host->isVMKernel()) {
      my $data = error("Could not find network connections: " .
                       "The VMkernel is not loaded.");
      return html($data);
   }

   if ($q->param("op") =~ /^(save)$/i) {
      return save($host);
   }

   return data($host);
}


# ---------------------------------------------------------------------------

sub data {
   my $host = shift();
   my $data = shift() || "";

   my @vmnics = vmnics($host);

   # Generate JavaScript code for the complete list of supported NIC speed and
   # duplex settings.
   $data .= qq{var nicSpdDplxs = ["} . join('","', @gNICSettings) . qq{"];$/};

   # Generate JavaScript code for the list of driver-specific speed and duplex
   # settings.
   my $driverData = "";
   for my $nic (@vmnics) {
      my $driver = $nic->{"drvr"};
      $driverData .= ($driverData ? "," : "") . qq{"$driver":};

      if (! defined($gNICConfigurationOptions{$driver})) {
         $driverData .= "null";
      } else {
         my $spdDplxData = "";

         for (@{$gNICConfigurationOptions{$driver}}) {
            if (defined($_)) {
               $spdDplxData .= ($spdDplxData ? "," : "") . qq{"$_"};
            }
         }
         $driverData .= qq{[$spdDplxData]};
      }
   }
   $data .= "var drvrSpdDplx = {$driverData};$/";

   # Generate JavaScript code for the complete list of NIC teams
   my @nicTeams = @{$host->getVMNetBondAdapters()};
   unshift @nicTeams, "none";
   $data .= qq{var nicTeams = ["} . join('","', @nicTeams) . qq{"];$/};

   # Network interface card prototype
   $data .= "function Nic(id,dnm,vnm,drvr,cfgSpd,curSpd,clstr,team,active,bus,dev,fun){" .
      "this.id=id;" .         # Device ID PCI bus:dev.fun
      "this.dnm=dnm;" .       # Device name (manufacturer, model, etc.)
      "this.vnm=vnm;" .       # Virtual device name (e.g., vmnic0)
      "this.drvr=drvr;" .     # Name of selected driver object file
      "this.cfgSpd=cfgSpd;" . # The adapter's configured speed and duplex
      "this.curSpd=curSpd;" . # The adapter's actual speed and duplex
      "this.clstr=clstr;" .   # Interrupt clustering, or null if disallowed
      "this.team=team;" .     # NIC team, or null if not in a team
      "this.active=active;" . # Boolean indicating whether NIC is is use
      "this.bus=bus;" .       # PCI bus number
      "this.dev=dev;" .       # PCI device number
      "this.fun=fun;}$/";     # PCI function number

   $data .= "var nics=new Array();$/";

   # Generate JavaScript code for each NIC.
   for my $nic (@vmnics) {
      $data .= qq{nics.push(} . nicData($nic) . qq{);$/};
   }

   $data .= "var nicNames=" . nicNameData($host) . ";";

   return html($data);
}


# ---------------------------------------------------------------------------

sub nicData {
   my $nic = shift();

   return qq[new Nic("$nic->{id}",] .
          qq["$nic->{dnm}","$nic->{vnm}","$nic->{drvr}",] .
          qq[$nic->{cfgSpd},"$nic->{curSpd}",$nic->{clstr},] .
          qq["$nic->{team}",$nic->{active},$nic->{bus},] .
          qq[$nic->{dev},$nic->{fun})];
}


# ---------------------------------------------------------------------------
#
# sortByNetworkType--
#
#      Compare network device types ensuring that bonds come before vmnics and
#      that vmnics come before vmnets.
#
# ---------------------------------------------------------------------------

sub sortByNetworkType($$) {
   if ($a =~ /^bond/ && $b =~ /^(vmnic|vmnet)/) {
      return -1;
   }

   if ($a =~ /^vmnic/ && $b =~ /^vmnet/) {
      return -1;
   }

   if ($a =~ /^vmnic/ && $b =~ /^bond/) {
      return 1;
   }


   if ($a =~ /^vmnet/ && $b =~ /^(bond|vmnic)/) {
      return 1;
   }

   return $a cmp $b;
}


# ---------------------------------------------------------------------------
# {vmnic0:name,...,bondN:name}

sub nicNameData {
   my $host = shift();
   my $ctx = &VMware::Management::Session::get_temp_context();
   my $data = "";

   my %nicNames;

   my $path = "/host2/#_mui/config/network/#";
   while (defined($path = $ctx->GetNextSibling($path))) {
      my $dev = $ctx->GetString("${path}device");
      my $net = $ctx->GetString("${path}network");

      $nicNames{$dev} = $net;
      VMware::Management::Util::trace("Device: $dev <-> Network: $net", "debug");
   }

   my $nicData = "";
   for (sort sortByNetworkType keys(%nicNames)) {
      my $net = jsStringLiteral($nicNames{$_});
      $nicData .= ($nicData ne "" ? "," : "");
      $nicData .= qq{"$_":$net};
   }
   $data .= "{$nicData}";

   VMware::Management::Util::trace("$data", "debug");

   return $data;
}


# ---------------------------------------------------------------------------

sub setNicNames {
   my $host = shift();
   my $ctx = &VMware::Management::Session::get_temp_context();
   my ($ok, $trnCtx, $reqPath);
   my $data = "";

   VMware::Management::Util::trace("Removing existing netmap devices...", "debug");

   # Track devices that we've already tried to handle to avoid any possibility
   # of an infinite loop.
   my %handled;

   # Handle existing NIC names.
   my $path = "/host2/#_mui/config/network/#";
   while (defined($path = $ctx->GetNextSibling($path))) {
      my $dev = $ctx->GetString("${path}device");
      my $old = $ctx->GetString("${path}network");
      my $net = $q->param("$dev:net");

      if (! $handled{$dev} && (! defined($net) || '' eq $net)) {
         VMware::Management::Util::trace("Netmap device exists: $dev >$old<", "debug");

         # Track devices that we've already tried to handle to avoid any
         # possibility of an infinite loop.
         $q->delete("$dev:net");
         $handled{$dev} = 1;

         # T1(removeDevice) BC {
         #    T1(in) BC {
         #       T2(network, T_STR);
         #       T2(device, T_STR);
         #    } EC;
         # } EC;

         $trnCtx = &VMware::Management::Session::get_temp_context();
         $trnCtx->BeginTransaction();

         $reqPath = $trnCtx->NewArrayIndex("/host2/#_mui/cmd/##");
         $reqPath = $trnCtx->GetAbsPath($reqPath);
         my $reqPathCp = $reqPath;
         $trnCtx->SetString("${reqPath}op", "removeDevice");
         $trnCtx->SetString("${reqPath}user",
                            &VMware::Management::Session::get_usertoken());
         $reqPath = $trnCtx->GetAbsPath("${reqPath}op/removeDevice/");

         $trnCtx->SetString("${reqPath}in/network", $old);
         $trnCtx->SetString("${reqPath}in/device", $dev);

         my ($ok, $msg) = VMware::Management::VmdbUtil::vmdbEndCmd($trnCtx,
                                                                   $reqPath);

         if (! $ok) {
            $data .= error(qq{Network Connections: $msg. } .
                           qq{Failed to remove mapping: "$dev" ~ "$old".});
         } else {
            # Be sure to start at the beginning since the /host2/#_mui/config/network
            # tree will be repopulated after the transaction completes
            # successfully.
            $path = "/host2/#_mui/config/network/#";
            VMware::Management::Util::trace("Removed netmap device: $dev >$net<", "debug");
         }

         $trnCtx->LocalArrayUnset($reqPathCp, 1);
      }
   }

   VMware::Management::Util::trace("Modifying existing netmap devices...", "debug");

   $path = "/host2/#_mui/config/network/#";
   while (defined($path = $ctx->GetNextSibling($path))) {
      my $dev = $ctx->GetString("${path}device");
      my $old = $ctx->GetString("${path}network");
      my $net = $q->param("$dev:net");

      if (! $handled{$dev} && (defined($net) && $net ne "")) {
         VMware::Management::Util::trace("Netmap device exists: $dev >$old<", "debug");

         # Track devices that we've already tried to handle to avoid any
         # possibility of an infinite loop.
         $q->delete("$dev:net");
         $handled{$dev} = 1;

         if ($net eq $old) {
            VMware::Management::Util::trace("Existing netmap device unchanged: $dev >$old< : >$net<", "debug");
            next;
         }

         # T1(setNetwork) BC {
         #    T1(in) BC {
         #       T2(oldNetwork, T_STR);
         #       T2(newNetwork, T_STR);
         #       T2(newDevice, T_STR);
         #    } EC;
         # } EC;

         $trnCtx = &VMware::Management::Session::get_temp_context();
         $trnCtx->BeginTransaction();

         $reqPath = $trnCtx->NewArrayIndex("/host2/#_mui/cmd/##");
         $reqPath = $trnCtx->GetAbsPath($reqPath);
         my $reqPathCp = $reqPath;
         $trnCtx->SetString("${reqPath}op", "setNetwork");
         $trnCtx->SetString("${reqPath}user",
                            &VMware::Management::Session::get_usertoken());
         $reqPath = $trnCtx->GetAbsPath("${reqPath}op/setNetwork/");

         $trnCtx->SetString("${reqPath}in/oldNetwork", $old);
         $trnCtx->SetString("${reqPath}in/newNetwork", $net);

         my ($ok, $msg) = VMware::Management::VmdbUtil::vmdbEndCmd($trnCtx,
                                                                   $reqPath);

         if (! $ok) {
            $data .= error(qq{Network Connections: $msg. } .
               qq{Failed to change mapping: "$dev" ~ "$net".});
         } else {
            # Be sure to start at the beginning since the /host2/#_mui/config/network
            # tree will be repopulated after the transaction complete
            # successfully.
            $path = "/host2/#_mui/config/network/#";
            VMware::Management::Util::trace("Modified netmap device: $dev >$old< : >$net<", "debug");
         }

         $trnCtx->LocalArrayUnset($reqPathCp, 1);
      }
   }

   VMware::Management::Util::trace("Adding new netmap devices...", "debug");

   # Handle new NIC names.
   for ($q->param()) {
      my $dev;

      if (/^(.*):net$/i) {
         $dev = $1;
      } else {
         next;
      }

      my $net = $q->param($_);

      if (defined($net) && $net ne "") {
         # T1(setNetwork) BC {
         #    T1(in) BC {
         #       T2(oldNetwork, T_STR);
         #       T2(newNetwork, T_STR);
         #       T2(newDevice, T_STR);
         #    } EC;
         # } EC;

         $trnCtx = &VMware::Management::Session::get_temp_context();
         $trnCtx->BeginTransaction();

         $reqPath = $trnCtx->NewArrayIndex("/host2/#_mui/cmd/##");
         $reqPath = $trnCtx->GetAbsPath($reqPath);
         my $reqPathCp = $reqPath;
         $trnCtx->SetString("${reqPath}op", "setNetwork");
         $trnCtx->SetString("${reqPath}user",
                            &VMware::Management::Session::get_usertoken());
         $reqPath = $trnCtx->GetAbsPath("${reqPath}op/setNetwork/");

         $trnCtx->SetString("${reqPath}in/newNetwork", $net);
         $trnCtx->SetString("${reqPath}in/newDevice", $dev);

         my ($ok, $msg) = VMware::Management::VmdbUtil::vmdbEndCmd($trnCtx,
                                                                   $reqPath);

         if (! $ok) {
            $data .= error(qq{Network Connections: $msg. } .
                           qq{Failed to save mapping: "$dev" ~ "$net".});
         } else {
            VMware::Management::Util::trace("Added new netmap device: $dev >$net<", "debug");
         }

         $trnCtx->LocalArrayUnset($reqPathCp, 1);
      }
   }

   return $data;
}


# ---------------------------------------------------------------------------

sub save {
   my $host = shift();
   my $data = "";
   my $profile = $host->getCurrentLabel();
   my $cfgDb = $host->getConfigDB();

   for my $vmnic (vmnics($host)) {
      my $id = $vmnic->{"id"};
      my $cfgSpeedAndDuplex = $q->param("$id:cfgSpd");
      $cfgDb->set("device.$profile.$id.options", $cfgSpeedAndDuplex);

      # Clustering -- not configurable through the MUI for ESX 2.0
      # my $clustering = $q->param("$id:clstr");
      # if (defined($clustering)) {
      #    $clustering = ($clustering =~ /^enbl$/i) ? 1 : 0;
      #    $cfgDb->set("device.$profile.$id.clustering", $clustering);
      #    my $name = $vmnic->{"vnm"};
      #    if (defined($name)) {
      #       $host->setVMNetAdapterClustering($name, $clustering);
      #    }
      # }

      my $name = $vmnic->{"vnm"};
      if (defined($name)) {
         my $oldTeam = $host->getVMNetAdapterNICTeam($name);
         my $newTeam = $q->param("$id:team");
         my $success = 1;

         if ($oldTeam && ($newTeam eq "none" || ($newTeam ne $oldTeam))) {
            $success = $host->removeBondSlave($oldTeam, $name);
         }
         if ($success && ($newTeam ne "none")) {
            $host->addBondSlave($newTeam, $name);
         }

         # Refetch the NIC team for this NIC.
         $vmnic->{"team"} = $host->getVMNetAdapterNICTeam($name) || "none";
         $vmnic->{"active"} = nicActive($host, $vmnic);
      }
   }

   # XXX rbosch: need to reload changed drivers here

   if (! $host->commitConfigDB()) {
      $data .= error(qq{Network connection changes were not committed to } .
                     qq{the config database.});
   }

   $data .= setNicNames($host);

   return data($host, $data);
}


sub vmnics {
   my $host = shift();
   my $profile = $host->getCurrentLabel();
   my $cfgDb = $host->getConfigDB();
   my @vmnics;

   for my $nic ($host->getPCIDeviceList($profile)->getPCIDevices()) {
      if (! $nic->isNetClassDevice() || $nic->getOwnerString() =~ /^host$/i) {
         next;
      }

      my $bus = $nic->getBus();
      my $device = $nic->getDevice();
      my $function = $nic->getFunction();
      my $id = "$bus.$device.$function";
      my $driver = $nic->getDrivers()->[0];
      my $cfgSpeed = $cfgDb->get("device.$profile.$id.options");

      my $cfgSpeedIdx = 0;
      my $i = 0;
      if (defined($gNICConfigurationOptions{$driver})) {
         for (@{$gNICConfigurationOptions{$driver}}) {
            if (defined($_) && $cfgSpeed =~ $_) {
               $cfgSpeedIdx = $i;
               last;
            }
            $i++;
         }
      }

      push(@vmnics, {"id"     => $id,
                     "dnm"    => $nic->getName(),
                     "vnm"    => "",
                     "drvr"   => $driver,
                     "cfgSpd" => $cfgSpeedIdx,
                     "curSpd" => "",
                     "clstr"  => "null",
                     "team"   => "none",
                     "active" => "false",
                     "bus"    => $bus,
                     "dev"    => $device,
                     "fun"    => $function});
   }


   if ($host->isVMKernel()) {
      # Get the vmnic mappings for the current profile.
      for my $nic (@{$host->getVMNetAdapters()}) {
         my ($bus, $device, $function) = $host->getVMNetAdapterPCIInfo($nic);
         my $id = "$bus.$device.$function";

         for my $vmnic (grep {$_->{"id"} eq $id} @vmnics) {
            $vmnic->{"vnm"} = $nic;
            # If getVMNetAdapterSpeed returns undefined, the link is down.
            $vmnic->{"curSpd"} = $host->getVMNetAdapterSpeed($nic) || "Not connected";
            $vmnic->{"team"} = $host->getVMNetAdapterNICTeam($nic) || "none";
            $vmnic->{"active"} = nicActive($host, $vmnic);
         }
      }
   }

   for my $nic (@vmnics) {
      if (grep { $_ eq $nic->{"drvr"} } @gNICClusteringDisabled) {
         $nic->{"clstr"} = "null";
      } else {
         if (defined($nic->{"vnm"})) {
            $nic->{"clstr"} = $host->getVMNetAdapterClustering($nic->{"vnm"});
         } else {
            $nic->{"vnm"} = "";
            $nic->{"clstr"} = $cfgDb->get("device.$profile.$nic->{id}.clustering");
         }
         $nic->{"clstr"} = $nic->{"clstr"} ? "'enbl'" : "'dsbl'";
      }
   }

   return @vmnics;
}


# ---------------------------------------------------------------------------

sub nicActive {
   my ($host, $vmnic) = @_;
   my $count = 0;
   my $nic = $vmnic->{"vnm"};
   if ($nic) {
      # If the NIC (or NIC team) has a MAC address assigned, it's in use
      if ($vmnic->{"team"} eq "none") {
         my $macs = $host->getVMNetAdapterMAC($nic);
         $count = defined($macs) ? @{$macs} : 0;
      } else {
         my $macs = $host->getVMNetBondAdapterMAC($vmnic->{"team"});
         $count = defined($macs) ? @{$macs} : 0;
      }
   }
   return $count > 0 ? "true" : "false";
}


# ---------------------------------------------------------------------------

sub error {
   my $msg = shift();
   my ($pkg, $fn, $ln) = caller();
   VMware::Management::Util::trace($msg, "error", undef, $pkg, $fn, $ln);
   return "err.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------

sub warning {
   my $msg = shift();
   my ($pkg, $fn, $ln) = caller();
   VMware::Management::Util::trace($msg, "warning", undef, $pkg, $fn, $ln);
   return "wrn.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------
#
# jsStringLiteral --
#
#      Make sure that $msg is safe for consumption in a JavaScript program.
#
# ---------------------------------------------------------------------------

sub jsStringLiteral {
   my $msg = shift();
   $msg =~ s/\\/\\\\/g;
   $msg =~ s/("|')/\\$1/g;
   my $lf = $/;
   $msg =~ s/$lf/\\n/g;

   return qq{"$msg"};
}


# ---------------------------------------------------------------------------

sub html {
   my $data = shift();
   my $cb = shift() || "if (parent.opCb) parent.opCb(self);";

   my $html = qq[<html><head><title></title><script>$/] .
      qq[var err=new Array();var wrn=new Array();$/] .
      qq[$data$/</script></head><body onload="$cb"></body></html>];

   return($html,
          "Pragma"        => "no-cache",
          "Cache-Control" => "no-cache",
          "Content-Type"  => "text/html; charset=utf-8");
}

1;
