### ---------------------------------------------------------------------------
# Copyright 2003 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::SS::ResourceEditor;
# File: VMware/Management/SS/ResourceEditor.pm

use strict;

use VMware::Control '1.01';
use VMware::Control::Server ();
use VMware::Management::Util qw(trace hash_credentials);
use VMware::Management::Session ();
use VMware::VmdbPerl ();
use VMware::HConfig::SSHostRemote ();
use VMware::Management::VmdbUtil ();

use CGI ();
my $q;


# ---------------------------------------------------------------------------

sub handler {
   my %credentials = hash_credentials();
   my $user = $credentials{"l"};
   my $pass = $credentials{"m"};
   
   my $host = VMware::HConfig::SSHostRemote->new(undef, 0, $user, $pass);
   my $cpuHt = $host->isCPUHyperthreadingEnabled();
   
   $q = new CGI(shift());
   for ($q->param()) {
      trace("::::: $_ = " . $q->param($_), "debug");
   }

   my $server = &VMware::Control::Server::new(undef, 0, $user, $pass);
   my $version = VMware::Control::version();
   my $want_version = "1.01";

   if ($want_version ne $version) {
      return html(error("Server version mismatch: expected $want_version, got $version."));
   }

   if (!defined($server)) {
      return html(error("Could not connect to server."));
   }

   if (!$server->connect()) {
      return html(error("Could not connect to server: " . $server->get_last_error() . "."));
   }

   my $vmCfg;        # $vmCfg is the path to the VM configuration file
   my $vmPath = "";  # $vmId is used to construct the needed paths in vmdb
   my $vmId = $q->param("vmid");
   my $ctx = VMware::Management::Session::get_temp_context();

   if ($vmId ne "console") {
      my $regVmPath = "/host2/#_mui/vms/registeredvms/#/";
      while (defined($regVmPath = $ctx->GetNextSibling($regVmPath))) {
         if ($ctx->GetString("${regVmPath}vm") =~ /\#\Q$vmId\E/) {
            $vmCfg = $ctx->GetString("${regVmPath}cfgPath");
	    $vmPath = $ctx->GetString("${regVmPath}vm"); 
	    last;
         }
      }
   } else {
      # For the service console, use the world ID as the config path
      my $cosPath = $ctx->GetString("/vmkernel/console");
      $cosPath =~ m{/vmkernel/world/\#(\d+)};
      $vmCfg = $1;
   }
   
   if (!$vmCfg) {
      return html(error("Cannot set resource settings for unknown virtual machine."));
   }

   my $data = "";

   my $device = $q->param("dev");

   if ($device eq "cpu") {
      $data .= cmd($server, "SetCPUMin", $vmCfg, "min", $q->param("min"));
      $data .= cmd($server, "SetCPUMax", $vmCfg, "max", $q->param("max"));
      $data .= cmd($server, "SetCPUShares", $vmCfg, "shares", $q->param("shares"));
      if ($q->param("affinity")) {
         # Convert affinity from mask to comma-separated list
         # XXX Currently only supports VM-wide affinity, not per-VCPU affinity
         my @affinity;
         for (my $p = 0, my $mask = $q->param("affinity"); $mask; $p++, $mask >>= 1) {
            if ($mask & 1) {
               push(@affinity, $p);
            }
         }
         $data .= cmd($server, "SetCPUAffinity", $vmCfg, "affinity", join(",", @affinity));
      }

      if ( $cpuHt!=0 ) { 
	# hyperthreadind is supported by the ESX CPU
	
	my $htShr = $q->param("ht");
	my $numVCpu = $ctx->GetInt($vmPath."vmx/cfgState/val/dev/#_vcpu/class/vcpu/num");

	my ($trnCtx, $reqPath) = cmdHead($ctx,$vmPath);

	if ( $htShr eq "1" ) { 
	  # user chose to isolate VM from hyper threading
	  
	  if ( $numVCpu > 1 ) {  # VM is using more than 1 virtual CPU
	    $trnCtx->SetString("${reqPath}in/cpu/htSharing", "internal");
	  } else { # VM is using 1 virtual CPU
	    $trnCtx->SetString("${reqPath}in/cpu/htSharing", "none");
	  }

	} else { 
	  # user chose not to isolate VM from hyper-threading

	  $trnCtx->SetString("${reqPath}in/cpu/htSharing", "any");
	}

	my ($ok, $data) = cmdTail($trnCtx, $reqPath);

      }

   } elsif ($device eq "mem") {
      $data .= cmd($server, "SetMemMin", $vmCfg, "min", $q->param("min"));
      $data .= cmd($server, "SetMemShares", $vmCfg, "shares", $q->param("shares"));
      if ($q->param("affinity")) {
         $data .= cmd($server, "SetMemAffinity", $vmCfg, "affinity", $q->param("affinity"));
      }
   } elsif ($device eq "dsk") {
      foreach my $param ($q->param()) {
         if ($param =~ /^(dev|vmid)$/) {
            next;
         } elsif ($param =~ /^(\S+)-(shares)$/) {
            my $tgt = $1;
            my $shares = $q->param($param);
            trace("tgt = $tgt, shares = $shares", "debug");
            $data .= cmd($server, "SetDiskShares", $vmCfg,
                         "target", $tgt, 
                         "shares", $shares);
         } else {
            trace("Unknown disk resource option $param", "error");
         }
      }
   } elsif ($device eq "net") {
      my $hasFilter = $q->param("nfshaper-old");
      my $enableFilter = $q->param("nfshaper");

      if ($enableFilter) {
         my $args = $q->param("avg") . $q->param("avgUnits") . " " . 
                    $q->param("peak") . $q->param("peakUnits") . " " .
                    $q->param("burst") . $q->param("burstUnits");
         if ($hasFilter) {
            $data .= cmd($server, "ChangeNetFilter", $vmCfg,
                         "instance", "nfshaper.x.y", "args", $args);
         } else {
            $data .= cmd($server, "AttachNetFilter", $vmCfg,
                         "nfclass", "nfshaper", "args", $args);
         }
      } elsif ($hasFilter) {
         $data .= cmd($server, "DetachNetFilter", $vmCfg);
      }
   } else {
      return html(error("Unknown device '$device'."));
   }

   return html($data);
}


# ---------------------------------------------------------------------------
#
# cmd --
#
#      
#      Issues a resource editing command to serverd and reports any errors.
#
# Prototype:
#
#      string cmd($server, $op, $cfg, ...)
#
# Arguments:
#
#      $server     ... VMware::Control::Server object
#      $op         ... Operation to perform: invokes exec
#                      handler ESXResourceEdit_$op
#      $cfg        ... Config file path of VM to edit
#      ...         ... Optional key-value pairs ($tag, $val) to add
#                      to the command: exec.in.$tag = $val
#
# Results:
#
#      Returns any errors generated by the operation.
#
# ---------------------------------------------------------------------------

sub cmd {
   my $server = shift;
   my $op = shift;
   my $cfg = shift;
   my $ret = "";

   my $di = new VMware::DOMAccess("exec");
   $di->setValue("exec.op", "ESXResourceEdit_$op");
   $di->setValue("exec.in.world", $cfg);
   while (defined(my $tag = shift) && defined(my $val = shift)) {
      $di->setValue("exec.in.$tag", $val);
   }
   my $xml = $di->toXmlString();
   $di->dispose();

   $xml = $server->exec($xml);

   my $do = VMware::DOMAccess->newXML($xml);
   if ($do->getValue("exec.returncode") != 1) {
      foreach my $i ($do->listElementNames("exec.messages")) {
         $ret .= error($do->getValue("exec.messages.$i"));
      }
   }      
   $do->dispose();

   return $ret;
}

# ---------------------------------------------------------------------------

sub cmdHead {
   my $ctx = shift();
   my $vmPath = shift();

   # Create a new context for this transaction to avoid
   # VMDB_E_DISTRIBUTED_COMMIT errors.
   my $trnCtx = &VMware::Management::Session::get_temp_context();
   $trnCtx->SetCurrentPath($ctx->GetCurrentPath());

   $trnCtx->BeginTransaction();

   # Initialize VMDB request.
   my $reqPath = $trnCtx->NewArrayIndex("/vmkernel/resources".$vmPath."cmd/##");
   $reqPath = $trnCtx->GetAbsPath($reqPath);
   $trnCtx->SetString("${reqPath}op", "setResources");
   $trnCtx->SetString("${reqPath}user",
                      &VMware::Management::Session::get_usertoken());
   return ($trnCtx, "${reqPath}op/setResources/");
}

# ---------------------------------------------------------------------------

sub cmdTail {
   my $trnCtx = shift();
   my $reqPath = shift();
   my $data = "";

   my ($ok, $msg) = VMware::Management::VmdbUtil::vmdbEndCmd($trnCtx,
                                                             $reqPath);

   if (! $ok) {
     $data .= error("Isolate Virtual Machine from Hyper-Threading: $msg.");
   } 

   $reqPath =~ s{^(.*/cmd/\#\#.*?/).*}{$1};
   $trnCtx->LocalArrayUnset($reqPath, 1);

   return wantarray ? ($ok, $data) : $data;
}


# ---------------------------------------------------------------------------

sub error {
   my $msg = shift();
   trace($msg, "error");
   return "err.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------

sub warning {
   my $msg = shift();
   trace($msg, "warning");
   return "wrn.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------

sub html {
   my $data = shift();
   my $cb = shift() || "if (parent.opCb) parent.opCb(self);";

   my $html = qq[<html><head><title></title><script>$/] .
      qq[var err=new Array();var wrn=new Array();$/] .
      qq[$data$/</script></head><body onload="$cb"></body></html>];

   return($html,
          "Pragma"        => "no-cache",
          "Cache-Control" => "no-cache",
          "Content-Type"  => "text/html; charset=utf-8");
}

# ---------------------------------------------------------------------------
#
# jsStringLiteral --
#
#      Make sure that $msg is safe for consumption in a JavaScript program.
#
# ---------------------------------------------------------------------------

sub jsStringLiteral {
   my $msg = shift();
   $msg =~ s/\\/\\\\/g;
   $msg =~ s/("|')/\\$1/g;
   my $lf = $/;
   $msg =~ s/$lf/\\n/g;

   return qq{"$msg"};
}

1;
