# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::SS::ScriptedInstall;
# File: VMware/Management/SS/ScriptedInstall.pm

use strict;

use CGI;
use Carp qw(confess);
$SIG{__DIE__} = \&confess;

use VMware::HConfig::Constant qw( %EsxScriptedInstall );

#use VMware::Management::Util ();
use VMware::Management::SS::ScriptedInstall::Lib;

sub handler {
   my $cgi = new CGI(shift());
   my $html;

   ## if (not admin)
   ## return unable to perform scripted installs

   if ($cgi->param) {
      ## Check for errors that shouldn't happen if people aren't mucking
      ## around with the form.

      my $errStr;
      my @formErrors = checkFormErrors($cgi);
      my @cFormParams = checkConditionalErrors($cgi);

      my (@allErrors) = (@formErrors, @cFormParams);

      if (scalar(@allErrors) > 0) { 
         foreach my $err (@allErrors) {
            $errStr .= error($err) . "<br>";
         }

         return ("In this case, do we forego the opCb stuff? $errStr",
          "Pragma"        => "no-cache",
          "Cache-Control" => "no-cache",
          "Content-Type"  => "text/html; charset=utf-8");
      }

      ## Check for user errors that *also* shouldn't happen (JavaScript
      ## should catch them)
      my @paramErrors = checkInstallParams($cgi);

      if (scalar(@paramErrors) > 0) { 
         my $errStr;
         foreach my $err (@paramErrors) {
            $errStr .= error($err);
         }
     
         return ("In this case, do we forego the opCb stuff? $errStr",
          "Pragma"        => "no-cache",
          "Cache-Control" => "no-cache",
          "Content-Type"  => "text/html; charset=utf-8");
      }

      my $filename = $EsxScriptedInstall{'DEFAULT_DISKIMAGE_NAME'};

      my $kickstartImage = buildKickstartDiskImage($cgi);
      my $imageSize = length($kickstartImage);

      return($kickstartImage, 
            'Content-type' => 'application/octet-stream',
            'Content-disposition' => "attachment, filename=$filename",
            'Content-length' => $imageSize);
   }
   else {
      ## A null query simply replies with whether or not the server is 
      ## capable of doing scripted installs.
      my $ksFileLocation = 
       "$EsxScriptedInstall{'WEBCONFIG_ROOT'}/$EsxScriptedInstall{'KS_FILE'}";

      if (not -e $ksFileLocation) {
         return html("var scriptedInstallSetup = false;");
      }
      else {
         return html("var scriptedInstallSetup = true;");
      }
   }
}

sub html {
   my $data = shift();
   my $cb = shift() || "if (parent.opCb) parent.opCb(self);";

   my $html = qq[<html><head><title></title><script>$/] .
      qq[var err=[];var wrn=[];$/] .
      qq[$data$/</script></head><body onload="$cb"></body></html>];

   return($html,
          "Pragma"        => "no-cache",
          "Cache-Control" => "no-cache",
          "Content-Type"  => "text/html; charset=utf-8");
}

# ---------------------------------------------------------------------------

sub error {
   my $msg = shift();
   #VMware::Management::Util::trace($msg, "error");
   return "err.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------

sub warning {
   my $msg = shift();
   #VMware::Management::Util::trace($msg, "warning");
   return "wrn.push(" . jsStringLiteral($msg) . ");$/";
}

## Shouldn't there be a library for this?
# ---------------------------------------------------------------------------
#
# jsStringLiteral --
#
#      Make sure that $msg is safe for consumption in a JavaScript program.
#
# ---------------------------------------------------------------------------
sub jsStringLiteral {
   my $msg = shift();
   $msg =~ s/\\/\\\\/g;
   $msg =~ s/("|')/\\$1/g;
   my $lf = $/;
   $msg =~ s/$lf/\\n/g;

   return qq{"$msg"};
}


############################################################################

sub checkFormErrors {
   my $cgiObj = shift;
   my @errors; 
   my $val;

   ## These are fields we know should be filled in
   my @requiredFields = qw( install_type
                            install_method
                            mbr
                            tz
                            reboot
                            passwd1
                            passwd2 );

   foreach my $field (@requiredFields) {
      push(@errors, "$field missing!") if ($cgiObj->param($field) eq undef);
   }

   foreach my $field (@requiredFields) {
      push(@errors, "Required field '$field' missing") 
       if ($cgiObj->param($field) eq undef);
   }

   for (my $part_id = 0; $part_id < $EsxScriptedInstall{'MAX_PARTITIONS'}; 
    $part_id++) {
      if ($cgiObj->param('mount' . $part_id) !~ /^\s*$/) {
         push(@errors, "Invalid partition mount on ID $part_id") if
          ($cgiObj->param('mount' . $part_id) !~ /^\/[\w\d\/]*$/ && 
           $cgiObj->param('mount' . $part_id) ne 'swap');

         push(@errors, "Invalid partition size on ID $part_id") if
          ($cgiObj->param('size' . $part_id) !~ /^\s*\d+\s*$/);
          ($cgiObj->param('size' . $part_id) <= 0);
      }

      push(@errors, "Invalid partition type on ID $part_id") if
       ($cgiObj->param('type' . $part_id) !~ /^ext2|ext3|swap|vmfs2$/);

      push(@errors, "Invalid partition drive on ID $part_id") if
       ($cgiObj->param('drive' . $part_id) ne 'primary');
   }
 
   $val = $cgiObj->param('install_type');
   push(@errors, "Invalid installation type")
    if ($val !~ /^install|upgrade$/);
 
   $val = $cgiObj->param('install_method');
   push(@errors, "Invalid installation type") if ($val !~ /^cdrom|lan|remote$/);
 
   $val = $cgiObj->param('mbr');
   push(@errors, "Invalid boot loader type") if ($val !~ /^mbr|custom$/);

   ## Revalidate password stuff
   push(@errors, "The root passwords you entered do not match")
    if ($cgiObj->param('passwd1') ne $cgiObj->param('passwd2')); 

   push(@errors, "Root password must be at least six characters long")
    if ($cgiObj->param('passwd1') !~ /^\S{6,}$/);
 
   return @errors;
}

sub checkConditionalErrors {
   my $cgiObj = shift;
   my @errors;

   push(@errors, "No remote server specified") if 
    ($cgiObj->param('install_type') eq 'remote' && 
     $cgiObj->param('remote_server') eq undef);

   push(@errors, "No MBR partition location") if 
    ($cgiObj->param('mbr') eq 'custom' && 
    $cgiObj->param('mbr_partition') eq undef);

   my @networkFields = qw ( network
                            hostname
                            ipaddress
                            netmask
                            defaultgw 
                            nameserver1 );

   push(@errors, 
    "Network install specified, but missing or invalid network setting") if 
    ($cgiObj->param('install_method') ne 'cdrom' && 
     $cgiObj->param('network') !~ /^dhcp|static$/);

   push(@errors, "Remote install specified, but remote_server missing") if 
    ($cgiObj->param('install_method') eq 'remote' && 
     $cgiObj->param('remote_server') eq undef);

   if ($cgiObj->param('network') eq 'static') {
      ## Then all the networkFields should be defined
      foreach my $field (@networkFields) {
         push(@errors, "Missing network field $field") 
          if ($cgiObj->param($field) eq undef);
      } 
   }

   return @errors;
}

sub checkInstallParams {
   my $cgiObj = shift;
   my @errors;


   ## Check the MBR settings 
   if ($cgiObj->param('mbr') eq 'custom' ) {
      push(@errors, "Invalid partition for the MBR") 
       if ($cgiObj->param('mbr_partition') !~ / ^ \/dev\/ /x );
   }

   ## Check the network settings
   if ($cgiObj->param('network') eq 'static') {
      push(@errors, "No hostname specified") 
       if ($cgiObj->param('hostname') =~ /^\s*$/);

      push(@errors, "Invalid IP address specified") 
       if (not checkValidIPAddress($cgiObj->param('ipaddress')));

      push(@errors, "Invalid netmask specified") 
       if (not checkValidIPAddress($cgiObj->param('netmask')));

      push(@errors, "Invalid gateway specified") 
       if (not checkValidIPAdderss($cgiObj->param('defaultgw')));
   }

   return @errors;
}

1;
