use strict;

package RedHat::BootImage;

#############################################################################
# RedHat::BootImage.pm
# by Patrick Devine
# for VMware Inc.
# 2002.04.23
#
# This is the first stab at creating a module which will allow simple access
# to various parts of the Red Hat installer.  It works fairly well right now
# for modifying boot floppies, the initial ram disk, the stage2 image and
# kernel module bundles.
#
# It still needs some work though..  it's biggest weakness is a lack of a
# destructor method for cleaning up after itself.  Ideally this would pop
# through the mount stack and unmount, re-compress, etc. all of the
# previously mounted image files.
#
#

my $MAX_LOOP_DEVICE = 9;

sub new {
  my $proto	= shift;
  my $file	= shift;

  my $class	= ref( $proto ) || $proto;
  my $self	= {};

  $self->{'image_file'}	= $file;
  $self->{'verbose'}	= 0;

  bless($self);

  $self;
}

#############################################################################
# Method:	mount_image
# Description:	mount any image which we can read from or write to.  usually
#		used for manipulating ms-dos floppy disks
# Date:		2002.04.23

sub mount_image {
  my $self	= shift;
  my $file	= shift || $self->{'image_file'};

  print "Mounting $file\n"
    if $self->{'verbose'};

  return
    unless -e $file;

  #
  # find a place to mount the image
  #

  my $mount	= '';

  for( my $cnt = 0; $cnt <= $MAX_LOOP_DEVICE; $cnt++ ) {
    $mount = "/mnt/loop$cnt";

    ## is 0777 right?!
    mkdir $mount, 0777
      unless -d $mount;

    #
    # this should be eval'd and return an error if it breaks
    #

    if( !check_if_dir_mounted( $mount )) {
      `mount -oloop $file $mount`;
      push @{ $self->{'mount_point'} }, $mount;
      last;
    }
  }

  die "Couldn't get a proper mount point"
    unless $mount;

  return $mount;
}

#############################################################################

sub umount_image {
  my $self	= shift;
  my $mount	= shift;

  $mount = pop @{ $self->{'mount_point'} }
    unless $mount;

  `umount $mount`;

  1;

}

#############################################################################
# Method:	mount_cramfs
# Description:	logically mount a cramfs partition so that we can do work
#		inside of it.  Usually this is Red Hat's stage2.img file.
# Date:		2002.05.07

sub mount_cramfs {
  my $self	= shift;
  my $file	= shift || $self->{'image_file'} || 'stage2.img';

  my $tmp_target	= find_temp_file_name( 'cramfs' );
  print "[$tmp_target]\n";

  #
  # mount our cramfs image and copy its contents to a temporary
  # directory where we can edit it directly
  #

  my $target	= $self->mount_image( $file );

  `cp -a $target $tmp_target`;

  $self->umount_image();

  push @{ $self->{'cramfs_file'} }, [ $file, $tmp_target ];
  push @{ $self->{'mount_point'} }, $tmp_target;

  1;

}

#############################################################################

sub umount_cramfs {
  my $self	= shift;

  my $target		= '';
  my $tmp_target	= '';

  ( $target, $tmp_target ) = @{ pop @{ $self->{cramfs_file} }};

  `mkcramfs $tmp_target $target`; 

  pop @{ $self->{'mount_point'} };

  1;
}

#############################################################################
# Method:	mount_initrd
# Description:	logically mount an initial ram disk so that we can change
#		around files inside of it.
# Date:		2002.04.23

sub mount_initrd {
  my $self	= shift;
  my $file	= shift || 'initrd.img';

  my $target		= '';
  my $tmp_target	= '';

  #
  # determine our target file from the last mount point
  # (if there was one)
  # 

  my @mounts	= @{ $self->{'mount_point'} };

  if( @mounts ) {
    if( -e $mounts[$#mounts] . "/$file" ) {
      $target	= $mounts[$#mounts] . "/$file";
    } elsif( -e $mounts[$#mounts] . "/modules/$file" ) {
      $target	= $mounts[$#mounts] . "/modules/$file";
    } else {
      return 0;
    }
  } else {
    $target	= $file;
  }

  return 0
    unless -e $target;

  my $tmp_target	= find_temp_file_name( 'initrd' );
  `zcat $target > $tmp_target`;

  #
  # save our targets for later and mount the image
  #

  push @{ $self->{'initrd_file'} }, [ $target, $tmp_target ];

  $self->mount_image( $tmp_target );

  1;

}

#############################################################################

sub umount_initrd {
  my $self	= shift;

  my $target		= '';
  my $tmp_target	= '';

  ( $target, $tmp_target ) = @{ pop @{ $self->{'initrd_file'} }};

  $self->umount_image();

  `gzip -9 -c $tmp_target > $target`; 
  
  1;

}

#############################################################################
# Method:	mount_cgz
# Description:	logically mount a cpio'd + gzipped file.  Red Hat does this
#		to store kernel modules inside the initial ram disk.
# Date:		2002.05.01

sub mount_cgz {
  my $self	= shift;
  my $file	= shift || 'modules/modules.cgz';

  my $target		= '';
  my $tmp_target	= '';

  #
  # determine our target file from the last mount point
  # (if there was one)
  # 

  my @mounts	= @{ $self->{'mount_point'} };

  if( @mounts ) {
    $target	= $mounts[$#mounts] . "/$file";
  } else {
    $target	= $file;
  }

  return 0
    unless -e $target;

  $tmp_target = find_temp_file_name( 'modules' ); 
  mkdir $tmp_target;

  `cd $tmp_target && zcat $target | cpio -id`;

  #
  # save our targets for later and mount the image
  #

  push @{ $self->{'cpio_dir'} }, [ $target, $tmp_target ];
  push @{ $self->{'mount_point'} }, $tmp_target;

  1;

}

#############################################################################

sub umount_cgz {
  my $self	= shift;

  my $target		= '';
  my $tmp_target	= '';

  return 0
    unless $self->{'cpio_dir'};

  ( $target, $tmp_target ) = @{ pop @{ $self->{'cpio_dir'} }};

  `cd $tmp_target && find | cpio -o -H crc | gzip -9 > $target`;

  pop @{ $self->{'mount_point'} };
  
  1;
  

}

#############################################################################
# Method:	add_file
# Description:	copies a file to the last logically mounted device
# Date:		2002.05.01

sub add_file {
  my $self	= shift;
  my $file	= shift;
  my $target	= shift;

  $target =~ s# ^ / ##x;

  return 0
    unless -e $file;

  my @mounts	= @{ $self->{'mount_point'} };

  if( @mounts ) {
    $target	= $mounts[$#mounts] . "/$target";
  } else {
    $target	= $target;
  }

  print "Copying $file to $target\n"
    if $self->{verbose};

  `cp $file $target`;

}

#############################################################################
# Method:	retrieve_file
# Description:	copies a file from the last logically mounted device
# Date:		2003.02.27

sub retrieve_file {
  my $self	= shift;
  my $file	= shift;
  my $target	= shift;

  $file =~ s# ^ / ##x;

  my @mounts	= @{ $self->{'mount_point'} };

  if( @mounts ) {
    $file	= $mounts[$#mounts] . "/$file";
  } else {
    $file	= $file;
  }

  return 0
    unless -e $file;

  print "Copying $file to $target\n"
    if $self->{'verbose'};

  `cp $file $target`;

}

#############################################################################
# Method:	remove_file
# Description:	removes a file to the last logically mounted device
# Date:		2002.05.01

sub remove_file {
  my $self	= shift;
  my $file	= shift;

  $file =~ s# ^ / ##x;

  my @mounts	= @{ $self->{'mount_point'} };
  my $target;
  if( @mounts ) {
    $target	= $mounts[$#mounts] . "/$file";
  } else {
    $target	= $file;
  }

  unlink $target;

}

#
# stubs for future method calls
#

sub add_module {
  my $self	= shift;

}

sub remove_module {
  my $self	= shift;
}

sub list_modules {
  my $self	= shift;

}


#############################################################################
# Function:	check_if_dir_mounted
# Description:	Read through /proc/mounts and see if a directory
#		is currently mounted
# Date:		2002.05.07

sub check_if_dir_mounted {
  my $file	= shift;

  my @dirs = read_file( '/proc/mounts' );

  for( @dirs ) {
    my( $target, $dest, $fs, $attr ) = split /\s/;
    return 1
      if $dest eq $file;
  }

  0;

}

#############################################################################
# Function:	find_temp_file_name
# Description:	Find a filename/directory in /tmp that we can use for
#		doing some work in
# Date:		2002.05.01

sub find_temp_file_name {
  my $file	= shift;
  my $random	= shift || 10000;

  my $num	= 0;
  my $target	= '';

  #
  # create a name for the initrd file and uncompress it into
  #

  while( 1 ) {
    $num	= int( rand( $random )) + 1;
    $target	= "/tmp/$file-$num";
    last
      unless -e $target;
  }

  $target;

}

#############################################################################
# Function:	read_file
# Date:		2002.04.23

sub read_file {
  my $file	= shift;

  return ''
    unless -e $file;

  open IN, $file;
  my @data = <IN>;
  close IN;

  @data;

}


1;
