#!/usr/bin/perl -w

#############################################################
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
#############################################################
#
# SecurityConfig.pm
# 
# A script that presents the setup options available.
package VMware::Management::SS::SecurityConfig;

# Script flow:
# First page to show is the EULA if it has not been seen or agreed
# If EULA is agreed, the configuration form is shown
# After configuration is submitted, the DHCP copyright (ISC) will be shown.
# A done page is shown

use strict;
use VMware::HConfig::SSHostRemote;
use VMware::HConfig::Constant qw(c_getHelper c_getFile
				 $URL_INDEX $URL_VMCONFIG $URL_VIEWPART
                                 $URL_SECURITY %gCertInfo
                                 $CFG_CERTIFICATE
				 );
use VMware::ExtHelpers qw(&Warning 
                          $gPanicCallback 
			  &clearWarnings);
use Carp qw(confess);
$SIG{__DIE__} = \&confess;
use CGI;
use VMware::Management::Util qw( hash_credentials );

use VMware::Management::SS::ESXConfig;
my $htmlTitle = "Security Configuration";

# Global variables

my $URL_PREV = $URL_VMCONFIG;
my $URL_NAME_PREV = "Server Configuration";
my $URL_NEXT = $URL_VIEWPART;
my $URL_NAME_NEXT = "Edit Partitions";
my $URL_NAME_SECURITY = "Security Configuration";
my $URL_NAME_CERTIFICATE = "Edit Certificate"; 

my $RCom = "request";
my $RCom_Save = "savesecurity";
my $RCom_SetLevel = "setSecurityLevel";
my $RCom_SaveCert = "savecertificate";
my $RCom_ReturnToLevelChooser = "returntoLevelChooser";
my $RCom_EditCert = "editcertificate";

my $RCom_Debug = "debug";

my $ssh = "sshd";
my $telnet =  "telnet";
my $ftp =  "ftp";
my $portmap = "portmap";

my $levelChoice = "levelchoice";
my $hiSecurity = 3;
my $medSecurity = 2;
my $lowSecurity = 1;
my $custSecurity = "4";

my $encryptRUI =  "ruissl";
my $encryptMUI =  "muissl";

# If at some point in the future we want to allow the user to select the 
# specific encryption algorithm, set the following to true 
# (further cleanup will still be required)
my $usePlaintext = "Plaintext";
my $useDES = "DES";
my $use3DES = "3DES";
my $useBlowfish = "Blowfish";

my $hostname;
my $port;
my $user;
my $password;
    
my %countryCodes = (
   "Afghanistan" => "AF",
   "Albania" => "AL",
   "Algeria" => "DZ",
   "American Samoa" => "AS",
   "Andorra" => "AD",
   "Angola" => "AO",
   "Anguilla" => "AI",
   "Antarctica" => "AQ",
   "Antigua and Barbuda" => "AG",
   "Argentina" => "AR",
   "Armenia" => "AM",
   "Aruba" => "AW",
   "Australia" => "AU",
   "Austria" => "AT",
   "Azerbaijan" => "AZ",
   "Bahama" => "BS",
   "Bahrain" => "BH",
   "Bangladesh" => "BD",
   "Barbados" => "BB",
   "Belarus" => "BY",
   "Belgium" => "BE",
   "Belize" => "BZ",
   "Benin" => "BJ",
   "Bermuda" => "BM",
   "Bhutan" => "BT",
   "Bolivia" => "BO",
   "Bosnia and Herzegovina" => "BA",
   "Botswana" => "BW",
   "Bouvet Island" => "BV",
   "Brazil" => "BR",
   "British Indian Ocean Territory" => "IO",
   "British Virgin Islands" => "VG",
   "Brunei Darussalam" => "BN",
   "Bulgaria" => "BG",
   "Burkina Faso" => "BF",
   "Burundi" => "BI",
   "Cambodia" => "KH",
   "Cameroon" => "CM",
   "Canada" => "CA",
   "Cape Verde" => "CV",
   "Cayman Islands" => "KY",
   "Central African Republic" => "CF",
   "Chad" => "TD",
   "Chile" => "CL",
   "China" => "CN",
   "Christmas Island" => "CX",
   "Cocos (Keeling) Islands" => "CC",
   "Colombia" => "CO",
   "Comoros" => "KM",
   "Congo" => "CG",
   "Cook Iislands" => "CK",
   "Costa Rica" => "CR",
   "Cote D'ivoire (Ivory Coast)" => "CI",
   "Croatia" => "HR",
   "Cuba" => "CU",
   "Cyprus" => "CY",
   "Czech Republic" => "CZ",
   "Denmark" => "DK",
   "Djibouti" => "DJ",
   "Dominica" => "DM",
   "Dominican Republic" => "DO",
   "East Timor" => "TP",
   "Ecuador" => "EC",
   "Egypt" => "EG",
   "El Salvador" => "SV",
   "Equatorial Guinea" => "GQ",
   "Eritrea" => "ER",
   "Estonia" => "EE",
   "Ethiopia" => "ET",
   "Falkland Islands (Malvinas)" => "FK",
   "Faroe Islands" => "FO",
   "Fiji" => "FJ",
   "Finland" => "FI",
   "France, Metropolitan" => "FX",
   "France" => "FR",
   "French Guiana" => "GF",
   "French Polynesia" => "PF",
   "French Southern Territories" => "TF",
   "Gabon" => "GA",
   "Gambia" => "GM",
   "Georgia" => "GE",
   "Germany" => "DE",
   "Ghana" => "GH",
   "Gibraltar" => "GI",
   "Greece" => "GR",
   "Greenland" => "GL",
   "Grenada" => "GD",
   "Guadeloupe" => "GP",
   "Guam" => "GU",
   "Guatemala" => "GT",
   "Guinea-Bissau" => "GW",
   "Guinea" => "GN",
   "Guyana" => "GY",
   "Haiti" => "HT",
   "Heard and McDonald Islands" => "HM",
   "Honduras" => "HN",
   "Hong Kong" => "HK",
   "Hungary" => "HU",
   "Iceland" => "IS",
   "India" => "IN",
   "Indonesia" => "ID",
   "Iraq" => "IQ",
   "Ireland" => "IE",
   "Islamic Republic of Iran" => "IR",
   "Israel" => "IL",
   "Italy" => "IT",
   "Jamaica" => "JM",
   "Japan" => "JP",
   "Jordan" => "JO",
   "Kazakhstan" => "KZ",
   "Kenya" => "KE",
   "Kiribati" => "KI",
   "Korea, Democratic People's Republic of" => "KP",
   "Korea, Republic of" => "KR",
   "Kuwait" => "KW",
   "Kyrgyzstan" => "KG",
   "Lao People's Democratic Republic" => "LA",
   "Latvia" => "LV",
   "Lebanon" => "LB",
   "Lesotho" => "LS",
   "Liberia" => "LR",
   "Libyan Arab Jamahiriya" => "LY",
   "Liechtenstein" => "LI",
   "Lithuania" => "LT",
   "Luxembourg" => "LU",
   "Macau" => "MO",
   "Madagascar" => "MG",
   "Malawi" => "MW",
   "Malaysia" => "MY",
   "Maldives" => "MV",
   "Mali" => "ML",
   "Malta" => "MT",
   "Marshall Islands" => "MH",
   "Martinique" => "MQ",
   "Mauritania" => "MR",
   "Mauritius" => "MU",
   "Mayotte" => "YT",
   "Mexico" => "MX",
   "Micronesia" => "FM",
   "Moldova, Republic of" => "MD",
   "Monaco" => "MC",
   "Mongolia" => "MN",
   "Monserrat" => "MS",
   "Morocco" => "MA",
   "Mozambique" => "MZ",
   "Myanmar" => "MM",
   "Nambia" => "NA",
   "Nauru" => "NR",
   "Nepal" => "NP",
   "Netherlands Antilles" => "AN",
   "Netherlands" => "NL",
   "New Caledonia" => "NC",
   "New Zealand" => "NZ",
   "Nicaragua" => "NI",
   "Niger" => "NE",
   "Nigeria" => "NG",
   "Niue" => "NU",
   "Norfolk Island" => "NF",
   "Northern Mariana Islands" => "MP",
   "Norway" => "NO",
   "Oman" => "OM",
   "Pakistan" => "PK",
   "Palau" => "PW",
   "Panama" => "PA",
   "Papua New Guinea" => "PG",
   "Paraguay" => "PY",
   "Peru" => "PE",
   "Philippines" => "PH",
   "Pitcairn" => "PN",
   "Poland" => "PL",
   "Portugal" => "PT",
   "Puerto Rico" => "PR",
   "Qatar" => "QA",
   "Reunion" => "RE",
   "Romania" => "RO",
   "Russian Federation" => "RU",
   "Rwanda" => "RW",
   "Saint Lucia" => "LC",
   "Samoa" => "WS",
   "San Marino" => "SM",
   "Sao Tome and Principe" => "ST",
   "Saudi Arabia" => "SA",
   "Senegal" => "SN",
   "Seychelles" => "SC",
   "Sierra Leone" => "SL",
   "Singapore" => "SG",
   "Slovakia" => "SK",
   "Slovenia" => "SI",
   "Solomon Islands" => "SB",
   "Somalia" => "SO",
   "South Africa" => "ZA",
   "South Georgia and the South Sandwich Islands" => "GS",
   "Spain" => "ES",
   "Sri Lanka" => "LK",
   "St. Helena" => "SH",
   "St. Kitts and Nevis" => "KN",
   "St. Pierre and Miquelon" => "PM",
   "St. Vincent and the Grenadines" => "VC",
   "Sudan" => "SD",
   "Suriname" => "SR",
   "Svalbard and Jan Mayen Islands" => "SJ",
   "Swaziland" => "SZ",
   "Sweden" => "SE",
   "Switzerland" => "CH",
   "Syrian Arab Republic" => "SY",
   "Taiwan, Province of China" => "TW",
   "Tajikistan" => "TJ",
   "Tanzania, United Republic of" => "TZ",
   "Thailand" => "TH",
   "Togo" => "TG",
   "Tokelau" => "TK",
   "Tonga" => "TO",
   "Trinidad and Tobago" => "TT",
   "Tunisia" => "TN",
   "Turkey" => "TR",
   "Turkmenistan" => "TM",
   "Turks and Caicos Islands" => "TC",
   "Tuvalu" => "TV",
   "Uganda" => "UG",
   "Ukraine" => "UA",
   "United Arab Emirates" => "AE",
   "United Kingdom (Great Britain)" => "GB",
   "United States Minor Outlying Islands" => "UM",
   "United States Virgin Islands" => "VI",
   "United States of America" => "US",
   "Unknown or unspecified country" => "ZZ",
   "Uruguay" => "UY",
   "Uzbekistan" => "UZ",
   "Vanuatu" => "VU",
   "Vatican City State (Holy See)" => "VA",
   "Venezuela" => "VE",
   "Viet Nam" => "VN",
   "Wallis and Futuna Islands" => "WF",
   "Western Sahara" => "EH",
   "Yemen" => "YE",
   "Yugoslavia" => "YU",
   "Zaire" => "ZR",
   "Zambia" => "ZM",
   "Zimbabwe" =>"ZW");

my @stateList = (
   "Other or Unknown",
   "Alabama",
   "Alaska",
   "Alberta",
   "American Samoa",
   "Arizona",
   "Arkansas",
   "British Columbia",
   "California",
   "Colorado",
   "Connecticut",
   "Delaware",
   "District of Columbia",
   "Florida",
   "Georgia",
   "Guam",
   "Hawaii",
   "Idaho",
   "Illinois",
   "Indiana",
   "Iowa",
   "Kansas",
   "Kentucky",
   "Louisiana",
   "Maine",
   "Manitoba",
   "Maryland",
   "Massachusetts",
   "Michigan",
   "Minnesota",
   "Mississippi",
   "Missouri",
   "Montana",
   "Nebraska",
   "Nevada",
   "New Brunswick",
   "New Hampshire",
   "New Jersey",
   "New Mexico",
   "New York",
   "Newfoundland",
   "North Carolina",
   "North Dakota",
   "Northwest Territories",
   "Nova Scotia",
   "Ohio",
   "Oklahoma",
   "Ontario",
   "Oregon",
   "Other or Unknown",
   "Palau",
   "Pennsylvania",
   "Prince Edward Island",
   "Puerto Rico",
   "Quebec",
   "Rhode Island",
   "Saskatchewan",
   "South Carolina",
   "South Dakota",
   "Tennessee",
   "Texas",
   "Utah",
   "Vermont",
   "Virginia",
   "Washington",
   "West Virginia",
   "Wisconsin",
   "Wyoming",
   "Yukon");


#####################################################################
# HTML producing functions                                          #
#####################################################################

sub PanicCallbackHTML(@) {
    my (@error) = @_;
    if ( $error[0] ) {
       print toHTMLPanic($htmlTitle, $error[0]);
    } else {
       print toHTMLPanic($htmlTitle, "");
    }
}

# 
# editCertificateHTML
# 
# Display the html page that lets users create a custom certificate
#
sub editCertificateHTML($$$) {
    my ($cgi, $host, $mustApplyChanges) = @_;
    my $url_self = $cgi->url(-relative=>1);
    my $wizard = $cgi->param("wizard");
    my %options;
 

    my $title = "Edit SSL Certificate";
    my @path = @htmlPath;
    push @path, [ $htmlTitle, $url_self ];

    loadSecurityOptions(\%options,"mui_certificate");

    my $s = 
        toHTMLPath(\@path, $title) . 
        toHTMLSystemInfo($host) . 
        toHTMLWarnings();

    $s .= <<END_HTML;
<script type="text/javascript">
  <!--
  function doConfirmSave() {
     return confirm("Are you sure you want to save the new certificate?");
  }
  // -->
</script>
<form name="save" method="POST" action="$url_self", onSubmit="return doConfirmSave()">
END_HTML

    $s .= ("<p class='txt'>In order to function properly, SSL requires a\n" .
           "certificate containing information about the location and owner of a server.\n".
           "All of this data is optional, but recommended.  Displayed below are the \n" .
           "contents of the current certificate.</p>");

    $s .= toHTMLTableHeader();
    foreach my $key (keys %gCertInfo) {
       my $curValue = $options{$key} || $gCertInfo{$key}{value};
       
       my $name = qq(<b>$gCertInfo{$key}{msg}</b>);
       my $value = "";

       # Initialize with default values
       if (($key =~ /commonname/) && ($curValue eq "")) {
          # Extract hostname from system info
          my $sysinfo = $host->getSystemInfo();
          if ($sysinfo =~ /on machine (\S+)/) {
             $curValue = $1;
          }
       }

       if ($key =~ m/countryname/) {
          $value .= qq(<select name="$key">\n);
          # Violate sortedness to make common choices appear at top of list
          $value .= qq(  <option value="US">United States of America (US)</option>\n);
          $value .= qq(  <option value="ZZ">Unknown or unspecified country (ZZ)</option>\n);
          foreach my $country (sort keys %countryCodes) {
             my $code = $countryCodes{$country};
             my $selected = ($code eq $curValue) ? "selected" : "";
                  $value .= qq(  <option value="$code" $selected>$country ($code)</option>\n);
          }
          $value .= qq(</select>\n);
       }elsif ($key =~m/stateorprovincename/) {
          $value .= qq(<select name="$key">\n);
          foreach my $state (@stateList) {
             my $selected = ($state eq $curValue) ? "selected" : "";
             $value .= qq(  <option value="$state" $selected>$state</option>\n);
          }
          $value .= "</select>\n";
       }else {
          #Limit field length arbitrarily.
          my $maxLen = 40;
          if (defined($gCertInfo{$key}{maxLen})) {
             $maxLen = $gCertInfo{$key}{maxLen};
          }
          $value .= qq( <input type="text" name="$key" value="$curValue" maxlength="$maxLen" size="40"></td>\n);
       }
       
       $s .= toHTMLTableRow($name, $value);
    }

    $s .= toHTMLTableFooter();
    

    $s .=
	("<table border=0>\n" .
         "<tr>\n" .
         "<td>\n" .
         "<input type=\"hidden\" name=\"$RCom\" value=\"$RCom_SaveCert\">\n" .
         ($wizard ? "<input type=\"hidden\" name=\"wizard\" value=\"$wizard\">\n" : "") .
         "<p><input type=\"submit\" value=\" Save Changes \">\n" .
         "</form>\n".
         "</td>\n" . 
         "<td>\n" .
	 "<form name=\"cancel\" method=\"POST\" action=\"$url_self\">\n" .
	 "<input type=\"hidden\" name=\"$RCom\" value=\"$RCom_ReturnToLevelChooser\">\n" .
         ($wizard ? "<input type=\"hidden\" name=\"wizard\" value=\"$wizard\">\n" : "") .
         "<p><input type=\"submit\" value=\"Cancel Changes\">\n" .
         "</form>\n" .
         "</td>\n" .
         "</tr>\n" .
         "</table>\n" .

         "<hr>\n"
	 );

    return($s);
}



# This function displays a page to make the user wait until
# the httpd server has restarted.
sub toWaitingForServerHTML($$$$$) {
   my ($cgi, $host, $nextPage, $title, $msg) = @_;
   my $url_self = $cgi->url(-relative=>1);

   my $title = "Waiting for completion";
   my @path = @htmlPath;
   push @path, [ $htmlTitle, $url_self ];

   my $s = 
       toHTMLPath(\@path, $title) . 
       toHTMLSystemInfo($host) . 
       toHTMLWarnings();
   
   $s .= <<END_HTML;
<meta http-equiv="Refresh" content="12;url=$nextPage">
<p class="txt">Please wait for the operation to complete.
$msg, click <a href="$nextPage">here</a> to continue.
</p>
END_HTML

   return $s;
}

#
# setSecurityLevelHTML
#
# Activate the chosen security level. (or display the custom properties
# page if the custom level was chosen).
#
sub setSecurityLevelHTML($$) {
    my ($cgi,$host) = @_;
    my $wizard = $cgi->param("wizard");
    my %options = ( $ssh => "yes", 
                    $encryptMUI => "yes", 
                    $encryptRUI => "yes", 
                    $ftp => "no", 
                    $telnet => "no",
                    $portmap => "no");

    my $secLevelChoice = $cgi->param($levelChoice);

    if (!($secLevelChoice =~ m/^($hiSecurity|$medSecurity|$lowSecurity)$/)) {
       $secLevelChoice = $hiSecurity;
       Warning("Got here when I shouldn't have\n");
    }

    if ($secLevelChoice < $hiSecurity) {
       $options{$telnet} = "yes";
       $options{$ftp} = "yes";
       $options{$portmap} = "yes";
    }
       
    if ($secLevelChoice < $medSecurity) {
       $options{$encryptMUI} = "no";
       $options{$encryptRUI} = "no";
    }

    $options{$levelChoice} = $secLevelChoice;

    activateChanges(\%options);

    # If we're in the wizard, advance to the next stage;
    # otherwise, return to the index page
    my $nextURL;
    if ($wizard) {
       $nextURL = toWizardURL($URL_NEXT, 1);
    } else {
       $nextURL = toWizardURL($URL_INDEX, 0);
    }

    return (toWaitingForServerHTML($cgi, $host, $nextURL, 
                                   "Applying security changes ...",
                                   "If this page doesn't automatically reload within 15 seconds"));
}

sub updateConfigVar($$$) {
   my ($cgi, $key, $options) = @_;

   $options->{$key} = ($cgi->param($key) eq "on") ?  "yes" : "no";
}

#
# saveCustomConfig()
#
# Make a serverd request to save the custom security settings,
# and activate the changes.
sub saveCustomConfig($$) {
    my ($cgi, $host) = @_;
    my $wizard = $cgi->param("wizard");
    my %options;

    updateConfigVar($cgi, $ssh, \%options);
    updateConfigVar($cgi, $telnet, \%options);
    updateConfigVar($cgi, $ftp, \%options);
    updateConfigVar($cgi, $encryptMUI, \%options);
    updateConfigVar($cgi, $encryptRUI, \%options);
    updateConfigVar($cgi, $portmap, \%options);
    $options{$levelChoice} = $custSecurity;
  
    activateChanges(\%options);

    # If we're in the wizard, advance to the next stage;
    # otherwise, return to the index page
    my $nextURL;
    if ($wizard) {
       $nextURL = toWizardURL($URL_NEXT, 1);
    } else {
       $nextURL = toWizardURL($URL_INDEX, 0);
    }

    return (toWaitingForServerHTML($cgi, $host, $nextURL, 
                                   "Applying security changes ...",
                                   "If this page doesn't automatically reload within 15 seconds"));
}

sub consoleOSSecurity($){
   my ($checked) = @_;
   my $s = "";
   $s .= 
        ("<h3>Console Operating System</h3>\n"  .
        "<table border=0>\n" .
        "<tr>\n" .
        "<td><input type=\"checkbox\" name=\"$ssh\" $checked->{$ssh}></td>" .
        "<td class='txt'>ssh enabled on console operating system</td>" .
        "</tr>\n" .
        "<tr>\n" .
        "<td><input type=\"checkbox\" name=\"$telnet\" $checked->{$telnet}></td>" .
        "<td class='txt'>telnet enabled on console operating system*</td>" .
        "</tr>\n" .
        "<tr>\n" .
        "<td><input type=\"checkbox\" name=\"$ftp\" $checked->{$ftp}></td>" .
        "<td class='txt'>ftp enabled on console operating system*</td>" .
        "</tr>\n" .
        "<tr>\n" .
        "<td><input type=\"checkbox\" name=\"$portmap\" $checked->{$portmap}></td>" .
        "<td class='txt'>portmap enabled on console operating system**</td>" .
        "</tr>\n" .
        "</table>\n" .
        "<p class='txt'>*Note: These protocols send unencrypted passwords over the net.  Enabling may\n" .
        "compromise the security of your server.</p>\n" .
        "<p class='txt'>**Note: This service must be enabled in order to\n" .
        "mount NFS file systems on the Console OS, but creates a\n" .
        "potential security vulnerability.</p>\n" 
        );
   return ($s);
}

sub remoteUISecurity($){
   my ($checked) = @_;
   my $s = "";
   $s .= 
        ("<h3>Remote Console</h3>\n"  .
        "<table border=0>\n" .
        "<tr>\n" .
        "   <td><input type=\"checkbox\" name=\"$encryptRUI\" $checked->{$encryptRUI}></td>\n" .
        "   <td class='txt'>Use Secure Sockets Layer (SSL) to encrypt remote console connections</td>\n" .
        "</tr>\n");
   $s .=   "</table>\n";
   return ($s);
}

sub managementUISecurity($$){
   my ($checked, $cgi) = @_;
   my $url_self = $cgi->url(-relative=>1);
   my $wizard = $cgi->param("wizard");
   my $editCertURL = toWizardURL("$url_self?$RCom=$RCom_EditCert", $wizard);
   my $s = "";
   $s .= 
        ("<h3>Management Interface</h3>\n" .
        "<table border=0>\n" .
        "<tr>\n" .
        "<td valign=\"top\"><input type=\"checkbox\" name=\"$encryptMUI\" $checked->{$encryptMUI}></td>" .
        "<td class='txt'>Use Secure Sockets Layer (SSL) to encrypt management interface sessions<br>\n" .
        "    To generate an optional, custom x509 certificate, click <a href=\"$editCertURL\">here</a></td>" .
        "</tr>\n" .
#        "<tr>\n" .
#        "<td></td>\n" .
#        "  <td>" .
#	"  <input type=\"hidden\" name=\"$RCom\" value=\"$RCom_EditCert\">\n" . 
#	"  <input type=\"submit\" value=\"$URL_NAME_CERTIFICATE>\">\n" .
#	"</td>\n" .
#        "</tr>\n" .
        "</table>\n" 
        );
   return ($s);
}

#
# toSecurityLevelHTML
#
# Displays the page that lets the user choose which security level 
# (hi, med, lo, cust) the server will use.
#
sub toSecurityLevelHTML($$) {
    my ($cgi, $host) = @_;
    my $url_self = $cgi->url(-relative=>1);
    my $wizard = $cgi->param("wizard");
    my %options;

    loadSecurityOptions(\%options,"host");

    #grab the default security level
    my $activeSecurity = $hiSecurity; # Default if none specified
    if (($options{$levelChoice} =~ 
         /^($hiSecurity|$lowSecurity|$medSecurity|$custSecurity)$/)) {
        $activeSecurity = $options{$levelChoice};
    }

    my %heading = ($hiSecurity => "High",
                   $medSecurity => "Medium",
                   $lowSecurity => "Low",
                   $custSecurity => "Custom");

    foreach my $security (keys %heading) {
        my $onclick = "";
        my $url = "$url_self?$levelChoice=$security";
        if ($security == $custSecurity) {
            $url .= "&$RCom=$RCom_ReturnToLevelChooser";
        } else {
            $url .= "&$RCom=$RCom_SetLevel";
            $onclick = qq(onclick="return doConfirmSave();");
        }
        $url = toWizardURL($url, $wizard);
        $heading{$security} = 
            qq(<a href="$url" $onclick>) .
            qq(<b>$heading{$security}<br>Security</b>) .
            qq(</a>);
        if ($security == $activeSecurity) {
            $heading{$security} .= qq(<br><b>[Selected]</b>);
        }
    }

    my $s = 
        toHTMLPath(\@htmlPath, $htmlTitle) . 
        toHTMLSystemInfo($host) . 
        toHTMLWarnings();

    $s .= <<END_HTML;
<script type="text/javascript">
  <!--
  function doConfirmSave() {
     return confirm("Are you sure you want to save the security configuration?");
  }
  function doSuccess() {
     return confirm("success!");
  }
  // -->
</script>

<h3>Security Configuration</h3>
<p class="txt">
Select one of the following security configurations:
</p>
END_HTML

    my $align = qq(align="center");
    $s .= toHTMLTableHeader();

    $s .= toHTMLTableRow([$heading{$hiSecurity}, $align], <<END_HTML);
    <p>
    <ul>
      <li>Management Interface and Remote Console network traffic
          are encrypted using SSL (Secure Sockets Layer).
      <li>ssh (Secure Shell) access to the Console OS is enabled.
      <li>telnet and ftp access to the Console OS are disabled.
      <li>portmap service on the Console OS is disabled.
    </ul>
    </p>
END_HTML

    $s .= toHTMLTableRow([$heading{$medSecurity}, $align], <<END_HTML);
    <p>
    <ul>
      <li>Management Interface and Remote Console network traffic
          are encrypted using SSL.
      <li>ssh, telnet, and ftp access to the Console OS are enabled.
      <li>portmap service on the Console OS is enabled.
    </ul>
    </p>
END_HTML

    $s .= toHTMLTableRow([$heading{$lowSecurity}, $align], <<END_HTML);
    <p>
    <ul>
      <li>Management Interface and Remote Console network traffic
          are unencrypted.
      <li>ssh, telnet, and ftp access to the Console OS are enabled.
      <li>portmap service on the Console OS is enabled.
    </ul>
    </p>
END_HTML

    $s .= toHTMLTableRow([$heading{$custSecurity}, $align], <<END_HTML);
    <p>
    <ul>
      <li>Allows customized security settings.
    </ul>
    </p>
END_HTML

    $s .= toHTMLTableFooter();

    if ($cgi->param("wizard")) {
       $s .= toHTMLNavWizard( [ $URL_PREV, $URL_NAME_PREV ],
                              [ $URL_NEXT, $URL_NAME_NEXT ] );
    }
    
    return $s;
}

sub toCustomSecurityHTML($$) {
    my ($cgi, $host) = @_;
    my $url_self = $cgi->url(-relative=>1);
    my $wizard = $cgi->param("wizard");

    my $title = "Customize Security Configuration";
    my @path = @htmlPath;
    push @path, [ $htmlTitle, $url_self ];

    my $s = 
        toHTMLPath(\@path, $title) . 
        toHTMLSystemInfo($host) . 
        toHTMLWarnings();

    $s .= <<END_HTML;
<script type="text/javascript">
  <!--
  function doConfirmSave() {
     return confirm("Are you sure you want to change the security configuration?");
  }
  function doSuccess() {
     return confirm("success!");
  }
  // -->
</script>
<form name="save" method="POST" action="$url_self", onSubmit="return doConfirmSave()" onError="return doSuccess()">
END_HTML

    my %checked;
    my %options;
    loadSecurityOptions(\%options,"host");

    foreach my $key ($ssh, $telnet, $ftp, $encryptRUI, $encryptMUI, $portmap) {
       if ($options{$key} =~ m/(Y|T|1)/i) {
         $checked{$key} = "CHECKED";
       } else {
         $checked{$key} = "";
       }
    }


    $s .= managementUISecurity(\%checked, $cgi);
    $s .= "<hr>\n";
    $s .= remoteUISecurity(\%checked);
    $s .= "<hr>\n";
    $s .= consoleOSSecurity(\%checked);


    $s .=("<br><hr><br>\n" .
	"<table border=0>\n" .
         "<tr>\n" .
         "<td>\n" .
	 "<input type=\"hidden\" name=\"$RCom\" value=\"$RCom_Save\">\n" .
         ($wizard ? "<input type=\"hidden\" name=\"wizard\" value=\"$wizard\">\n" : "") .
         "<input type=\"submit\" value=\" Save Changes \">\n" .
         "</form>\n".
         "</td>\n" . 
         "<td>\n" .
	 "<form name=\"cancel\" method=\"POST\" action=\"$url_self\">\n" .
         ($wizard ? "<input type=\"hidden\" name=\"wizard\" value=\"$wizard\">\n" : "") .
         "<input type=\"submit\" value=\"Cancel Changes\">\n" .
         "</form>\n" .
         "</td>\n" .
         "</tr>\n" .
         "</table>\n"
	 );
    return($s);
}


#XXX unused delete
sub toDoneCertificateHTML($$) {
    my ($cgi, $host) = @_;
    my $url_self = $cgi->url(-relative=>1);

    my $title = "Certificate Saved";
    my @path = @htmlPath;
    push @path, [ $htmlTitle, $url_self ];

    my $s = 
        toHTMLPath(\@path, $title) . 
        toHTMLSystemInfo($host) . 
        toHTMLWarnings();

# eventually add this check back in
#    if ($host->querySecurityOption($encryptMUI) =~ m/Y|T|1/i) {
       $s .= "<p class='txt'>The new certificate is now being used.</p>\n\n";
#    } else {
#       $s .= 
#           "<p class='txt'>A New certificate has been created, but is not being used " .
#           "because SSL encryption for the management interface is not enabled</p>\n";
#    }

    if ($cgi->param("wizard")) {
       $s .= toHTMLNavWizard( [ "$url_self?$RCom=$RCom_EditCert", $URL_NAME_CERTIFICATE ],
                              [ $URL_NEXT, $URL_NAME_NEXT ] );
    }

    return($s);
}

#
# loadSecurityOptions
#
# make a serverd request to load all of the security.$subfield entries
# from the host pref database.
#
sub loadSecurityOptions($$) {
   my ($optRef,$subfield) = @_;
   my $in = VMware::HConfig::SSHostRemote::makeServerdRequest("Security_GetConfig");
   my $key;
   my $i = 0;

   $in->setValue(".in.pattern","security.$subfield");
   my $out = VMware::HConfig::SSHostRemote::doServerdRequest($hostname, $port, $user, $password, $in);
   if (!$out) {
      Warning("Serverd request failed. Internal error.");
      $in->dispose();
      return;
   }

   my $rc = VMware::HConfig::SSHostRemote::getReturnCode($out);
   if (!$rc) {
      &VMware::HConfig::SSHostRemote::ServerdWarning($out);
   }

   while ($key = $out->getValue(".out.pref[$i].name")) {
      $optRef->{$key} = $out->getValue(".out.pref[$i].value");
      $i++;
   } 

   $in->dispose();
   $out->dispose();
}


# 
# activateChanges
#
# make a serverd request to save, and activate the new security settings
#
sub activateChanges($) {
   my ($optRef) = @_;
   my $in = VMware::HConfig::SSHostRemote::makeServerdRequest("Security_ActivateConfig");
   my $key;
   my $i = 0;

   foreach $key (keys %{$optRef}) {
      $in->setValue(".in.pref[$i].name", $key);
      $in->setValue(".in.pref[$i].value", $optRef->{$key});
      $i++;
   }

   my $out = VMware::HConfig::SSHostRemote::doServerdRequest($hostname, $port, $user, $password, $in);
   if (!$out) {
      Warning("Serverd request failed. Internal error.");
      $in->dispose();
      return;
   }

   my $rc = VMware::HConfig::SSHostRemote::getReturnCode($out);
   if (!$rc) {
      &VMware::HConfig::SSHostRemote::ServerdWarning($out);
   }

   $in->dispose();
   $out->dispose();
}

#
# Make a serverd request to create a new certificate on the server
#
sub createCertificate($$) {
   my ($which, $optRef) = @_;
   my $in = VMware::HConfig::SSHostRemote::makeServerdRequest("Security_CreateCertificate");
   my $key;
   my $i = 0;

   $in->setValue(".in.certname",$which);
   foreach $key (keys %{$optRef}) {
      $in->setValue(".in.pref[$i].name", $key);
      $in->setValue(".in.pref[$i].value", $optRef->{$key});
      $i++;
   }

   my $out = VMware::HConfig::SSHostRemote::doServerdRequest($hostname, $port, $user, $password, $in);
   if (!$out) {
      Warning("Serverd request failed. Internal error.");
      $in->dispose();
      return;
   }

   my $rc = VMware::HConfig::SSHostRemote::getReturnCode($out);
   if (!$rc) {
      &VMware::HConfig::SSHostRemote::ServerdWarning($out);
   }

   $in->dispose();
   $out->dispose();
}


sub saveCertificate($$) {
   my ($cgi, $host) = @_;
   my $wizard = $cgi->param("wizard");
   my $url_self = $cgi->url(-relative=>1);
   my %options;

   foreach my $key (keys %gCertInfo) {
      my $val = $cgi->param($key);
      next if (!defined($val));
      $options{$key} = $val;
   }

   createCertificate("management-ui", \%options);
   
   my $msg = "If this page doesn't automatically reload within 15 seconds";
#   if ($host->querySecurityOption($encryptMUI) =~ m/Y|T|1/i) {
#     $msg = "If you are not prompted to accept the new certificate within 15 seconds";
#   }

   my $nextURL = "$url_self?$RCom=$RCom_ReturnToLevelChooser";
   $nextURL = toWizardURL($nextURL, $wizard);
   return (toWaitingForServerHTML($cgi, $host, $nextURL, 
            "Generating New Certificate ...",$msg));
}


sub handleRequest($$) {
    my ($cgi, $host) = @_;
    my $s = "";
    
    # XXX Hack
    $gPanicCallback = \&PanicCallbackHTML;


    my $req = $cgi->param($RCom);

    if ( defined($req)) {
       if ($req eq $RCom_Save ) {
          return(saveCustomConfig($cgi, $host));
       }elsif ($req eq $RCom_SetLevel) {
          return(setSecurityLevelHTML($cgi, $host));
       }elsif ($req eq $RCom_SaveCert){
          return(saveCertificate($cgi, $host));
       }elsif ($req eq $RCom_EditCert) {
          return(editCertificateHTML($cgi, $host, 0));
       }elsif ($req eq $RCom_ReturnToLevelChooser) {
          return(toCustomSecurityHTML($cgi, $host));
       }else {
          return
              toHTMLPath(\@htmlPath, $htmlTitle) .
              toHTMLSystemInfo($host) . 
              toHTMLNotImplemented($req);
       }
    }

    return(toSecurityLevelHTML($cgi, $host));
}


sub handler {
    # Delete the PATH to get by the tainted security requirements of Perl
    delete($ENV{'PATH'}); 

    umask 022;   # Create files that are not writable by group
    
    &clearWarnings();

    my $cgi = new CGI( shift( ) );
    my %credentials = hash_credentials( );

    $hostname = undef;
    $port = 0;
    $user = $credentials{ "l" };
    $password = $credentials{ "m" };

    my $s = toHTMLHeader($htmlTitle, $user);

    if ($user ne 'root') {
       my $url_self = $cgi->url(-relative=>1);
       $s .= toHTMLPath(\@htmlPath, $htmlTitle) . toHTMLNotRoot($url_self);
       goto footer;
    }
    
    my $host = VMware::HConfig::SSHostRemote->new($hostname, 0, $user, $password);

    if (!defined($host)) {
       $s .= toHTMLPath(\@htmlPath, $htmlTitle) . toHTMLHostFailure($htmlTitle);
       goto footer;
    }

    $s .= handleRequest($cgi, $host);

footer:
    $s .= toHTMLFooter();
    return ($s, %MimeHeaders);
}

1;
