# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::SS::SecuritySettings;
# File: VMware/Management/SS/SecuritySettings.pm

use strict;

use VMware::HConfig::SSHostRemote ();
use VMware::HConfig::MFSFileSystemRemote;
use VMware::HConfig::RawDisk;
use VMware::HConfig::Constant qw($gVmkDumpPartSize $gVmkDumpPartType
				 @gPartitionCreateTypes $gVMFSPartType
				 $gVMFSMaxKBlocks $gVMFSBlockSizeLimit);
use VMware::Management::SS::ESXConfig;
use VMware::Management::Util ();

use CGI ();
my $q;

my $hostname;
my $port;
my $user;
my $pass;

my $https =  "muissl";
my $rcssl =  "ruissl";
my $ssh = "sshd";
my $telnet =  "telnet";
my $ftp =  "ftp";
my $nfs = "portmap";

# Defaults in the absence of config data.
my %options = ($https  => "yes",
               $rcssl  => "yes",
               $ssh    => "yes",
               $ftp    => "no",
               $telnet => "no",
               $nfs    => "no");

# ---------------------------------------------------------------------------

sub handler {
   $q = new CGI(shift());

   my %credentials = VMware::Management::Util::hash_credentials();
   $user = $credentials{"l"};
   $pass = $credentials{"m"};
   my $host = VMware::HConfig::SSHostRemote->new($hostname,
                                                 $port,
                                                 $user,
                                                 $pass);

   if (! defined($host)) {
      my $data = error("Could not create remote host object.");
      return html($data);
   }

   if ($q->param("op") =~ /^save$/i) {
      return save($host);
   }

   return data($host);
}


# ---------------------------------------------------------------------------

sub data {
   my $host = shift();
   my $data = shift() || "";

   $data .= "function SecCfg(https,rcssl,ssh,telnet,ftp,nfs) {" .
      "this.https=https;" .   # Whether or not Management UI SSL is required
      "this.rcssl=rcssl;" .   # Whether or not Remote Console SSL is required
      "this.ssh=ssh;" .       # Whether or not SSH is enabled
      "this.telnet=telnet;" . # Whether or not FTP is enabled
      "this.ftp=ftp;" .       # Whether or not Telnet is enabled
      "this.nfs=nfs;}$/";     # Whether or not NFS is enabled

   $data .= "lvls={high:new SecCfg(true,true,true,false,false,false)," .
      "normal:new SecCfg(true,true,true,true,true,true)," .
      "low:new SecCfg(false,false,true,true,true,true)};$/";

   $data .= levelData($host);

   return html($data);
}


# ---------------------------------------------------------------------------

sub getCurrentLevel {
   my $data = "";

   my $in = VMware::HConfig::SSHostRemote::makeServerdRequest("Security_GetConfig");
   my $key;

   $in->setValue(".in.pattern","security.host");
   my $out = VMware::HConfig::SSHostRemote::doServerdRequest($hostname,
                                                             $port,
                                                             $user,
                                                             $pass,
                                                             $in);

   if (! $out) {
      $data .= error("Serverd request failed.");
      $in->dispose();
      return $data;
   }

   my $rc = VMware::HConfig::SSHostRemote::getReturnCode($out);
   if (! $rc) {
      for ($out->listElementNames('.messages')) {
         $data .= error($out->getValue(".messages.$_"));
      }
      return $data;
   } else {
      my $i = 0;
      while ($key = $out->getValue(".out.pref[$i].name")) {
         $options{$key} = $out->getValue(".out.pref[$i].value");
         $i++;
      }

   }

   $in->dispose();
   $out->dispose();

   return $data;
}


# ---------------------------------------------------------------------------

sub levelData {
   my $host = shift();
   my $data = "";

   # Make sure %options is current.
   $data .= getCurrentLevel();

   my $str = "";
   for ($https, $rcssl, $ssh, $telnet, $ftp, $nfs) {
      $str .= ($str ? "," : "") . ($options{$_} eq "yes" ? "true" : "false");
   }
   $data .= "var lvl=new SecCfg($str);";

   return $data;
}


# ---------------------------------------------------------------------------

sub save {
   my $host = shift();
   my $data = "";

   my $in = VMware::HConfig::SSHostRemote::makeServerdRequest("Security_ActivateConfig");
   my $key;
   my $i = 0;

   # The MUI talks about allowing unencrypted web and remote console sessions
   # rather than enabling encryption. The phrasing (IMO) makes more sense to a
   # human reader, despite the fact that it complicates things here. - mikol
   # March 16, 2003 4:50 PM
   $options{$https}  = $q->param("https")  || "yes"; # If unspecified
   $options{$rcssl}  = $q->param("rcssl")  || "yes"; # If unspecified
   $options{$ssh}    = $q->param("ssh")    || "no";  # If unspecified
   $options{$telnet} = $q->param("telnet") || "no";  # If unspecified
   $options{$ftp}    = $q->param("ftp")    || "no";  # If unspecified
   $options{$nfs}    = $q->param("nfs")    || "no";  # If unspecified

   for (keys(%options)) {
      $in->setValue(".in.pref[$i].name", $_);
      $in->setValue(".in.pref[$i].value", $options{$_});
      $i++;
   }

   my $out = VMware::HConfig::SSHostRemote::doServerdRequest($hostname,
                                                             $port,
                                                             $user,
                                                             $pass,
                                                             $in);

   if (! $out) {
      $data .= error("Serverd request failed.");
      $in->dispose();
      return html($data);
   }

   my $rc = VMware::HConfig::SSHostRemote::getReturnCode($out);
   if (! $rc) {
      for ($out->listElementNames('.messages')) {
         $data .= error($out->getValue(".messages.$_"));
      }
   }

   $in->dispose();
   $out->dispose();

   return data($host, $data);
}


# ---------------------------------------------------------------------------

sub error {
   my $msg = shift();
   VMware::Management::Util::trace($msg, "error");
   return "err.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------

sub warning {
   my $msg = shift();
   VMware::Management::Util::trace($msg, "warning");
   return "wrn.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------
#
# jsStringLiteral --
#
#      Make sure that $msg is safe for consumption in a JavaScript program.
#
# ---------------------------------------------------------------------------

sub jsStringLiteral {
   my $msg = shift();
   $msg =~ s/\\/\\\\/g;
   $msg =~ s/("|')/\\$1/g;
   my $lf = $/;
   $msg =~ s/$lf/\\n/g;

   return qq{"$msg"};
}


# ---------------------------------------------------------------------------

sub html {
   my $data = shift();
   my $cb = shift() || "if (parent.opCb) parent.opCb(self);";

   my $html = qq[<html><head><title></title><script>$/] .
      qq[var err=[];var wrn=[];$/] .
      qq[$data$/</script></head><body onload="$cb"></body></html>];

   return($html,
          "Pragma"        => "no-cache",
          "Cache-Control" => "no-cache",
          "Content-Type"  => "text/html; charset=utf-8");
}

1;