# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::SS::SwapFile;
# File: VMware/Management/SS/SwapFile.pm

use strict;

use VMware::Control '1.00';
use VMware::Control::Server ();
use VMware::Management::Util qw( trace hash_credentials );

use VMware::HConfig::SSHostRemote;
use VMware::HConfig::Constant qw( $URL_VIEWPART $URL_LOGOUT 
                                  $URL_WIZ $URL_REBOOT );
use VMware::ExtHelpers qw( &p_number &Warning &getWarnings &clearWarnings );

use VMware::Management::SS::ESXConfig;
my $htmlTitle = "Swap File Configuration";

my $CFG_SWAPFILE = "swapfile.";

my $URL_PREV = $URL_VIEWPART;
my $URL_NAME_PREV = "Edit Partitions";
my $URL_NEXT = "${URL_LOGOUT}?n=$URL_WIZ";
my $URL_NAME_NEXT = "VM Wizard";

my $RCom = "request";
my $RCom_Update = "update";

use CGI;

# Write swap file information to ConfigDB
sub updateSwapFile($$) {
   my ($cgi, $host) = @_;
   my $configDB = $host->getConfigDB();

   my $volume = $cgi->param("volume");
   my $filename = $cgi->param("name");
   my $size = $cgi->param("size");
   my $enable = $cgi->param("enable");

   # Validate arguments
   if ($volume eq "") {
      Warning("Must specify VMFS volume for swap file.");
      return 0;
   }
   if ($filename !~ /.vswp$/) {
      Warning("Swap file name must end in '.vswp'.");
      return 0;
   }
   if (($size !~ /^\d+$/) || ($size <= 0)) {
      Warning("Swap file size must be a positive integer.");
      return 0;
   }

   # Start immediately if requested; don't save configuration if it fails
   my $start = $cgi->param("start");
   if (defined($start) && ($start eq "yes")) {
      my $rc = $host->startSwapper($volume, $filename, $size);
      if ($rc == 0) {
         return 0;
      }
   }

   $configDB->set($CFG_SWAPFILE . "enable", $enable);
   $configDB->set($CFG_SWAPFILE . "volume", $volume);
   $configDB->set($CFG_SWAPFILE . "filename", $filename);
   $configDB->set($CFG_SWAPFILE . "sizeMB", ${size});

   if (!$host->commitConfigDB()) {
      Warning("Failed to commit changes to config database.\n");
   }

   return 1;
}

sub toSwapFileHTML($$) {
   my ($cgi, $host) = @_;
   my $url_self = $cgi->url(-relative=>1);
   my $wizard = ($cgi->param("wizard"));
   my $configDB = $host->getConfigDB();

   # Retrieve active swap file status, convert from KB to MB
   my ($currSize, $currFree, $currUsed, $currUnrsvd, $currVol, $currFile) = $host->getSwapInfo();
   $currSize = p_number($currSize / 1024);
   $currFree = p_number($currFree / 1024);
   $currUsed = p_number($currUsed / 1024);

   # Retrieve saved swap file configuration
   my $enable = $configDB->get($CFG_SWAPFILE . "enable");
   my $volume = $configDB->get($CFG_SWAPFILE . "volume");
   my $name = $configDB->get($CFG_SWAPFILE . "filename");
   my $size = $configDB->get($CFG_SWAPFILE . "sizeMB");

   my $suggestedSize = int($host->getTotalMemSize() / 1024);

   my $s = <<END_HTML;
<h3>Swap File Configuration</h3>

<script language="JavaScript">
   function BrowseVMFS(size) {
      browseWin = window.open('/vmfsbrowse?extension=.vswp&defaultName=SwapFile&defaultSize=' + size, 'test', 'width=460,height=400,resizable,scrollbars');
   }
   function BrowseHandler(clientData, partition, filename, size) {
      window.document.swap.volume.value = partition;
      window.document.swap.name.value = filename;
      window.document.swap.size.value = size;
   }
</script>

<p class="txt">
VMware ESX Server uses standard demand paging techniques to enable
your virtual machines to use more memory than is physically available.
This page allows you to create and configure a swap file to be used
to store pages that are not currently resident in physical memory.
</p>

<p class="txt">
<u>Suggested settings</u>: We recommend that you create a swap file that 
is at least as large as the physical memory of your server (approximately
$suggestedSize&nbsp;MB).
</p>

END_HTML

   if ($currSize > 0) {
      $s .= <<END_HTML;
<p class="txt">
This machine has an active swap file:
<ul>
<li class="txt"><b>VMFS volume</b>: $currVol
<li class="txt"><b>File name</b>: $currFile
<li class="txt"><b>Total swap space</b>: $currSize&nbsp;MB
<li class="txt"><b>Unused swap space</b>: $currFree&nbsp;MB
</ul>
</p>

<p class="txt">
Any changes to the swap file configuration will take effect 
when you <a href="${URL_REBOOT}?message=Swap%20file%20changes">
reboot the machine</a>.
</p>
END_HTML
   } else {
      $s .= <<END_HTML;
<p class="txt">
This machine does not have an active swap file.  Selecting the 
"Enable immediately" option will allow you to begin using the swap file
without rebooting.  Otherwise, changes to the swap file configuration 
will take effect when you <a href="/reboot">reboot the machine</a>.
</p>
END_HTML
   }

   $s .= 
       qq(<p class="txt">Click "Update Swap Configuration" ) .
       qq(to save the new settings.</p>) .
       qq(<form name="swap" method="POST" action="$url_self">\n) .
       toHTMLTableHeader(["&nbsp;", "width=33%"], 
                           ["Saved Settings", "width=33%"], 
                           ["New Settings", "width=33%"]);

   my ($field, $input, $default, $y, $n);

   # Volume select
   $field = "VMFS Volume";
   # Choosing the default volume:
   # * If a swapfile is already specified, use it as the default
   # * If not, choose the first volume with enough space to hold
   #   a swapfile of the suggested size
   # * If none have enough space, choose the volume with the most
   #   free space and reduce the suggested size accordingly
   $default = undef;
   if (defined($volume) && ($volume ne "")) {
      $default = $volume;
   } else {
      $volume = "&nbsp;";
      my $maxSize = 0;
      foreach my $vmfs (@{ $host->getAllMFS() }) {
         my $freeSize = $vmfs->getFreeCapacity();
         if ($freeSize >= $suggestedSize) {
            $default = $vmfs->getVmkName();
            $maxSize = $suggestedSize;
            last;
         } elsif ($freeSize > $maxSize) {
            $default = $vmfs->getVmkName();
            $maxSize = $freeSize;
         }
      }
      $suggestedSize = $maxSize;
   }
   $input = qq(<select name="volume">\n);
   foreach my $vmfs (@{ $host->getAllMFS() }) {
      my $freecap = $vmfs->getFreeCapacity();
      my $lname = $vmfs->getLogName();
      my $vname = $vmfs->getVmkName();
      my $vlabel;
      my $option;
      if ($lname eq "") {
         $option = $vname;
         $vlabel = qq($vname -- $freecap MB Free);
      } else {
         $option = $lname;
         $vlabel = qq($lname ($vname) -- $freecap MB Free);
      }
      my $selected = "";
      if (defined($default) && (($vname eq $default) || ($lname eq $default))) {
         $selected = "selected";
      }
      $input .= qq(  <option value="$option" $selected>$vlabel</option>\n);
   }
   $input .= qq(</select>\n);
   $s .= toHTMLTableRow("<b>$field</b>", $volume, $input);

   # Filename input + browse
   $field = "File name";
   if (defined($name) && ($name ne "")) {
      $default = $name;
   } else {
      $name = "&nbsp;";
      $default = "SwapFile.vswp";
   }
   $input = qq(<table cellpadding="0" cellspacing="0"><tr><td>);
   $input .= qq(<input type="text" name="name" value="$default" size="25">\n);
   $input .= qq(</td><td>&nbsp;</td><td>);
   $input .= toHTMLButton("xxs", "Browse", "javascript:BrowseVMFS($suggestedSize)");
   $input .= qq(</td></tr></table>);
   $s .= toHTMLTableRow("<b>$field</b>", $name, $input);

   # Swap space input
   $field = "Swap space";
   if (defined($size) && ($size ne "")) {
      $default = $size;
      $size .= "&nbsp;MB";
   } else {
      $default = $suggestedSize;
      $size = "&nbsp;";
   }
   $input = qq(<input type="text" name="size" value="$default" size="5">);
   $input .= qq(&nbsp;MB\n);
   $s .= toHTMLTableRow("<b>$field</b>", "$size", $input);

   # Enable on startup select
   $field = "Enable on startup";
   if (defined($enable) && ($enable ne "")) {
      ($y, $n, $enable) = 
          $enable eq "yes" ? ("selected", "", "Yes") : ("", "selected", "No");
   } else {
      ($y, $n, $enable) = ("selected", "", "&nbsp;");
   }
   $input = 
       qq(<select name="enable">\n) .
       qq(  <option value="yes" $y>Yes</option>\n) .
       qq(  <option value="no" $n>No</option>\n) .
       qq(</select>);
   $s .= toHTMLTableRow("<b>$field</b>", $enable, $input);

   # Enable immediately select (only if swapper isn't currently enabled)
   if ($currSize == 0) {
       $field = "Enable immediately";
       my $start = $cgi->param("start");
       if (defined($start) && ($start ne "")) {
          ($y, $n) = $start eq "yes" ? ("selected", "") : ("", "selected");
       } else {
          ($y, $n) = ("selected", "");
       }
       $input = 
          qq(<select name="start">\n) .
          qq(  <option value="yes" $y>Yes</option>\n) .
          qq(  <option value="no" $n>No</option>\n) .
          qq(</select>);
       $s .= toHTMLTableRow("<b>$field</b>", "&nbsp;", $input);
   }

   $s .= toHTMLTableFooter();

   $s .= 
       qq(<input type="hidden" name="$RCom" value="$RCom_Update">\n) .
       ($wizard ? qq(<input type="hidden" name="wizard" value="$wizard">\n) : "") .
       qq(<input type="submit" value="Update Swap Configuration">\n) .
       qq(</form>\n);
   
   if ($wizard) {
      # Force user to log out before proceeding to VM Wizard
      $s .= <<END_HTML;
<script type="text/javascript">
  <!--
  function doConfirmNextWiz() {
     alert("You are currently logged in as root.  We suggest that " +
           "you create virtual machines as a user\\n" +
           "other than root.  As a result, you are about to be " +
           "logged out.  When you log back in, you will\\n" +
           "be taken to the page where you may create " +
           "a new virtual machine.");
  }
  // -->
</script>
END_HTML
      $s .= toHTMLNavWizard( [ $URL_PREV, $URL_NAME_PREV ],
                             [ $URL_NEXT, $URL_NAME_NEXT, "return doConfirmNextWiz()"] );
   }

   return $s;
}


sub handler {
   # Delete the PATH to get by the tainted security requirements of Perl
   delete($ENV{'PATH'}); 
   
   &clearWarnings();
   
   my $cgi = new CGI( shift( ) );
   
   my %credentials = hash_credentials( );
   my $user = $credentials{ "l" };
   my $password = $credentials{ "m" };
   
   my $s = toHTMLHeader($htmlTitle, $user) . toHTMLPath(\@htmlPath, $htmlTitle);
   
   if($user ne 'root') {
      my $url_self = $cgi->url(-relative=>1);
      $s .= toHTMLNotRoot($url_self);
      goto footer;
   }
   
   my $host = VMware::HConfig::SSHostRemote->new(undef, 0, $user, $password);
   
   if(!defined($host)) {
      $s .= toHTMLHostFailure($htmlTitle);
      goto footer;
   }

   my $req = $cgi->param($RCom);
   if (defined($req) && ($req eq $RCom_Update)) {
      my $rc = updateSwapFile($cgi, $host);
      if ($rc == 0) {
         Warning("Swap file configuration unchanged.");
      }
   }

   $s .= toHTMLSystemInfo($host) . toHTMLWarnings();
   $s .= toSwapFileHTML($cgi, $host);
   
 footer:
   $s .= toHTMLFooter();
   return ($s, %MimeHeaders);
}

1;
