#!/usr/bin/perl
# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::SS::VMDelete;
# File: VMware/Management/VMDelete.pm

use strict;

use VMware::Control '1.00';
use VMware::Control::Server;
use VMware::Control::Profiler;
use VMware::Management::Util;
use VMware::DOMAccess;

use CGI;

use XML::DOM;
use XML::Sablotron;

my $p = VMware::Control::Profiler::getInstance("mui");

sub handler {
    my $stub = $p->profile();
    my $cgi = new CGI( shift( ) );

    my $req = $cgi->param('request');

    if( defined($req) && $req eq 'delete' ) {
	return(handleDelete($cgi));
    }

    return(handleDeleteInfo($cgi));
}

sub handleDeleteInfo($) {
    my $stub = $p->profile();
    my $cgi = shift;
    my $doc;
    my $root;
    my $xml;

    my $cfgName = $cgi->param('c');

    # Get the information needed to do a delete
    my($rc, $ret) = getDeleteInfo($cfgName);
    if( $rc == 0 ) {
	$doc = VMware::DOMAccess->new('server');
	$doc->setValue('.messages.msg', $ret);
	$doc->setAttribute('.messages.msg', 'severity', 'error');
	goto output_info;
    }

    # Add the error messages
    my $message_doc = $ret->getSubTree('.messages');

    # Build the XML information for the delete info so that the xsl
    # template can convert it into an HTML doc
    $doc = toDeleteInfoXML($ret, $message_doc);

    if( defined($message_doc) ) {
	$message_doc->dispose();
    }

    $ret->dispose();

output_info:
    my @results = toDeleteInfoHTML($doc);

    $doc->dispose();

    return(@results);
}


sub handleDelete($) {
    my $stub = $p->profile();
    my $cgi = shift;
    my $doc;

    my $cfgName = $cgi->param('c');
    my $timestamp = $cgi->param('timestamp');
    my @deleteDisks;

    my @params = $cgi->param();
    my $param;
    foreach $param (@params) {
	if( $param !~ /^d_(\S+)$/ ) {
	    next;
	}
	my $vmfsDiskName = $1;

	if( $cgi->param($param) ne 'del' ) {
	    next;
	}
	push(@deleteDisks, $vmfsDiskName);
    }

    # Build the request object
    $doc = VMware::DOMAccess->new("exec");
    $doc->setValue('.op', 'ESXVMDeleteEdit_DeleteVM');
    $doc->setValue('.in.config', $cfgName);
    $doc->setValue('.in.timestamp', $timestamp);
    my $disk;
    my $count = 0;
    foreach $disk (@deleteDisks) {
	$doc->setValue('.in.disks.disk[' . $count++ . ']', $disk);
    }
    my $encodedString = $doc->toEncodedString("");

    # Send the request to delete the VM
    my ($rc, $result) = doServerdRequest($encodedString);

    if( $rc == 0 ) {
	$doc = VMware::DOMAccess->new('server');
	$doc->setValue('.messages.msg', $result);
	$doc->setAttribute('.messages.msg', 'severity', 'error');
	goto output_delete;
    }

    # On success, we show a different page which redirects
    if( isSuccess($result) ) {
	return(toDeleteSuccessHTML());
    }

    # Some error occurred.  Display the error and show the delete info page.
    # Get the information needed to do a delete
    my($rc, $ret) = getDeleteInfo($cfgName);
    if( $rc == 0 ) {
	$doc = VMware::DOMAccess->new('server');
	$doc->setValue('.messages.msg', $ret);
	$doc->setAttribute('.messages.msg', 'severity', 'error');
	goto output_delete;
    }

    my @errors;
    my $delete_errors = $result->getSubTree('.messages');
    if( defined($delete_errors) ) {
	push(@errors, $delete_errors);
    }
    my $deleteinfo_errors = $ret->getSubTree('.messages');
    if( defined($deleteinfo_errors) ) {
	push(@errors, $deleteinfo_errors);
    }

    # Build the XML information for the delete info so that the xsl
    # template can convert it into an HTML doc
    $doc = toDeleteInfoXML($ret, $delete_errors, $deleteinfo_errors);

    my $err_node;
    foreach $err_node (@errors) {
	$err_node->dispose();
    }

    $ret->dispose();

output_delete:
    my @results = toDeleteInfoHTML($doc);

    $doc->dispose();

    return(@results);
}


# Builds the XML part of delete info page this is run through the XSLT processor
#
# First argument is the list of nodes for the object.  Second argument is the config file
# of the VM.  Remaining nodes are error nodes just appended to the object at the root node.
#
# Returns a new node representing the XML that can be passed through the XSLT processor
sub toDeleteInfoXML(@) {
    my $stub = $p->profile();
    my($in, @errors) = @_;

    my $out = VMware::DOMAccess->new('server');

    # Build the XML information for the delete info so that the xsl
    # template can convert it into an HTML doc
    my @names = $in->listElementNames("exec.out");
    my $n;
    foreach $n (@names) {
	my $s = $in->getSubTree("exec.out." . $n);
	$s = VMware::DOMAccess->newXML($s->getValue(""));
	$out->addSubTree("server", $s);
	$s->dispose();
    }

    # Add the username
    my %credentials = hash_credentials( );
    $out->setValue('.username', $credentials{'l'});

    # Add the error messages
    my $err_node;
    foreach $err_node (@errors) {
	if( !defined($err_node) ) {
	    next;
	}
	$out->addSubTree('.', $err_node);
    }

    return($out);
}


sub toDeleteInfoHTML($) {
    my $stub = $p->profile();
    my($doc) = @_;

    my $xsl = $ENV{ "style_DEFAULT" };

    if ( $ENV{ "vmware_DEBUG_LEVEL" } > 2 ) {
	string_to_file($doc->prettyPrint( ), &VMware::Management::Util::tmpdir() . "vmdelete.xml", 1);
    }

    my $processor = new XML::Sablotron();

    $processor->SetBase( "file://" . $ENV{ "vmware_BASEHREF" } );
    $p->start("XML::Sablotron::RunProcessor");
    $processor->RunProcessor( "arg:/template", "arg:/data", "arg:/result",
                              undef,
                              [ "template", file_to_string( $xsl ), "data", $doc->getValue("") ] );
    $p->end();
    return( $processor->GetResultArg("result"),
	    "Content-Type"  => "text/html; charset=iso-8859-1",
	    "Pragma"        => "no-cache",
	    "Cache-Control" => "no-cache" );
}


sub toDeleteSuccessHTML() {
    my $stub = $p->profile();
    return(qq(
	      <html>
	      <head>
	        <title>Delete Success Dialog</title>
	        <SCRIPT language="JavaScript">
	          function DeleteSuccess() \{
                    if (opener.main) {
                      if (opener.main.getUpdates) {
                        opener.main.getUpdates();
                      }
                    } else {
	              opener.location.href = "/overview";
                    }
	            self.close();
	          \}
	        </SCRIPT>
	        </head>
	        <body bgcolor="#FFFFE6" onload="DeleteSuccess()">
                <p>Deleting of VM succeeded.  This window should be closed.  If it is
                not closed, follow this <a href="Javascript:DeleteSuccess()">link</a>.
                </p>
                </body>
                </html>
	      ),
	   "Content-Type"  => "text/html; charset=iso-8859-1",
	   "Pragma"        => "no-cache",
	   "Cache-Control" => "no-cache" );

}


# Takes in a string corresponding to a request.  Returns a DOMAccess
# object which must be freed.
sub doServerdRequest($) {
    my $stub = $p->profile();
    my $request = shift;

    my $doc;
    my $ret;
    my $xml;

    my %credentials = hash_credentials( );
    my $server = &VMware::Control::Server::new( undef, 0,
						$credentials{ "l" },
						$credentials{ "m" } );
    if( !$server->connect() ) {
	my ($err, $errstr) = $server->get_last_error();
	return(0, "VMControl Error $err: $errstr");
    }

    my $doc = VMware::DOMAccess->new("server");
    $xml = $server->exec($request);

    if( !defined($xml) ) {
	my ($err, $errstr) = $server->get_last_error();
	return(0, "VMControl Error $err: $errstr");
    }

    $server->disconnect();

    $ret = VMware::DOMAccess->newXML($xml);
    return(1, $ret);
}

sub getDeleteInfo($) {
    my $stub = $p->profile();
    my($cfgName) = shift;
    return(doServerdRequest(makeDeleteInfoRequest($cfgName)));
}

sub makeDeleteInfoRequest($) {
    my $stub = $p->profile();
    my($cfgName) = shift;
    return("<exec><op>ESXVMDeleteInfo_DeleteVMInfo</op><in><config>$cfgName</config></in></exec>");
}

sub isSuccess($) {
    my($doc) = @_;
    if( $doc->getValue('.returncode') eq 1 ) {
	return(1);
    }
    return(0);
}

1;
