#!/usr/bin/perl -w

#############################################################
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
#############################################################
#
# ViewPart.pm
# 
# A script that presents the partitions available to have VMFS file systems
# created.
package VMware::Management::SS::ViewPart;

use strict;
use VMware::ExtHelpers qw(&getWarnings &Debug $gDEBUG &p_number &Warning &System 
                          $gPanicCallback &shell_string &clearWarnings);
use VMware::HConfig::MFSFileSystemRemote;
use VMware::HConfig::RawDisk;
use VMware::HConfig::RawDiskRemote;
use VMware::HConfig::SSHostRemote;
use VMware::HConfig::Constant qw($gVmkDumpPartSize $gVmkDumpPartType
				 @gPartitionCreateTypes $gVMFSPartType
				 $gVMFSMaxKBlocks $gVMFSBlockSizeLimit
				 $URL_VIEWPART $URL_NICCONFIG $URL_OVERVIEW
				 $URL_HSTATUS $URL_SECURITY $URL_SWAPFILE
				 );
use VMware::Management::Util qw( hash_credentials );

use VMware::Management::SS::ESXConfig;
my $htmlTitle = "Partition/VMFS Editor";

use VMware::Control::Profiler;
use CGI;

my $p = VMware::Control::Profiler::getInstance("mui");

# XXX Ugh.  A global variable.  It's used here to be convenient, but it's not
#     a good idea for modularity's sake.
my %mimeHeaders;       

$gDEBUG = 3;

my $gHttpMethod = 'GET';

my $gUseDeleteButton = 0;

my $URL_PREV = $URL_SECURITY;
my $URL_NAME_PREV = "Security Configuration";
my $URL_NEXT = $URL_SWAPFILE;
my $URL_NAME_NEXT = "Swap File Configuration";

my $RCom = "request";
my $RCom_ConvertMFS = "convertMFS";
my $RCom_SetMFS = "setMFS";
my $RCom_SetMFSName = "setMFSName";
my $RCom_SetMFSAccessibility = "setMFSAccessibility";
my $RCom_AttachMFS = "attachMFS";
my $RCom_Delete = "clearPart";
my $RCom_Create = "createPart";
my $RCom_Fdisk = "fdisk";
my $RCom_Debug = "debug";

my $Fdisk = "fdisk_action";
my $Fdisk_Delete = "delete";
my $Fdisk_Change = "change";
my $Fdisk_Add = "add";
my $Fdisk_Commit = "save";
my $Fdisk_Show = "show";

my $FdiskChange = "changed";

# Flags to specify to preDiskOperation and postDiskOperation indicating what
# operations to perform
my $DISK_SET_VMKDUMP = (1 << 0);

#####################################################################
# Common Functions used by both the web and command line version    #
#####################################################################

sub isInSet($$) {
    my ($input, $set) = @_;

    my @res = grep { $input eq $_ } @$set;
    return( $#res > -1 );
}

sub isNotInSet($$) {
    my ($input, $set) = @_;

    my @res = grep { $input eq $_ } @$set;

    return( $#res == -1 );
}

# Checks for vmkdump partition and generates an error message when this happens
sub checkForVmkDumpPart($$) {
    my $stub = $p->profile();
    my($host, $partitionTable) = @_;

    # Look in sshost
    my @vmkdumpParts = @{ $host->getVmkDumpPartitions($gVmkDumpPartSize) };
    if( $#vmkdumpParts > -1 ) {
	return(1);

    }
	
    # See if the user is in the process of creating one
    if( defined($partitionTable) ) {
	my $partnum;
	foreach $partnum ($partitionTable->getPartNums()) {
	    if( $partitionTable->getPartType($partnum) == $gVmkDumpPartType ) {
		return(1);
	    }
	}
    }

    Warning("VmkDump partition that is " . ($gVmkDumpPartSize / 1024) . "MB does not exist.  " .
            "It is recommended that you create a " . ($gVmkDumpPartSize / 1024) . "MB " .
	    "partition that will be used to contain core dump and log files.  You can create such " .
	    "a partition through this web page by creating a partition of type 'VMware Core Dump'.");
    return(0);
}

# Determines a good default block size for a VMFS -- one that allows
# a single VMFS file to encompass the entire VMFS.
sub getVMFSBlockSize($) {
    my($fsSizeMB) = @_;
    my $blockSize = 1;

    while ($blockSize * $gVMFSMaxKBlocks < $fsSizeMB) {
	$blockSize *= 2;

	if ($blockSize >= $gVMFSBlockSizeLimit) {
	    $blockSize = $gVMFSBlockSizeLimit;
	    last;
	}
    }
    return($blockSize);
}

# Gets a list of VMFS block sizes that are valid
sub getVMFSBlockSizeList() {
    my $blockSize = 1;
    my @blockSizeList;
    while ($blockSize <= $gVMFSBlockSizeLimit) {
	push (@blockSizeList, $blockSize);
	$blockSize *= 2;
    }
    return @blockSizeList;
}

# Determines if this partition is part of a VMFS, whether it is a
# head of an list of extents, and the VMFS itself.
#
# Returns ($isPartOfVmfs, $isHead, $VMFS)
sub getMFSInfo($$) {
   my $host = shift;
   my $partname = shift;
   my @allVmfs = @{$host->getAllMFS()};

   foreach my $vmfs (@allVmfs) {
      if ($partname eq $vmfs->getVmkName()) {
         return (1, 1, $vmfs);
      } elsif (grep($_ eq $partname, @{$vmfs->getExtents()})) {
         return (1, 0, $vmfs);
      }
   }
   return (0);
}


#####################################################################
# HTML producing functions                                          #
#####################################################################

sub PanicCallbackHTML(@) {
    my (@error) = @_;
    if ( $error[0] ) {
       print toHTMLPanic($htmlTitle, $error[0]);
    } else {
       print toHTMLPanic($htmlTitle, "");
    }
}

sub toErrorHTML($$) {
   my ($eSummary, $eDesc) = @_;
   return toHTMLPath(\@htmlPath, $htmlTitle) . toHTMLError($eSummary, $eDesc);
}

sub toViewPartitionHTML($$) {
    my $stub = $p->profile();
    my ($cgi, $host) = @_;
    my $url_self = $cgi->url(-relative=>1);
    my $wizard = $cgi->param("wizard");
    my $hasVmkDumpPart = 0;

    my @vmkDumpParts = @{ $host->getVmkDumpPartitions($gVmkDumpPartSize) };
    if( $#vmkDumpParts > -1 ) {
	$hasVmkDumpPart = 1;
    } else {
       Warning("VmkDump partition that is " . ($gVmkDumpPartSize / 1024) . "MB does not exist.  " .
               "It is recommended that you create a " . ($gVmkDumpPartSize / 1024) . "MB " .
               "partition that will be used to contain core dump and log files.  You can create such " .
               "a partition by using Fdisk.  If you have a disk without partitions, you can use the " .
               "'Create New Partition' button to have one created for you automatically.");
    }

    # Perform a check for duplicate VMFS names
    $host->checkDuplicateVmfsNames($host->getAllMFS());

    my @disks = @{ $host->getVMSCSIDisks() };
    my $disk;

    my $s =
        toHTMLPath(\@htmlPath, $htmlTitle) . 
        toHTMLSystemInfo($host) . 
        toHTMLWarnings();

    $s .= <<END_HTML;
<script type="text/javascript">
  <!--
  function doConfirm() {
     return confirm("This may cause some data to be lost on your disk.  Are you sure?");
  }
  function doConfirmDelete() {
     return confirm("You will lose all your data on the disk if you delete all\\nthe partitions on it.  Are you sure you want to do this?");
  }
  function doConfirmFormat() {
     return confirm("You will lose all your data in this partition if you format this partition\\nas a VMFS file system.  Are you sure you want to do this?");
  }
  function doConfirmFormatMFS() {
     return confirm("This partition contains a valid VMFS file system.  If you format this\\npartition, you will lose all your virtual disks in this VMFS.  Are you\\nsure you want to do this?");
  }
  function doConfirmAttach() {
     return confirm("You will lose all your data in this VMFS if you attach this partition\\nto a VMFS file system.  Are you sure you want to do this?");
  }
  function doConfirmAttachMFS() {
     return confirm("This partition contains a valid VMFS file system.  If you attach this\\npartition to another, you will lose all your virtual disks in this VMFS.  \\nAre you sure you want to do this?");
  }
  function doConfirmNext() {
     return confirm("Warning: You have not created any VMFS file systems.  You may be unable to\\nset up a virtual machine using the configuration wizard in the next step.\\nAre you sure you want to continue?");
  }
  function doConfirmSetName() {
     return confirm("Are you sure you want to change the\\nvolume name of your VMFS file system?");
  }
  function doConfirmAccessibility() {
     return confirm("Are you sure you want to change the accessibility of your VMFS file system?");
  }
  // -->
</script>
END_HTML

    my ($clustering) = $host->getLicenseInfo("clustering");
    $s .= 
        ("<h3>Editing Partitions and Creating VMFS File Systems</h3>\n" .
	 "\n" .
	 "<p class='txt'>\n" .
	 "Here you can set up your SCSI/RAID disks to use the VMFS file system where\n" .
	 "you can store disks for your virtual machines.\n" .
	 "</p>\n" .
	 "\n" .
	 "<p class='txt'>\n" .
	 "If you have any partitions defined, you can convert them to VMFS file systems by changing\n" .
	 "the type of the partition using fdisk.  To format a VMFS file system, click on the\n" .
	 "\"Format VMFS\" button next to the VMFS partition you wish to format.\n" .
	 "You can only format a partition if it has a VMFS type.\n" .
	 "</p>\n" .
	 "\n" .
	 "<p class='txt'>\n" .
	 "You can assign a logical name to each VMFS file system that can be used in the config files\n" .
	 "for your virtual machines.  Using logical names allows you to move SCSI/RAID\n" .
	 "disks without requiring a change to the configuration files on your machine.\n" .
	 "</p>\n");

    if ($clustering) {
       $s .=
           "<p class='txt'>\n" .
               "For VMFS file systems, you can also specify whether the file system is\n" .
               "<i>private</i> (accessible only by this server),\n" .
               "<i>public</i> (accessible by multiple physical servers), or\n" .
               "<i>shared</i> (accessible by multiple physical servers,\n" .
               "and may be used simultaneously by virtual machines on these servers).\n" .
               "</p>\n" .
               "<p class='txt'>\n" .
               "<b>Note</b>: You should set the accessibility to \"private\" unless\n" .
               "the VMFS is on shared storage that will definitely be accessed\n" .
               "by multiple ESX Server machines.\n" .
               "</p>";
    }

    $s .= "<p class='txt'>The following disks were detected on your system:</p>\n";
    
    # Print the disks that are on the system
    if( $#disks > -1 ) {
	$s .= "<ul>\n";
	
	foreach $disk (@disks) {
	    my $name = $disk->getVmkName();

	    if( !VMware::HConfig::RawDisk::isLogVmkDiskName($name) ) {
		next;
	    }

	    $s .= "   <li class='txt'>$name\n";
	}

	$s .= "</ul>\n";
    }
    else {
	$s .= "<p class='txt'><i>No SCSI disks found.</i></p>\n";
    }

    $s .= 
        "\n" .
        "<hr>\n" .
	"\n";

    if( $#disks > -1 ) {
	foreach $disk (@disks) {
	    my $name = $disk->getVmkName();
            my $geo_cyl = p_number($disk->getGeoCylinder());
            my $geo_head = p_number($disk->getGeoHead());
            my $geo_sect = p_number($disk->getGeoSector());
            my $mbSize = p_number($disk->getTotalSectors() / 2 / 1024);

	    if( !VMware::HConfig::RawDisk::isLogVmkDiskName($name) ) {
		next;
	    }
	    
	    # Print the disk name
	    $s .= qq(
                     <h4>Disk $name</h4>
                     <p class='txt'>
                     $mbSize MB total size ($geo_cyl cylinders, $geo_head heads, $geo_sect sectors)
                     </p>
                     );

	    my @partnums = $disk->getPartNums();
	    if( $#partnums > -1 ) {
                my @fields = ("Partition", "Type", "Start Cyl",
                              "End Cyl", "Size (MB)", "System",
                              "VMFS Name", "Actions");
                if ($clustering) {
                    # Insert accessibility field after VMFS Name
                    splice(@fields, @fields - 1, 0, "VMFS Accessibility");
                }
		$s .= toHTMLTableHeader(@fields);

		# Print out information about each partition.
		my $part;
		foreach $part (@partnums) {
		    my $part_name = VMware::HConfig::RawDisk::makeLogVmkPartName($name, $part);
		    my $part_type = sprintf("%02x", $disk->getPartType($part));
		    my $part_scyl = $disk->getPartStartCyl($part);
		    my $part_ecyl = $disk->getPartEndCyl($part);
		    my $part_sect = $disk->getPartSectors($part);
		    my $part_mb_size = sprintf("%d", $part_sect / 2 / 1024);
		    my $part_logMfsName = "N/A";
                    my $part_accessibility = "N/A";
                    my $part_isVMware = VMware::HConfig::RawDisk::isPartVmware($disk->getPartType($part));
		    my $part_isVmkDump = VMware::HConfig::RawDisk::isPartVmkDump($disk->getPartType($part));

		    # If there is only 1 partition and that partition is extended, just show it.
		    # Otherwise, we get a table that has a heading but no rows.
		    if (scalar(@partnums) > 1 &&
			&VMware::HConfig::RawDisk::isPartExtended($disk->getPartType($part))) {
			next;
		    }

                    my ($part_isMFS, $part_isHead, $part_vmfs) = getMFSInfo($host, $part_name);

                    if ($part_isMFS && !$part_isVMware) {
                        Warning("Partition $part_name is a VMFS partition, but partition type " .
                                "0x$part_type is not a VMware partition.\n");
                    }

                    # Set the System field in the name
		    my $part_type_name = $disk->getPartTypeName($part);
                    if ($part_isMFS) {
                        $part_type_name .= "-" . $part_vmfs->getVersion() . "." . $part_vmfs->getMinorVersion();

                        if (!$part_isHead) {
                           $part_type_name .= " (extent)";
                        }
                    } elsif (!$part_isMFS && $part_isVMware) {
			$part_type_name .= " (unformatted)";
                    }

		    if ($part_isMFS) {
			if (defined($part_vmfs)) {
			    $part_logMfsName = $part_vmfs->getLogName();
                            if ($clustering) {
                               $part_accessibility = $part_vmfs->getAccessibility();
                            }
			} else {
			    Warning("Partition $part_name is a VMFS partition, but could not find a " .
				    "corresponding VMFS object for it.");
			}
		    }

                    # Set the VMFS name field
                    my $part_vmfs_name = "&nbsp;";
                    if ($part_isMFS && $part_isHead) {
                       $part_vmfs_name =
			   (
			    "      <form name=\"set$name\" action=\"$url_self\" method=\"$gHttpMethod\"\" onSubmit=\"return doConfirmSetName()\">\n" .
			    "      <table border=0 cellpadding=0>\n".
			    "      <tr>\n" .
			    "        <td><input name=\"log_name\" size=12 value=\"$part_logMfsName\"></td>\n" .
                            "        <td><input type=\"submit\" value=\"Save\"></td>\n" . 
			    "      </tr>\n" .
			    "      </table>\n".
			    "      <input type=\"hidden\" name=\"disk\" value=\"$name\">\n" .
			    "      <input type=\"hidden\" name=\"part\" value=\"$part\">\n" .
			    "      <input type=\"hidden\" name=\"$RCom\" value=\"$RCom_SetMFSName\">\n" .
                            ($wizard ? "      <input type=\"hidden\" name=\"wizard\" value=\"$wizard\">\n" : "") .
			    "      </form>\n"
			    );
                    } elsif ($part_isMFS) {
                       $part_vmfs_name = $part_logMfsName . " (" .  $part_vmfs->getVmkName() . ")";
		    }

                    # Set the Accessibility field
                    my $part_vmfs_accessibility = "&nbsp;";
                    if ($clustering && $part_isMFS && $part_isHead) {
                        my ($isPrivate, $isShared, $isPublic) = ("", "", "");
                        if ($part_accessibility =~ /^shared/) {
                            $isShared = "selected";
                        } elsif ($part_accessibility =~ /^public/) {
                            $isPublic = "selected";
                        } else {
                            $isPrivate = "selected";
                        }
                        my $wizardInput = "";
                        if ($wizard) {
                            $wizardInput = qq(        <input type="hidden" name="wizard" value="$wizard">\n);
                        }
                        $part_vmfs_accessibility = <<END_HTML;
      <form name="acc$name" action="$url_self" method="$gHttpMethod"
            onSubmit="return doConfirmAccessibility()">
        <table border=0 cellpadding=0>
          <tr>
            <td><select name="accessibility">
              <option value="private" $isPrivate>Private</option>
              <option value="public" $isPublic>Public</option>
              <option value="shared" $isShared>Shared</option>
            </select></td>
            <td><input type="submit" value="Save"></td>
          </tr>
        </table>
        <input type="hidden" name="disk" value="$name">
        <input type="hidden" name="part" value="$part">
        <input type="hidden" name="$RCom" value="$RCom_SetMFSAccessibility">
        $wizardInput
      </form>
END_HTML
                    } elsif ($clustering && $part_isMFS) {
                       $part_vmfs_accessibility = ($part_accessibility =~ /^shared/ ? "Shared" :
                                                   $part_accessibility =~ /^public/ ? "Public" :
                                                   "Private");
                    }

                    # Set the action field.
                    my $part_actions = "N/A";

                    # Do not allow a partition to be formatted if
                    # - it is a vmkdump partition
                    # - it is an attached extent since it would destroy the VMFS.

                    if ($part_isVMware && !$part_isVmkDump &&
                        !($part_isMFS && !$part_isHead)) {

			my $part_confirm = $part_isMFS ? "doConfirmFormatMFS" : "doConfirmFormat";
			$part_actions = 
			    "      <form name=\"set$name\" action=\"$url_self\" method=\"$gHttpMethod\" onSubmit=\"return $part_confirm()\">\n" .
			    "      <table border=\"0\">\n" .
                            "        <tr>\n" .
                            "          <td colspan=\"2\" align=\"right\"><input type=\"submit\" value=\"Format VMFS\"></td>\n" .
                            "          <td class=\"txt\" align=\"right\" nowrap=\"1\"><nobr>with max file size:</nobr></td>\n" .
                            "          <td><select name=\"blocksize\">\n";

			my $fs = $host->getMFS($part_name);
                        my $part_blockSize = defined($fs) ? $fs->getBlockSize() : 1;
                        foreach my $blockSize (getVMFSBlockSizeList()) {
                            my $maxFileSize = p_number($blockSize * $gVMFSMaxKBlocks);
                            my $selected = 
                                ($part_blockSize == $blockSize) ? "selected" : "";
                            $part_actions .= 
                                qq(            <option value="${blockSize}m" $selected>) .
                                qq($maxFileSize GB</option>\n);
                        }

                        $part_actions .= 
                           ("          </select></td>\n" . 
                            "   </tr>\n" . 
                            "      </table>\n" .
			    "      <input type=\"hidden\" name=\"disk\" value=\"$name\">\n" .
			    "      <input type=\"hidden\" name=\"part\" value=\"$part\">\n" .
			    "      <input type=\"hidden\" name=\"$RCom\" value=\"$RCom_SetMFS\">\n" .
                            ($wizard ? "      <input type=\"hidden\" name=\"wizard\" value=\"$wizard\">\n" : "") .
			    "      </form>\n"
                            );

                        if (defined($fs) && $fs->getVersion() == 1) {
                           $part_actions .= 
                              ("      <form name=\"set$name\" action=\"$url_self\" method=\"$gHttpMethod\" onSubmit=\"return $part_confirm()\">\n" .
                               "      <table border=\"0\">\n" .
                               "        <tr>\n" .
                               "          <td colspan=\"2\" align=\"right\"><input type=\"submit\" value=\"Convert VMFS\"></td>" .
                               "          <td class=\"txt\" align=\"right\" nowrap=\"1\"><nobr>to VMFS-2.</nobr></td>\n" .
                               "          <input type=\"hidden\" name=\"disk\" value=\"$name\">\n" .
                               "          <input type=\"hidden\" name=\"part\" value=\"$part\">\n" .
                               "          <input type=\"hidden\" name=\"$RCom\" value=\"$RCom_ConvertMFS\">\n" .
                               ($wizard ? "      <input type=\"hidden\" name=\"wizard\" value=\"$wizard\">\n" : "") .
                               "        </td></tr>\n" . 
                               "      </table>\n" .
                               "      </form>\n"
                               );
                        }
                    }

                    # Only allow a partition to be extended if
                    # - it is not a vmkdump partition
                    # - the partition is not already an extent
                    # - a VMFS-2 exists that is not the same as the one that the partition defines.
                    # - it is not the head of a VMFS that has more than 1 extent.

                    my $part_vmfsExtents = defined($part_vmfs) ?
                       scalar(@{$part_vmfs->getExtents()}) : 0;

                    # Get a list of existing VMFS2 that are different
                    my @allVmfs2 = @{$host->getAllMFS()};
                    @allVmfs2= grep($_->getVersion() == 2 && $_->getVmkName ne $part_name, @allVmfs2);

                    if ($part_isVMware && !$part_isVmkDump &&
                        !($part_isMFS && !$part_isHead) &&
                        scalar(@allVmfs2) > 0 &&
                        !($part_isHead && $part_vmfsExtents > 1)) {

			my $part_confirm = $part_isMFS ? "doConfirmAttachMFS" : "doConfirmAttach";
                        $part_actions .=
                            ("      <form name=\"extend$name\" action=\"$url_self\" method=\"$gHttpMethod\" onSubmit=\"return $part_confirm()\">\n" .
                             "      <table border=\"0\">\n" .
                             "        <tr>\n" .
                             "          <td colspan=\"2\" align=\"right\"><input type=\"submit\" value=\"Attach Extent\"></td>\n" .
                             "          <td class=\"txt\" align=\"right\" nowrap=\"1\"><nobr>to VMFS </nobr>\n"
                             );

                        $part_actions .= "          <select name=\"vmfsname\">\n";

                        foreach my $vmfs (@allVmfs2) {
                            my $vmfsname = $vmfs->getVmkName();
                            my $vmfsLogName = $vmfs->getLogName();
                            my $text = $vmfsLogName ? "$vmfsLogName" : $vmfsname;

                            # Don't allow a partition to attach to itself
                            if ($vmfsname ne $part_name) {
                                $part_actions .= "          <option value=\"$vmfsname\">$text</option>\n";
                            }
                        }

                        $part_actions .= "          </select>\n";
                        $part_actions .=  "        </td>\n";
                           
                        $part_actions .= 
                            ("   </tr>\n" . 
                             "      </table>\n" .
                             "      <input type=\"hidden\" name=\"disk\" value=\"$name\">\n" .
                             "      <input type=\"hidden\" name=\"part\" value=\"$part\">\n" .
                             "      <input type=\"hidden\" name=\"$RCom\" value=\"$RCom_AttachMFS\">\n" .
                             ($wizard ? "      <input type=\"hidden\" name=\"wizard\" value=\"$wizard\">\n" : "") .
                             "      </form>\n"
                             );
                    }

                    my @data = ($part_name, 
                                [ "0x$part_type", "align='center'" ],
                                [ p_number($part_scyl), "align='right'" ],
                                [ p_number($part_ecyl), "align='right'" ],
                                [ p_number($part_mb_size), "align='right'" ],
                                $part_type_name,
                                $part_vmfs_name,
                                $part_actions);
                    if ($clustering) {
                        # Insert accessibility field after VMFS Name
                        splice(@data, @data - 1, 0, $part_vmfs_accessibility);
                    }
		    $s .= toHTMLTableRow(@data);

		} # foreach $part (@partnums)

                $s .= toHTMLTableFooter();

		$s .=
		    (
		     "\n" .
		     "    <p class='txt'>\n" .
		     "    To remove all the partitions, click on the \"Delete All Partitions\" button.\n" .
		     "    </p>\n" .
		     "    <table border=0>\n" .
		     "      <tr>\n" .
		     "          <td>\n" .
		     "          <form name=\"delete$name\" action=\"$url_self\" method=\"$gHttpMethod\" onSubmit=\"return doConfirmDelete()\">\n" .
		     "          <input type=\"hidden\" name=\"disk\" value=\"$name\">\n" .
		     "          <input type=\"hidden\" name=\"$RCom\" value=\"$RCom_Delete\">\n" .
                     ($wizard ? "        <input type=\"hidden\" name=\"wizard\" value=\"$wizard\">\n" : "") .
		     "          <input type=\"submit\" value=\"Delete All Partitions\">\n" . 
		     "          </form>\n" .
		     "          </td>\n" .
		     
		     "          <td>\n" .
		     "          <form name=\"delete$name\" action=\"$url_self\" method=\"$gHttpMethod\"\">\n" .
		     "          <input type=\"hidden\" name=\"disk\" value=\"$name\">\n" .
		     "          <input type=\"hidden\" name=\"$FdiskChange\" value=\"0\">\n" .
		     "          <input type=\"hidden\" name=\"$Fdisk\" value=\"$Fdisk_Show\">\n" .
		     "          <input type=\"hidden\" name=\"$RCom\" value=\"$RCom_Fdisk\">\n" .
                     ($wizard ? "        <input type=\"hidden\" name=\"wizard\" value=\"$wizard\">\n" : "") .
		     "          <input type=\"submit\" value=\"Use Expert Mode Fdisk\">\n" . 
		     "          </form>\n" .
		     "          </td>\n" .
		     "      <tr>\n" .
		     "    </table>\n"
		     );
	    } else {   
		$s .=
		    ("<table border=0 cellspacing=4>\n" .
		     "<tr>\n" .
		     "  <td>&nbsp&nbsp&nbsp</td>\n" .
		     "  <td class='txt'>\n" .
		     "    No partitions defined.\n" .
		     ($hasVmkDumpPart ? 
		      "    <p class='txt'>To create a single partition that will fill your whole disk, " .
		      "    click the \"Create New Partition\" button.</p>\n" :
		      "    <p class='txt'>To create two partitions, one to contain a core dump and the other to contain virtual disks, " .
		      "    click the \"Create New Partition\" button.</p>\n") .
		     "    <p class='txt'>\n" .
		     "    <table border=0>\n" .
		     "      <tr>\n" .
		     "          <td>\n" .
		     "          <form name=\"create$name\" action=\"$url_self\" method=\"$gHttpMethod\">\n" .
		     "          <input type=\"hidden\" name=\"disk\" value=\"$name\">\n" .
		     "          <input type=\"hidden\" name=\"$RCom\" value=\"$RCom_Create\">\n" .
                     ($wizard ? "        <input type=\"hidden\" name=\"wizard\" value=\"$wizard\">\n" : "") .
		     "          <input type=\"submit\" value=\"Create New Partition\">\n" . 
		     "          </form>\n" .
		     "          </td>\n" .
		     
		     "          <td>\n" .
		     "          <form name=\"delete$name\" action=\"$url_self\" method=\"$gHttpMethod\"\">\n" .
		     "          <input type=\"hidden\" name=\"disk\" value=\"$name\">\n" .
		     "          <input type=\"hidden\" name=\"$FdiskChange\" value=\"0\">\n" .
		     "          <input type=\"hidden\" name=\"$Fdisk\" value=\"$Fdisk_Show\">\n" .
		     "          <input type=\"hidden\" name=\"$RCom\" value=\"$RCom_Fdisk\">\n" .
                     ($wizard ? "        <input type=\"hidden\" name=\"wizard\" value=\"$wizard\">\n" : "") .
		     "          <input type=\"submit\" value=\"Use Expert Mode Fdisk\">\n" . 
		     "          </form>\n" .
		     "          </td>\n" .
		     "      <tr>\n" .
		     "    </table>\n" .
		     "    </p>\n" .
		     "  </td>\n" .
		     "</tr>\n" .
		     "</table>\n"
		     );
	    }

	    $s .= "<hr>\n\n";

	} # foreach $disk (@disks)
    } # if( $#disks > -1 )
    else {
	# No disks found
    }

    if ($wizard) {
       $s .= toHTMLNavWizard( [ $URL_PREV, $URL_NAME_PREV ],
                              [ $URL_NEXT, $URL_NAME_NEXT ] );
    }

    return($s);
}


sub toFdiskHTML($$$$) {
    my $stub = $p->profile();
    my($cgi, $host, $disk, $partitionTable) = @_;
    my $wizard = $cgi->param("wizard");

    # Add a check that will warn the user
    checkForVmkDumpPart($host, $partitionTable);

    my $url_self = $cgi->url(-relative=>1);

    my $diskName = $disk->getVmkName();
    my $geo_cyl = $disk->getGeoCylinder();
    my $geo_head = $disk->getGeoHead();
    my $geo_sect = $disk->getGeoSector();

    if( !defined($partitionTable) ) {
	return(toErrorHTML('Fdisk Failure', 
			   "The partition table could not be found.  Cannot proceed with fdisk."));
	return(0);
    }

    if( $geo_cyl != $partitionTable->getGeoCylinder() ||
	$geo_head != $partitionTable->getGeoHead() ||
	$geo_sect != $partitionTable->getGeoSector() ) {
	return(toErrorHTML('Fdisk Failure', 
			   "The partition table geometry (" . $partitionTable->getGeoCylinder() . ',' .
			   $partitionTable->getGeoHead() . ',' . $partitionTable->getGeoSector() .
			   ") does not match the disk geometry ($geo_cyl, $geo_head, $geo_sect).  " .
			   "Cannot proceed with fdisk."));
	return(0);
    }

    # Calculate the amount of unallocated space.  Must use this function in the case
    # that the cylinder number is the size of the disk.  The cylinder number is out
    # the range of the CHS.
    my( $free_phy_cyl, $free_log_cyl ) = $partitionTable->getUnpartSpace();
    my $free_log_sect = VMware::HConfig::RawDisk::fromChsToLba($free_log_cyl, 0, 1, 
							       $geo_cyl + 1, $geo_head, $geo_sect);
    my $free_phy_sect = VMware::HConfig::RawDisk::fromChsToLba($free_phy_cyl, 0, 1,
							       $geo_cyl + 1, $geo_head, $geo_sect);
    my $free_log_mb = ($free_log_sect - ($free_log_sect % 2000)) / 2000;
    my $free_phy_mb = ($free_phy_sect - ($free_phy_sect % 2000)) / 2000;

    # Only applicable if the disk is shared between the console and vms.  This 
    # information is ignored at the commit point in any case.


    my $title = "Expert Mode Fdisk";
    my @path = @htmlPath;
    push @path, [ $htmlTitle, $url_self ];

    my $s =
        toHTMLPath(\@path, $title) . 
        toHTMLSystemInfo($host) . 
        toHTMLWarnings();

    $s .= <<END_HTML;
<script type="text/javascript">
  <!--
  function doConfirm() {
     return confirm("This may cause some data to be lost on your disk.  Are you sure?");
  }
  function doConfirmExtended() {
     return confirm("Warning: Deleting an extended partition will also delete\\nall your logical partitions.  Are you you want to do this?");
  }
  function doConfirmQuitFdisk() {
     return confirm("You will lose any changes that you have made to your\\ndisk if you quit.  Are you sure you want to quit fdisk?");
  }
  // -->
</script>
END_HTML

    $s .=
	("\n" .
	 "<h3>Editing Partitions and Creating VMFS File Systems</h3>\n" .
	 "\n" .
	 "<p class='txt'>\n" .
	 "Edit the partition table to the desired configuration.  Changes to the\n" .
	 "partition table will not be saved until you select 'Save partition table'\n" .
         "and click the <i>Perform Action</i> button.\n" .
	 "</p>\n" .
         "<p class='txt'>\n" .
         "<b>Note</b>: There is a limit of four primary partitions per disk.\n" .
         "If you wish to place more than four partitions on this disk\n" .
         "(for example, if you want to put VMware Core Dump and VMFS partitions\n" .
         "on the same disk as your Console OS file systems), you must\n" .
         "first create an extended partition that spans the remainder of the\n" .
         "disk; you may then create additional logical partitions within the\n" .
         "extended partition.\n" .
         "</p>\n" .
	 "<h4>Disk $diskName</h4>\n\n"
	 );

    my @partnums = $partitionTable->getPartNums();
    if( $#partnums > -1 ) {
        my @headers = ("Partition", "Type", "Start Cylinder",
                       "End Cylinder", "Size (MB)", "System");
        if ($gUseDeleteButton) {
           push @headers, "&nbsp;";
        }

	$s .= toHTMLTableHeader(@headers);
    
	# Print out information about each partition.
	my $part;
	foreach $part (@partnums) {
	    my $part_name = VMware::HConfig::RawDisk::makeLogVmkPartName($partitionTable->getDiskName(), $part);
	    my $part_type = sprintf("%02x", $partitionTable->getPartType($part));
	    my $part_scyl = $partitionTable->getPartStartCyl($part);
	    my $part_ecyl = $partitionTable->getPartEndCyl($part);
	    my $part_sect = $partitionTable->getPartSectors($part);
	    my $part_mb_size = sprintf("%d", $part_sect / 2 / 1024);
	    my $part_type_name = $partitionTable->getPartTypeName($part);
	    
            my @fields = ($part_name,
                          [ "0x$part_type", "align='center'" ],
                          [ p_number($part_scyl), "align='right'" ],
                          [ p_number($part_ecyl), "align='right'" ],
                          [ p_number($part_mb_size) . " MB", "align='right'" ],
                          $part_type_name);
            if ($gUseDeleteButton) {
               my $delete =
		 ("<form name=\"delete_$part_name\" method=\"POST\" action=\"$url_self\">\n" .
		  toPartHiddensFdiskHTML($cgi, $host, $partitionTable) .
		  "        <input type=\"hidden\" name=\"disk\" value=\"$diskName\">\n" .
		  "        <input type=\"hidden\" name=\"$FdiskChange\" value=\"1\">\n" .
		  "        <input type=\"hidden\" name=\"$RCom\" value=\"$RCom_Fdisk\">\n" .
		  "        <input type=\"hidden\" name=\"$Fdisk\" value=\"$Fdisk_Delete\">\n" .
		  "        <input type=\"hidden\" name=\"delete_partname\" value=\"$part_name\">\n" .
                  ($wizard ? "        <input type=\"hidden\" name=\"wizard\" value=\"$wizard\">\n" : "") .
		  "        <input type=\"submit\" value=\"Delete\">\n" .
		  "        </form>");
               push @fields, $delete;
            }

            $s .= toHTMLTableRow(@fields);
	} # foreach $part ($partitionTable->getPartNums())

        $s .= toHTMLTableFooter();
    } else {
	$s .= 
	    ("<ul>\n" .
	     "<p class='txt'><i>No partitions defined.</i></p>\n" .
	     "</ul>\n"
	     );
    }

    $s .=
	("<p class='txt'>Disk geometry information: $geo_cyl cylinders, $geo_head heads, $geo_sect sectors.</p>\n\n" .
	 ($free_phy_cyl > 0 ?
	  "<p class='txt'>There is " . p_number($free_phy_mb) . " MB(s) (" . p_number($free_phy_cyl) . 
	  " cylinders) of space not used by the physical partitions.</p>\n\n" :
	  "") .
	 ($free_log_cyl > 0 ?
	  "<p class='txt'>There is " . p_number($free_log_mb) . " MB(s) (" . p_number($free_log_cyl) . 
	  " cylinders) of space available for logical partitions.</p>\n\n" :
	  "").
         "</p>\n\n" .
	 "    <h5>Actions:</h5>\n" .
	 "    <p class='txt'>Select one of the following actions to perform on the disk and fill in the appropriate fields for that action:</p>\n" .
	 "    <form name=\"fdisk_action\" method=\"GET\" action=\"$url_self\">\n" .
	 "    <table border='0' cellpadding='2'>\n" .
	 toDeleteActionFdiskHTML($cgi, $host, $partitionTable) .
	 "      <tr><td colspan=\"5\"><small><small>&nbsp;</small></small></td></tr>" .
	 toChangeActionFdiskHTML($cgi, $host, $partitionTable) .	 
	 "      <tr><td colspan=\"5\"><small><small>&nbsp;</small></small></td></tr>" .
	 toAddActionFdiskHTML($cgi, $host, $partitionTable) .	 
	 "      <tr><td colspan=\"5\"><small><small>&nbsp;</small></small></td></tr>" .
	 "      <tr>\n" .
	 "        <td><input type=\"radio\" name=\"$Fdisk\" value=\"$Fdisk_Commit\"></td>\n" .
	 "        <td class='txt'><b>Save partition table</b></td>\n" .
         "        <td colspan='3'></td>\n" .
	 "      </tr>\n" .
	 "    </table>\n" .
	 "\n" .
	 "    <p class='txt'>\n" .
	 "\n" .
	 "    </p>\n" .
	 "\n" .
	 "<p class='txt'><b>\n" .
	 "Warning: Using fdisk can cause adverse side effects on your system especially if you are changing a\n" .
	 "disk that is currently mounted.  We recommend that you take adequate precautions such as backing\n" .
	 "up your disk before modifying it.\n" .
	 "</b></p>" .
	 "\n" .
	 "    <table border=0 cellpadding=0>\n" .
	 "      <tr>\n" .
	 "      <td>\n" .
	 toPartHiddensFdiskHTML($cgi, $host, $partitionTable) .
	 "          <input type=\"hidden\" name=\"disk\" value=\"$diskName\">\n" .
	 "          <input type=\"hidden\" name=\"$FdiskChange\" value=\"1\">\n" .
	 "          <input type=\"hidden\" name=\"$RCom\" value=\"$RCom_Fdisk\">\n" .
         ($wizard ? "        <input type=\"hidden\" name=\"wizard\" value=\"$wizard\">\n" : "") .
	 "          <input type=\"submit\" value=\"Perform Action\">\n" .
	 "          </form>\n" .
	 "      </td>\n" .
	 "	<td>\n" .
	 "          <form name=\"fdisk_revert\" method=\"POST\" action=\"$url_self\">\n" .
	 "          <input type=\"hidden\" name=\"disk\" value=\"$diskName\">\n" .
	 "          <input type=\"hidden\" name=\"$FdiskChange\" value=\"0\">\n" .
	 "          <input type=\"hidden\" name=\"$Fdisk\" value=\"$Fdisk_Show\">\n" .
	 "          <input type=\"hidden\" name=\"$RCom\" value=\"$RCom_Fdisk\">\n" .
         ($wizard ? "        <input type=\"hidden\" name=\"wizard\" value=\"$wizard\">\n" : "") .
	 "          <input type=\submit\ value=\"Revert to Current Settings\">\n" .
	 "          </form>\n" .
	 "      </td>\n" .
	 "	<td></td>\n" .
	 "      </tr>\n" .
	 "    </table>\n" .
	 "\n" .
	 "<hr>\n" .
	 "<p class='txt'>\n" .
	 "<form name=\"fdisk_quit\" method=\"GET\" action=\"$url_self\" onSubmit=\"return doConfirmQuitFdisk()\">\n".
         ($wizard ? "    <input type=\"hidden\" name=\"wizard\" value=\"$wizard\">\n" : "") .
	 "    <input type=\"submit\" value=\"Quit Expert Mode Fdisk\">\n" .
	 "</form>\n" .
	 "</p>\n" .
	 "</td>\n" .
	 "</tr>\n" .
	 "</table>\n"
	 );

    return($s);
}


sub toDeleteActionFdiskHTML($$$) {
    my $stub = $p->profile();
    my($cgi, $host, $partitionTable) = @_;
    my $s = "";

    # Check the partition table for partitions
    my @partnums = $partitionTable->getPartNums();
    if( $#partnums == -1 ) {
	return($s);
    }

    $s .=
	("      <tr>\n" .
	 "        <td><input type=\"radio\" name=\"$Fdisk\" value=\"$Fdisk_Delete\"></td>\n" .
	 "	  <td class='txt'><b>Delete</b></td>\n" .
         "        <td class='txt' align='right'>Partition:</td>\n" .
	 "        <td>\n" .
	 "          <select name=\"delete_partname\">\n" .
	 toExistPartOptionsFdiskHTML($cgi, $host, $partitionTable) .
	 "          </select>\n" .
	 "        </td>\n" .
	 "      </tr>\n"
	 );
    return($s);
}


sub toChangeActionFdiskHTML($$$) {
    my $stub = $p->profile();
    my($cgi, $host, $partitionTable) = @_;
    my $s = "";

    # Check the partition table for partitions
    my @partnums = $partitionTable->getPartNums();
    if( $#partnums == -1 ) {
	return($s);
    }

    $s .=
	("      <tr>\n" .
	 "	<td><input type=\"radio\" name=\"$Fdisk\" value=\"$Fdisk_Change\"></td>\n" .
	 "        <td class='txt'><b>Change</b></td>\n" .
         "        <td class='txt' align='right'>Partition:</td>\n" .
	 "        <td>\n" .
	 "          <select name=\"change_partname\">\n" .
	 toExistPartOptionsFdiskHTML($cgi, $host, $partitionTable) .
	 "	    </select>\n" .
	 "        </td>\n" .
         "      </tr>\n" .
         "      <tr>\n" .
	 "        <td class='txt' colspan='3' align='right'>New file system type:</td>\n" .
	 "        <td>\n" .
	 "	    <select name=\"change_type\">\n" .
	 toPartTypeOptionsFdiskHTML($cgi, $host, $partitionTable) .
	 "	    </select>\n" .
	 "        </td>\n" .
	 "      </tr>\n"
	 );
    return($s);
}


sub toAddActionFdiskHTML($$$) {
    my $stub = $p->profile();
    my($cgi, $host, $partitionTable) = @_;
    my $s = "";

    # Check to see if we can still make partitions
    my @physicalPartNums = $partitionTable->getCanAddPhyPartNum();
    my $canAddLogPart = $partitionTable->canAddLogPartNum();

    if( $#physicalPartNums == -1 && !$canAddLogPart ) {
	return($s);
    }

    # Calculate a default range.  First look for physical ranges, then
    # use logical ones.
    my @freeRanges = $partitionTable->getFreeCylinderRange();
    my @logFreeRange = grep($_->{LOGICAL} == 1, @freeRanges);
    my @phyFreeRange = grep($_->{LOGICAL} == 0, @freeRanges);

    my($default_start_cyl, $default_end_cyl) = (-1, -1);

    if( $#physicalPartNums > -1 && $#phyFreeRange > -1 ) {
	$default_start_cyl = $phyFreeRange[0]->{START_CYL};
	$default_end_cyl = $phyFreeRange[0]->{END_CYL};
    } elsif( $canAddLogPart && $#logFreeRange > -1 ) {
	$default_start_cyl = $logFreeRange[0]->{START_CYL};
	$default_end_cyl = $logFreeRange[0]->{END_CYL};
    } else {
	Warning("Internal Error: Could not find a free range to use as defaults");
	$default_start_cyl = 0;
	$default_end_cyl = 0;
    }

    # Set appropriate start/end cylinders depending on partition type:
    # -- For core dump partition, use gVmkDumpPartSize
    # -- For all other partition types, use defaults chosen above
    my $dumpType = sprintf("%02x", $gVmkDumpPartType);
    my $dumpSize = int($gVmkDumpPartSize / 1024);
    $s .= <<END_HTML;
<script type="text/javascript">
  <!--
  function addSetCylDefaults() {
     document.fdisk_action.add_start_cyl.value = $default_start_cyl;
     if (document.fdisk_action.add_type.value == "$dumpType") {
        document.fdisk_action.add_end_cyl.value = "+${dumpSize}M";
     } else {
        document.fdisk_action.add_end_cyl.value = "$default_end_cyl";
     }
  }
  // -->
</script>
END_HTML

    $s .=
	("      <tr>\n" .
	 "        <td><input type=\"radio\" name=\"$Fdisk\" value=\"$Fdisk_Add\"></td>\n" .
	 "        <td class='txt'><b>Add</b></td>\n" .
         "        <td class='txt' align='right'>Partition type:</td>\n" .
	 "        <td>\n" .
	 "          <select name=\"add_partname\">\n" .
	 toPossNewPartOptionsFdiskHTML($cgi, $host, $partitionTable) .
	 "          </select>\n" .
	 "        </td>\n" .
         "      </tr>\n" .
         "      <tr>\n" .
	 "        <td class='txt' colspan='3' align='right'>File system type:</td>\n" .
	 "        <td>\n" .
	 "	    <select name=\"add_type\" onchange=\"addSetCylDefaults()\">\n" .
	 toPartTypeOptionsFdiskHTML($cgi, $host, $partitionTable) .
	 "	    </select>\n" .
	 "        </td>\n" .
	 "      </tr>\n" .
         "      <tr>\n" .
         "        <td class='txt' colspan=\"3\" align=\"right\">First cylinder:</td>\n" .
         "        <td><input size=6 name=\"add_start_cyl\" value=\"$default_start_cyl\"></td>\n" .
         "      </tr>\n" .
         "      <tr>\n" .
         "        <td class='txt' colspan=\"3\" align=\"right\">Last cylinder (or +&lt;size&gt;M):</td>\n" .
         "        <td><input size=6 name=\"add_end_cyl\" value=\"$default_end_cyl\"></td>\n" .

	 "      </tr>\n");
    return($s);
}


sub toPartHiddensFdiskHTML($$$) {
    my $stub = $p->profile();
    my($cgi, $host, $partitionTable) = @_;
    my $s = "";
    
    my $partnum;
    foreach $partnum ($partitionTable->getPartNums()) {
	my $part_name = VMware::HConfig::RawDisk::makeLogVmkPartName($partitionTable->getDiskName(), $partnum);
	my $part_scyl = $partitionTable->getPartStartCyl($partnum);
	my $part_ecyl = $partitionTable->getPartEndCyl($partnum);
	my $part_type = $partitionTable->getPartType($partnum);

	$part_type = sprintf("%02x", $part_type);
	my $part_key = 'part_' . $part_name;
	$s .= (
	       "          <input type=\"hidden\" name=\"$part_key" . "_scyl\" value=\"$part_scyl\">\n" .
	       "          <input type=\"hidden\" name=\"$part_key" . "_ecyl\" value=\"$part_ecyl\">\n" .
	       "          <input type=\"hidden\" name=\"$part_key" . "_type\" value=\"$part_type\">\n" .
	       "\n");
    }
    return($s);
}

sub toPossNewPartOptionsFdiskHTML($$$) {
    my $stub = $p->profile();
    my($cgi, $host, $partitionTable) = @_;
    my $s = "";

    my @physicalPartNums = $partitionTable->getCanAddPhyPartNum();

    my $partnum;
    foreach $partnum (@physicalPartNums) {
	my $part_name = VMware::HConfig::RawDisk::makeLogVmkPartName($partitionTable->getDiskName(), $partnum);
	$s .= "	    <option value=\"$part_name\">$part_name</option>\n";
    }

    if( $partitionTable->canAddLogPartNum() ) {
	$s .= "	    <option value=\"logical\">logical</option>\n";
    }
    return($s);
}

sub toExistPartOptionsFdiskHTML($$$) {
    my $stub = $p->profile();
    my($cgi, $host, $partitionTable) = @_;
    my $s = "	    <option value=\"none\">none</option>\n";
    
    my $partnum;
    foreach $partnum ($partitionTable->getPartNums()) {
	my $part_name = VMware::HConfig::RawDisk::makeLogVmkPartName($partitionTable->getDiskName(), 
								  $partnum);
	$s .= "	    <option value=\"$part_name\">$part_name</option>\n";
    }
    return($s);
}

sub toPartTypeOptionsFdiskHTML($$$) {
    my $stub = $p->profile();
    my($cgi, $host, $partitionTable) = @_;
    my $s = "";

    my $parttype;
    foreach $parttype (@gPartitionCreateTypes) {
	if( VMware::HConfig::RawDisk::isPartExtended($parttype) && 
	    $partitionTable->getExtendedPartNum() != 0 ) {
	    # Don't give the user the option to make an extended partition if there 
	    # already is one in the partition table
	    next;
	}
	my $typeString = sprintf("%02x", $parttype);
	$s .= "	    <option value=\"$typeString\">" . VMware::HConfig::RawDisk::lookupPartType($parttype) . "</option>\n";
    }

    return($s);
}

#####################################################################
# Function unique to this version                                   #
#####################################################################

# Format a partition as a VMFS
sub setMFS {
    my $stub = $p->profile();
    my ($cgi, $host) = @_;
    my $err;

    # Perform some checks first
    # 1. Make sure that the disk exists
    # 2. Make sure that the partition exists
    my $name = $cgi->param('disk');
    my $part = $cgi->param('part');
    my $blocksize = $cgi->param('blocksize');

    my $disk = $host->getVMSCSIDiskByName($name);
    if (!defined($disk)) {
	return("Setting VMFS: " .
	       "Could not set to VMFS file system because invalid disk name '$name' received.\n");
    }
	
    my @part_nums = $disk->getPartNums();

    if( !isInSet($part, \@part_nums) ) {
	return("Setting VMFS: " .
	       "Could not set to VMFS file system because invalid partition number '$part' received.\n");
    }

    # Untaint the partition number and perform a number check on it
    if( $part =~ /(\d+)/ ) {
	$part = $1;
    }
    else {
	return("Setting VMFS: " . 
	       "Partition number '$part' is not a number.\n");
    }

    # Check if there is an active swap file on this partition
    my $fullpname = VMware::HConfig::RawDisk::makeLogVmkPartName($disk->getVmkName(), $part);
    if ($host->getMFS($fullpname)) {
        my (undef, undef, undef, undef, $swapvol, undef) = $host->getSwapInfo();
        if (defined($swapvol) && ($swapvol ne "")) {
            my $swapMFS = $host->getMFSLookup($swapvol);
            if (defined($swapMFS) && 
                ($name eq $swapMFS->getDisk()) && 
                ($part eq $swapMFS->getPartition())) {
                return(<<END_HTML);
Setting VMFS: Could not reformat partition $part because it contains
an active swap file.  If you wish to reformat this partition, use the 
<a href="$URL_SWAPFILE">swap file configuration page</a> to disable the
swap file or move it to another partition, and then reboot the server.
END_HTML
            }
        }
    }

    # rbosch: Hard-code to VMFS2 for now; should be configurable
    my $version = 2;

    if( !$disk->createNewMFS($part, $blocksize, $version) ) {
       $err = "Setting VMFS: " .
	      "Could not create VMFS file system in partition $part.  Reason: $!.\n";
    }

    return $err;
}

# Convert a VMFS-1 to VMFS-2
sub convertMFS {
   my $stub = $p->profile();
   my ($cgi, $host) = @_;
   my $err;

   # Perform some checks first
   # 1. Make sure that the disk exists
   # 2. Make sure that the partition exists
   my $name = $cgi->param('disk') . ':' . $cgi->param('part');

   my $vmfs = $host->getMFSLookup($name);
   if (!defined($vmfs)) {
      return("Converting VMFS: " .
             "Could not set to VMFS file system because invalid disk name '$name' received.\n");
   }
	
   if (!$vmfs->convertToVmfs2()) {
      $err = "Converting VMFS: " .
             "Could not convert VMFS file system $name.  Reason: $!.\n";
   }

   return $err;
}

sub setMFSName {
    my $stub = $p->profile();
    my ($cgi, $host) = @_;
    my $err;

    # Perform some checks first
    # 1. Make sure that the disk exists
    # 2. Make sure that the partition exists
    my $diskName = $cgi->param('disk');
    my $partnum = $cgi->param('part');

    my $logname = $cgi->param('log_name');
    # Remove any leading or trailing whitespace
    $logname =~ s/^\s*(.*?)\s*$/$1/;

    my $partname = VMware::HConfig::RawDisk::makeLogVmkPartName($diskName, $partnum);
    if( !defined($partname) ) {
	return("Setting VMFS Name: " .
	       "Could not set VMFS file system name because invalid diskname ($diskName) " .
	       "and partition number ($partnum)\n");
    }

    my $fs = $host->getMFS($partname);
    if( !defined($fs) ) {
	return("Setting VMFS Name: " .
	       "Could not get VMFS file system object for partition name $partname.\n");
    }

    if( $logname =~ /:/ ) {
	return("Setting VMFS Name: " .
	       "Colon ':' characters cannot be used in the VMFS name.");
    }

    if( $logname ne "" ) {
        my $newvmfs = $host->getMFSByLogName($logname);
        if( defined($newvmfs) && $newvmfs != $fs ) {
            my $newname = $newvmfs->getVmkName();
            return("Setting VMFS Name: " .
                   "Could not set the VMFS name to '$logname' because\n" .
                   "the name is being used by VMFS '$newname'.\n");
        }
    }

    # Nothing to do if the name is unchanged
    my $origName = $fs->getLogName();
    if( $logname ne $origName ) {
        if( !$fs->setVMFSLogName($logname) ) {
            $err = "Setting VMFS: Could not change VMFS name to '$logname'.";
        }
    }
	
    return $err;
}

sub setMFSAccessibility {
    my $stub = $p->profile();
    my ($cgi, $host) = @_;
    my $err;

    # Perform some checks first
    # 1. Make sure that the disk exists
    # 2. Make sure that the partition exists
    my $diskName = $cgi->param('disk');
    my $partnum = $cgi->param('part');
    my $accessibility = $cgi->param('accessibility');

    my $partname = VMware::HConfig::RawDisk::makeLogVmkPartName($diskName, $partnum);
    if( !defined($partname) ) {
	return("Setting VMFS Accessibility: " .
	       "Could not set VMFS accessibility because invalid diskname ($diskName) " .
	       "and partition number ($partnum)\n");
    }

    my $fs = $host->getMFS($partname);
    if( !defined($fs) ) {
	return("Setting VMFS Accessibility: " .
	       "Could not get VMFS file system object for partition name $partname.\n");
    }

    if( !$fs->setAccessibility($accessibility) ) {
        $err = "Setting VMFS: Could not change VMFS accessibility to $accessibility.\n";
        $accessibility = $fs->getAccessibility();
    }

    return $err;
}

sub attachMFS {
    my $stub = $p->profile();
    my ($cgi, $host) = @_;

    # Perform some checks first
    # 1. Make sure that the VMFS to which this partition is to be attached exists.
    my $diskName = $cgi->param('disk');
    my $partnum = $cgi->param('part');
    my $vmfsname = $cgi->param('vmfsname');

    my $partname = VMware::HConfig::RawDisk::makeLogVmkPartName($diskName, $partnum);
    if (!defined($partname)) {
	return("Attaching VMFS: " .
	       "Could not attach to VMFS because invalid diskname ($diskName) " .
	       "and partition number ($partnum)\n");
    }

    my $vmfs = $host->getMFS($vmfsname);
    if (!defined($vmfs)) {
	return("Attaching VMFS: " .
	       "Could not get VMFS file system object for partition name $vmfsname.\n");
    }

    if (!$vmfs->attachExtent($partname)) {
       return("Attaching VMFS: " .
              "Could not attach VMFS '$partname' to '$vmfsname'.\n");
    }

    return undef;
}

# Creates a vmkdump partition if it does not already exist
sub createSimplePartition {
    my $stub = $p->profile();
    my ($cgi, $host) = @_;

    # Perform some checks first
    # 1. Make sure that the disk exists
    # 2. Make sure there are no partitions on the disk
    my $name = $cgi->param('disk');

    my $disk = $host->getVMSCSIDiskByName($name);
    if (!defined($disk)) {
	return("Creating a Partition: " .
	       "Could not create a partition because invalid disk name '$name' received.\n");
    }

    my $num_cyl = $disk->getGeoCylinder();

    my @part_nums = $disk->getPartNums();

    if( $#part_nums > -1 ) {
	return("Creating a Partition: " .
	       "Could not create a partition because one already exists.  \n".
	       "This program only allows a partition to be created if none exist.");
    }

    # See if we need to create a dump partition
    my $needVmkDumpPart = 1;
    my @vmkDumpParts = @{ $host->getVmkDumpPartitions($gVmkDumpPartSize) };
    if( $#vmkDumpParts > -1 ) {
	$needVmkDumpPart = 0;
    }

    my $partTable = $disk->createPartitionTable();
    my $partnum = 1;
    my $cylnum = 0;

    if( $needVmkDumpPart ) {
	my $sectPerCyl = $disk->getChsToLba(1, 0, 1) + 1;
	my $kbytesPerCyl = ($sectPerCyl - ($sectPerCyl % 2)) / 2;
	my $numCyls = ($gVmkDumpPartSize - ($gVmkDumpPartSize % $kbytesPerCyl)) / $kbytesPerCyl;
	if( $gVmkDumpPartSize % $kbytesPerCyl > 0 ) {
	    # Round up
	    $numCyls++;
	}
	# Round this number higher

	$partTable->addPart($partnum ,$cylnum, $numCyls - $cylnum - 1, $gVmkDumpPartType, 0, 0);
	$partnum++;
	$cylnum += $numCyls;
    }

    # Create a partition taking up the remaining space
    $partTable->addPart($partnum ,$cylnum, $num_cyl - 1, $gVMFSPartType, 0, 0);

    Debug(2, "Creating partitions " . $partTable->toSfdiskString());

    # Create a partition and make it of type $gVMFSPartType
    if( !$disk->makePartitions($partTable) ) {
	return( "Creating Partitions: " .
		"Could not create partition.  Reason: $!.\n");
    }

    # Pick a blocksize large enough to create a file to encompass the whole VMFS
    my $fsSizeGB = $disk->getCylToKBytes($num_cyl) / 1024 / 1024;
    my $blockSize = getVMFSBlockSize($fsSizeGB) . "m";

    # Now make it a VMFS2 partition
    if (!$disk->createNewMFS($partnum, $blockSize, 2)) {
	return("Creating Partitions: " . 
	       "Could not create VMFS file system on the file system.  Reason: $!.\n");
    }

    # Mount the new VMFS partition.
    my $fullpname = VMware::HConfig::RawDisk::makeLogVmkPartName($disk->getVmkName(), $partnum);

    return(undef);
}


sub deleteAllPartitions {
    my $stub = $p->profile();
    my ($cgi, $host) = @_;

    # Perform some checks first
    # 1. Make sure that the disk exists
    my $name = $cgi->param('disk');

    my $disk = $host->getVMSCSIDiskByName($name);
    if (!defined($disk)) {
	return("Deleting All Partitions: " .
	       "Could not delete all partitions because invalid disk name '$name' received.\n");
    }

    if( !$disk->deleteAllPartitions() ) {
        return("Deleting All Partitions: " .
	       "Could not delete all partitions.  Reason: $!\n");
    }

    return(undef);
}


# Returns a disk and a partition table for that disk
sub buildPartitionTable($$) {
    my $stub = $p->profile();
    my($cgi, $host) = @_;
    my @partnames;

    my $diskName = $cgi->param("disk");
    if( !defined($diskName) ) {
	Warning("Internal Error: Could not find disk name to perform operations.");
	return(undef);
    }

    my $disk = $host->getVMSCSIDiskByName($diskName);
    if( !defined($disk) ) {
	Warning("Internal Error: Could not perform operation because disk $diskName does " .
		"not exist.");
	return(undef);
    }

    my $partitionTable = $disk->createPartitionTable();

    # Recreate the partition table information
    my @params = $cgi->param();
    @partnames = grep( /^part_\S+_type$/, @params );

    # Extract only the valid partition names
    foreach (@partnames) {
	s/^part_(\S+)_type$/$1/;
    }
    @partnames = grep( VMware::HConfig::RawDisk::isLogVmkPartName($_), @partnames );

    my $partname;
    foreach $partname (@partnames) {
	my($dname, $partnum) = VMware::HConfig::RawDisk::unmakeLogVmkPartName($partname);
	if( $dname ne $diskName ) {
	    Warning("Internal Error: Got a partition $dname:$partnum that does not match the disk name $diskName.");
	    next;
	}

	my $parttype = hex($cgi->param('part_' . $partname . '_type'));
	my $partscyl = $cgi->param('part_' . $partname . '_scyl');
	my $partecyl = $cgi->param('part_' . $partname . '_ecyl');
	
	# Calculate the amount of unallocated space.  Must use this function in the case
	# that the cylinder number is the size of the disk.  The cylinder number is out
	# the range of the CHS.
	my $sectors = VMware::HConfig::RawDisk::fromChsToLba($partecyl - $partscyl + 1, 0, 1,
							     $disk->getGeoCylinder() + 1,
							     $disk->getGeoHead(),
							     $disk->getGeoSector());

	$partitionTable->addPart($partnum, $partscyl, $partecyl,
				 $parttype, $sectors, 0);
    }
    return($disk, $partitionTable);
}

# Gets the disk and partition to be used for fdisk
sub getDiskPartInfoFdisk($$) {
    my $stub = $p->profile();
    my($cgi, $host) = @_;

    # Get flag to indicate if we are in a state where the partitions are being changed
    my $changed = $cgi->param($FdiskChange);
    if( !defined($changed) ) {
	$changed = 0;
    }

    my($disk, $partitionTable);
    if( $changed ) {
	# Build the disk and partition table object from the state information passed
	# through CGI
	($disk, $partitionTable) = buildPartitionTable($cgi, $host);
	if( defined($disk) && defined($partitionTable) ) {
	    return($disk, $partitionTable);
	}

	Warning("Could not get partition information.");
    }

    # Get the current disk info
    my $diskName = $cgi->param("disk");
    $disk = $host->getVMSCSIDiskByName($diskName);
    if( !defined($disk) ) {
	return(undef, undef);
    }

    return($disk, $disk->getPartitionTable());
}

# Functions for handling fdisk delete operations
sub deletePartitionFdisk($$$$) {
    my $stub = $p->profile();
    my($cgi, $host, $disk, $partitionTable) = @_;

    Debug(1, "Entered");

    my $partname = $cgi->param('delete_partname');
    if( !defined($partname) ) {
	Warning("Internal Error: Could not delete partition because did not get partition name.");
	return(undef);
    }

    if( $partname eq "none" ) {
	# No action performed.  Just refresh.
	Warning("Did not delete partition because no partition was selected.");
	return(undef);
    }

    my($diskName, $partnum) = VMware::HConfig::RawDisk::unmakeLogVmkPartName($partname);
    if( !defined($diskName) || !defined($partnum) ) {
	Warning("Internal Error: Could not delete partition because could not get " .
		"the disk name or partition number from partition name '$partname'.");
	return(undef);
    }

    # Sanity check
    if( $disk->getVmkName() ne $diskName ) {
	Warning("Trying to change partition on disk " . $diskName . " but got information " .
		"for disk " . $disk->getVmkName() . ".");
	return(undef);
    }

    my $parttype = $partitionTable->getPartType($partnum);
    if( VMware::HConfig::RawDisk::isPartExtended( $parttype ) && $partitionTable->hasLogicalParts() ) {
	Warning("Cannot remove extended partition if there are logical partitions.");
	return(undef);
    }

    $partitionTable->removePart($partnum);

    # In the case of a logical partition, we may have to shift them all down
    if( VMware::HConfig::RawDisk::isLogicalPart($partnum) ) {
	$partitionTable->reorderLogicalParts();
    }

    return(undef);
}

# Functions for handling fdisk change partition operation
sub changePartitionFdisk($$$$) {
    my $stub = $p->profile();
    my($cgi, $host, $disk, $partitionTable) = @_;
    Debug(1, "Entered");

    # Get the partition information
    my $partname = $cgi->param('change_partname');
    if( !defined($partname) ) {
	Warning("Internal Error: Could not change partition because did not get partition name.");
	return(undef);
    }

    if( $partname eq "none" ) {
	# No action performed.  Just refresh.
	Warning("Did not change partition type because no partition was selected.");
	return(undef);
    }

    my($diskName, $partnum) = VMware::HConfig::RawDisk::unmakeLogVmkPartName($partname);
    if( !defined($diskName) || !defined($partnum) ) {
	Warning("Internal Error: Could not change partition type because could not get " .
		"the disk name or partition number from partition name '$partname'.");
	return(undef);
    }

    # Get the type information
    my $parttype = hex($cgi->param('change_type'));
    if( !defined($parttype) ) {
	Warning("Internal Error: Could not get partition type.");
	return(undef);
    }

    # Sanity check
    if( $disk->getVmkName() ne $diskName ) {
	Warning("Trying to change partition on disk " . $diskName . " but got information " .
		"for disk " . $disk->getVmkName() . ".");
	return(undef);
    }

    my $ptype = $partitionTable->getPartType($partnum);
    if( VMware::HConfig::RawDisk::isPartExtended( $ptype ) && $partitionTable->hasLogicalParts() ) {
	Warning("Cannot change the type of an extended partition if there " .
		"are logical partitions.");
	return(undef);
    }

    if( defined($partitionTable->getPart($partnum)) ) {
	# Change the partition information
	my $part_scyl = $partitionTable->getPartStartCyl($partnum);
	my $part_ecyl = $partitionTable->getPartEndCyl($partnum);
	my $part_sect = $partitionTable->getPartSectors($partnum);
	$partitionTable->removePart($partnum);
	$partitionTable->addPart($partnum, $part_scyl, $part_ecyl,
				 $parttype, $part_sect, 0);
    } else {
	Warning("Partition $partnum is not defined for disk $diskName");
    }
	
    return(undef);
}

sub addPartitionFdisk($$$$) {
    my $stub = $p->profile();
    my($cgi, $host, $disk, $partitionTable) = @_;

    # Get the partition information
    my $partname = $cgi->param('add_partname');
    if( !defined($partname) ) {
	Warning("Internal Error: Could not add partition because did not get partition name.");
	return(undef);
    }

    if( $partname eq "none" ) {
	# No action performed.  Just refresh.
	Warning("Did not add partition because no partition was selected.");
	return(undef);
    }

    my($diskName, $partnum);
    if( $partname eq "logical" ) {
	# Find the first unused logical partition
	$diskName = $disk->getVmkName();
        $partnum = 5;
	while( defined($partitionTable->getPart($partnum)) ) {
	    $partnum++;
	}
    } else {
	($diskName, $partnum) = VMware::HConfig::RawDisk::unmakeLogVmkPartName($partname);
	if( !defined($diskName) || !defined($partnum) ) {
	    Warning("Internal Error: Could not add partition because could not get " .
		    "the disk name or partition number from partition name '$partname'.");
	    return(undef);
	}
    }

    # Get the type information
    my $parttype = hex($cgi->param('add_type'));
    if( !defined($parttype) ) {
	Warning("Internal Error: Could not get partition type.");
	return(undef);
    }

    # Get the start cylinder
    my $part_scyl = $cgi->param('add_start_cyl');
    if( !defined($part_scyl) || $part_scyl !~ /^[0-9]+$/ ) {
	Warning("Invalid start cylinder specified.");
	return(undef);
    }
    if( $part_scyl < 0 || $part_scyl >= $disk->getGeoCylinder() ) {
	Warning("Start cylinder does not fit within disk boundaries.  The start cylinder must " . 
		" be between 0 and " . $disk->getGeoCylinder() . " cylinders\n");
	return(undef);
    }

    # Get the end cylinder or size in megabytes
    my $part_end = $cgi->param('add_end_cyl');
    if( !defined($part_end) || ($part_end !~ /^[0-9]+$/ && 
				$part_end !~ /^\+[0-9]+[Mm]$/ ) ) {
	Warning("Invalid end boundary specified.  The end boundary must be a cylinder number or " .
		"of the form +sizeM.");
	return(undef);
    }

    my $part_ecyl;
    if( $part_end =~ /^\+([0-9]+)[Mm]$/ ) {
	# Mbytes were specified
	$part_end = $1 * 1024;            # kbytes
	$part_end = $disk->getKBytesToCyl($part_end);
	if( $part_end <= 0 ) {
	    $part_end = 1;
	}
	$part_ecyl = $part_scyl + $part_end - 1;
    } else {
	# Cylinders were specified
	$part_ecyl = $part_end;
    }

    if( $part_ecyl < 0 || $part_ecyl >= $disk->getGeoCylinder() ) {
	Warning("The requested end boundary is too large.  End cylinder does not fit within disk boundaries.  " .
		" The end cylinder must be between 0 and " . $disk->getGeoCylinder() . " cylinders\n");
	return(undef);
    }

    # Sanity checks
    if( $disk->getVmkName() ne $diskName ) {
	Warning("Trying to add partition on disk " . $diskName . " but got information " .
		"for disk " . $disk->getVmkName() . ".");
	return(undef);
    }
    if( $part_ecyl < $part_scyl ) {
	Warning("Invalid start and end cylinders.  End cylinder must be greater than start cylinder.\n");
	return(undef);
    }


    if( defined($partitionTable->getPart($partnum)) ) {
	Warning("Partition $partnum on disk $diskName already exists.");
	return(undef);
    }

    # Calculate the amount of unallocated space.  Must use this function in the case
    # that the cylinder number is the size of the disk.  The cylinder number is out
    # the range of the CHS.
    my $part_sect = VMware::HConfig::RawDisk::fromChsToLba($part_ecyl - $part_scyl + 1, 0, 1,
							   $disk->getGeoCylinder() + 1,
							   $disk->getGeoHead(),
							   $disk->getGeoSector());

    # Add the partition if it's ok with our check function
    $partitionTable->addPart($partnum, $part_scyl, $part_ecyl,
			     $parttype, $part_sect, 0);
    
    # Save a reference to this partition object since it may get reordered
    my $saved_part = $partitionTable->getPart($partnum);

    # In the case of a logical partition, we may have to reorder them
    if( VMware::HConfig::RawDisk::isLogicalPart($partnum) ) {
	$partitionTable->reorderLogicalParts();
    }

    if( !$partitionTable->checkPartInfo() ) {
	$partitionTable->removePart($saved_part->{PARTNUM});
	Warning("Could not add partition.");
	return(undef);
    }
	
    return(undef);
}


# Make the changes for fdisk
sub commitPartitionFdisk($$$$) {
    my $stub = $p->profile();
    my($cgi, $host, $disk, $partitionTable) = @_;

    Debug(1, "Entered");

    my $partnum;
    if( !$disk->makePartitions($partitionTable) ) {
	Warning("Could not write partition information to disk.");
	return(undef);
    }

    # No longer need to depend on the temporary partitionTable.  We should
    # get the partition information directly from the disk.  So we just 
    # print it ourselves.

    # XXX We need to tell the user to reboot if the ioctl call to reread
    #     partition tables fails.

    return(toFdiskHTML($cgi, $host, $disk, $disk->getPartitionTable()));
}


# Prepares disk operations by unsetting the vmkdump partition.
#
# Returns undef on success, an error string on failure.
sub preDiskOperation($$) {
    my($host, $opt) = @_;

    if (($opt & $DISK_SET_VMKDUMP) && !$host->unsetVmkDumpPartition()) {
	return("Could not execute vmkdump to unset the dump partition.\n");
    }
    return undef;
}

# Finishes disk operations by setting the vmkdump partition.
#
# Returns undef on success, error string on failure.
sub postDiskOperation($$) {
    my($host, $opt) = @_;

    if (($opt & $DISK_SET_VMKDUMP) && !$host->setVmkDumpPartition()) {
	return("Could not execute vmkdump to set the dump partition.\n");
    }
    return undef;
}


# Show partition information for fdisk
sub showPartitionFdisk($$$$) {
    my($cgi, $host, $disk, $partitionTable) = @_;

    return(toFdiskHTML($cgi, $host, $disk, $partitionTable));
}


sub handleRequest($$) {
    my $stub = $p->profile();
    my ($cgi, $host) = @_;
    my $wizard = $cgi->param("wizard");

    my $url_viewpart = toWizardURL($URL_VIEWPART, $wizard);

    Debug(1, "Entered");

    my $req = $cgi->param($RCom);
    
    # XXX Hack
    $gPanicCallback = \&PanicCallbackHTML;

    $host->checkModules();

    # This part of the installation can only run if the VMnix is running.
    if( !$host->isVMKernel() ) {
	my $message =
	    "The VMkernel must be loaded in order to perform this step.  Please load the\n" .
	    "VMkernel before doing this step.  You may also have to load in a SCSI VMkernel\n" .
	    "module in order to edit any partitions.  You may skip this step if you do not\n" .
	    "wish to use the VMFS file system.\n";

        if ($wizard) {
           $message .= toHTMLNavWizard( [ $URL_PREV, $URL_NAME_PREV ],
                                        [ $URL_NEXT, $URL_NAME_NEXT ] );
        }

        return toErrorHTML("View SCSI Partitions Failure", $message);
    }

    my $rc = undef;

    if( !defined($req) ) {
	# do nothing
    } elsif( $req eq $RCom_SetMFS) {
	# Formatting a VMFS should not require the the vmkdump partition to be
	# unset and set.  However there is currently a bug where the vmkdump will
	# not be set following the creation of a vmkdump partition.  Since formatting
	# generally always follows partitioning, set the vmkdump partition here.
	# This workaround will do for now.  See bug 15474 for more details. -jhu
	my $diskErr;

	$diskErr = preDiskOperation($host, $DISK_SET_VMKDUMP);
	if (defined($diskErr)) {
	    return(toErrorHTML("Formatting VMFS", $diskErr));
	}

	$rc = setMFS($cgi, $host);

	$diskErr = postDiskOperation($host, $DISK_SET_VMKDUMP);
	if (defined($diskErr)) {
	    Warning($diskErr);
	}

	if (!defined($rc)) {
	    # Only force a redirect if the action succeeds
	    force_redirect(\%mimeHeaders, $url_viewpart);
	}
    } elsif( $req eq $RCom_ConvertMFS) {
	# Converting a VMFS should not require the the vmkdump partition to be
	# unset and set.  However there is currently a bug where the vmkdump will
	# not be set following the creation of a vmkdump partition.  Since formatting
	# generally always follows partitioning, set the vmkdump partition here.
	# This workaround will do for now.  See bug 15474 for more details. -jhu
	my $diskErr;

	$diskErr = preDiskOperation($host, $DISK_SET_VMKDUMP);
	if (defined($diskErr)) {
	    return(toErrorHTML("Formatting VMFS", $diskErr));
	}

	$rc = convertMFS($cgi, $host);

	$diskErr = postDiskOperation($host, $DISK_SET_VMKDUMP);
	if (defined($diskErr)) {
	    Warning($diskErr);
	}

	if (!defined($rc)) {
	    # Only force a redirect if the action succeeds
	    force_redirect(\%mimeHeaders, $url_viewpart);
	}
    } elsif( $req eq $RCom_SetMFSName) {
	my $diskErr;

	$diskErr = preDiskOperation($host, 0);
	if (defined($diskErr)) {
	    return(toErrorHTML("Setting VMFS Name", $diskErr));
	}

	$rc = setMFSName($cgi, $host);

	$diskErr = postDiskOperation($host, 0);
	if (defined($diskErr)) {
	    Warning($diskErr);
	}

	if( !defined($rc) ) {
	    # Only force a redirect if the action succeeds
	    force_redirect(\%mimeHeaders, $url_viewpart);
	}
    } elsif( $req eq $RCom_SetMFSAccessibility ) {
	$rc = setMFSAccessibility($cgi, $host);
	if( !defined($rc) ) {
	    # Only force a redirect if the action succeeds
	    force_redirect(\%mimeHeaders, $url_viewpart);
	}
    } elsif( $req eq $RCom_AttachMFS) {
        $rc = attachMFS($cgi, $host);

        if (!defined($rc)) {
            # Only force a redirect if the action succeeds
            force_redirect(\%mimeHeaders, $url_viewpart);
        }
    } elsif($req eq $RCom_Create) {
	# When creating and deleting partitions, we must unset vmkdump
	# in case the partition information changes and the vmkdump partition
	# info is no longer correct.  The general rule is that the first vmkdump
	# partition in order of Vmk SCSI adapter/target is used as the vmkdump 
	# partition.  We unset and set to be correct before and after the partition
	# change.
	# 
	# It's sort of a blunt instrument to keep the vmkdump partition consistent,
	# but it's simple.  The down side is we lose core dumps that occur during a 
	# partitioning operation.
	my $diskErr;

	$diskErr = preDiskOperation($host, $DISK_SET_VMKDUMP);
	if (defined($diskErr)) {
	    return(toErrorHTML("Creating Partitions", $diskErr));
	}

	$rc = createSimplePartition($cgi, $host);

	$diskErr = postDiskOperation($host, $DISK_SET_VMKDUMP);
	if (defined($diskErr)) {
	    Warning($diskErr);
	}

	if( !defined($rc) ) {
	    # Only force a redirect if the action succeeds
	    force_redirect(\%mimeHeaders, $url_viewpart);
	}
    } elsif($req eq $RCom_Delete) {
	my $diskErr;

	$diskErr = preDiskOperation($host, $DISK_SET_VMKDUMP);
	if (defined($diskErr)) {
	    return(toErrorHTML("Deleting Partitions", $diskErr));
	}

	$rc = deleteAllPartitions($cgi, $host);

	$diskErr = postDiskOperation($host, $DISK_SET_VMKDUMP);
	if (defined($diskErr)) {
	    Warning($diskErr);
	}

	if( !defined($rc) ) {
	    # Only force a redirect if the action succeeds
	    force_redirect(\%mimeHeaders, $url_viewpart);
	}
    } elsif($req eq $RCom_Fdisk) {
	# Get the action that the user is performing in Fdisk.
	my $action = $cgi->param($Fdisk);
	if( !defined($action) ) {
	    $action = "";
	}

	my($disk, $partitionTable) = getDiskPartInfoFdisk($cgi, $host);
	if( !defined($disk) ) {
	    return(toErrorHTML('Fdisk Failure',
			       "Could not find disk information."));
	}
	if( !defined($partitionTable) ) {
	    return(toErrorHTML('Fdisk Failure',
			       "Could not find partition information for disk."));
	}

	if( $action eq $Fdisk_Delete ) {
	    $rc = deletePartitionFdisk($cgi, $host, $disk, $partitionTable);
	} elsif( $action eq $Fdisk_Change ) {
	    $rc = changePartitionFdisk($cgi, $host, $disk, $partitionTable);
	} elsif( $action eq $Fdisk_Add ) {
	    $rc = addPartitionFdisk($cgi, $host, $disk, $partitionTable);
	} elsif( $action eq $Fdisk_Show ) {
	    # Do nothing.
	} elsif( $action eq $Fdisk_Commit ) {
	    my $diskErr;

	    $diskErr = preDiskOperation($host, $DISK_SET_VMKDUMP);
	    if (defined($diskErr)) {
		return(toErrorHTML("Creating Partitions", $diskErr));
	    }
	    
	    $rc = commitPartitionFdisk($cgi, $host, $disk, $partitionTable);

	    $diskErr = postDiskOperation($host, $DISK_SET_VMKDUMP);
	    if (defined($diskErr)) {
		Warning($diskErr);
	    }

	    # Only because commit PartitionFdisk returns a valid HTML page
	    if( defined($rc) ) {
		my $url_self = $cgi->url(-relative=>1);
                $url_self = toWizardURL($url_self, $wizard);
		force_redirect(\%mimeHeaders, $url_self);
		return($rc);
	    }
	} else {
	    Warning("No action specified.");
	}
	
	if( !defined($rc) ) {
	    return(showPartitionFdisk($cgi, $host, $disk, $partitionTable));
	} else {
	    return(toErrorHTML("Fdisk Error", $rc));
	}
    } elsif( $req eq "" ) {
	# do nothing
    } elsif( $req eq $RCom_Debug ) {
	return("<pre>\n" . $host->toString() . "</pre>\n");
    } else {
	return(toHTMLPath(\@htmlPath, $htmlTitle) .
               toHTMLSystemInfo($host) . 
               toHTMLNotImplemented($req));
    }
    
    if( !defined($rc) ) {
	return(toViewPartitionHTML($cgi, $host));
    } else {
	return(toErrorHTML("Fdisk Error", $rc));
    }
}


sub force_redirect($$) {
    my $stub = $p->profile();
    my($mimeHeaders, $url) = @_;
    Debug(1, "Redirecting to '$url'");

    if( !defined($mimeHeaders->{'Location'}) ) {
	$mimeHeaders->{'Location'} = $url;
    }
}

sub handler {
    my $stub = $p->profile();
    # Delete the PATH to get by the tainted security requirements of Perl
    delete($ENV{'PATH'}); 

    umask 022;   # Create files that are not writable by group

    &clearWarnings();
    
    my $cgi = new CGI( shift( ) );
    my %credentials = hash_credentials( );
    
    %mimeHeaders = %MimeHeaders;

    my $user = $credentials{ "l" };
    my $password = $credentials{ "m" };

    my $s = toHTMLHeader($htmlTitle, $user);

    if( $user ne 'root' ) {
	my $url_self = $cgi->url(-relative=>1);
        $s .= toHTMLPath(\@htmlPath, $htmlTitle) . toHTMLNotRoot($url_self);
        goto footer;
    }
    
    my $host = VMware::HConfig::SSHostRemote->new(undef, 0, $user, $password);

    if(!defined($host)) {
       $s .= toHTMLPath(\@htmlPath, $htmlTitle) . toHTMLHostFailure($htmlTitle);
       goto footer;
    }

    $s .= handleRequest($cgi, $host);

footer:
    $s .= toHTMLFooter();
    return ($s, %mimeHeaders);
}
