# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::Session;
# File: VMware/Management/Session.pm

use strict;
use MIME::Base64 ();

require Exporter;

use vars qw( @ISA @EXPORT );
@ISA = qw( Exporter );
@EXPORT = qw( validate_session
              session_is_valid
	      delete_session
	      get_temp_context
	      get_updates
	      );

use VMware::ModVmdb;
use VMware::VmdbPerl;

#
# TODO:
#   o Cleanup sessions when Apache shuts down.
#
############################################################
##
## validate_session_vmdb
##   Given a valid session id, extend its expiration time.
##   If no session id is given, a new session is created.
##   This is the vmdb-based session function.
##
## Returns:
##   New session cookie.
##
## Side effects:
##   A new session is created in VMDB, if necessary.
##
############################################################
sub validate_session_vmdb {
    my $sc = shift();
    my $l = shift();
    my $m = shift();

    if ($sc) {
	my $sid = MIME::Base64::decode_base64($sc);
        my $newsid = VMware::ModVmdb::ValidateSession($sid);
	$sc = "vmware.mui.sid=" . MIME::Base64::encode_base64($newsid, "");
	chomp($sc);
    } else {
	my $sessionLength = $ENV{"vmware_SESSION_LENGTH"};
	if (!defined($sessionLength) || ($sessionLength < -1)) {
	    return "$sc;path=/;";
	} else {
	    $sessionLength *= 60;   # make it seconds
	}
	my $newsid = VMware::ModVmdb::NewSession($l, $m, $sessionLength);
	$sc = "vmware.mui.sid=" . MIME::Base64::encode_base64($newsid, "");
   	chomp($sc);
    }
    
    return "$sc;path=/;";
}

############################################################
##
## validate_session
##   Given a valid session id, extend its expiration time.
##
## Returns:
##   New session cookie.
##
## Side effects:
##
############################################################
sub validate_session {
    my ($sc, $username, $password) = @_;

    if (&VMware::Management::Util::use_vmdb()) {
       $username = MIME::Base64::decode_base64($username);
       $password = MIME::Base64::decode_base64($password);
        
       return validate_session_vmdb($sc, $username, $password);
    }

    return &VMware::Management::Util::validate_session_non_vmdb($username, $sc);
}


############################################################
##
## session_is_valid_vmdb
##
##   Check if the given session is valid.
##   This is the vmdb-based session function.
##
## Returns:
##   TRUE if the session is valid. FALSE otherwise.
##
## Side effects:
##   If the session duration has expired, it will be deleted.
##   In that case this function returns FALSE.
##
############################################################
sub session_is_valid_vmdb {
    my $sc = shift();

    $sc = MIME::Base64::decode_base64($sc);

    # IsSessionValid deletes the vmdb session if it is no longer valid.
    my $isValid = VMware::ModVmdb::IsSessionValid($sc);
    
    return $isValid;
}


############################################################
##
## session_is_valid
##
##   Check if the given session is valid.
##
## Returns:
##   TRUE if the session is valid. FALSE otherwise.
##
## Side effects:
##
############################################################
sub session_is_valid {
    if (&VMware::Management::Util::use_vmdb()) {
	return session_is_valid_vmdb(@_);
    }

    return &VMware::Management::Util::session_is_valid_non_vmdb(@_);
}

############################################################
##
## delete_session
##
##   Deletes the session given by the session key
##
## Returns:
##
## Side effects:
##
############################################################
sub delete_session($) {
   my ($key) = @_;

   if (!&VMware::Management::Util::use_vmdb()) {
       return;
   }

   $key = MIME::Base64::decode_base64($key);
   VMware::Management::Util::trace("========= delete_session called with key $key \n", "error");
   VMware::ModVmdb::DeleteSession($key);
}

############################################################
##
## get_temp_context
##
##   Get handle to a temporary Vmdb Context that can be used to
##   Get/Set vmdb data.
##
## Returns:
##   An object of class VMware::VmdbPerl::VmdbCtx if successful.
##   undef in case of error or an invalid session.
##
## Side effects:
##   A new context object is allocated. It is the responsibility
##   of the caller to free this context.
##
############################################################
sub get_temp_context {
   my %cookies = &VMware::Management::Util::hash_cookies();
   my $key = $cookies{"vmware.mui.sid"};

   if (!&VMware::Management::Util::use_vmdb()) {
       return;
   }

   $key = MIME::Base64::decode_base64($key);

   my $ctx = VMware::ModVmdb::GetTempContext($key);

   return $ctx;
}

############################################################
##
## get_full_context
##
##   Get handle to a temporary Vmdb Context that has full access
##   to vmdb, no matter who the caller is. Use with care.
##
## Returns:
##   An object of class VMware::VmdbPerl::VmdbCtx if successful.
##   undef in case of error
##
## Side effects:
##   A new context object is allocated. It is the responsibility
##   of the caller to free this context.
##
############################################################
sub get_full_context {
  return VMware::ModVmdb::GetNoUserContext();
}

############################################################
##
## get_updates
##
##   Returns an array of updates to the vmdb paths that this
##   session has mounted.
##
## Returns:
##   An array of hash references. Each hash has three fields,
##   PATH, VALUE and ACTION that describe the vmdb path and
##   the action (should be either SET or UNSET) and the value
##   that it was assigned in case of SET.
##
## Usage:
##   Caller should use this function as follows:
##      @updates = get_updates();
##      for (my $i=0; $i < scalar(@updates); $i++) {
##         my $path   = $updates[$i]->{"PATH"};
##         my $value  = $updates[$i]->{"VALUE"};
##         my $action = $updates[$i]->{"ACTION"};
##      }
##
## Side effects:
##
############################################################
sub get_updates {
   my %cookies = &VMware::Management::Util::hash_cookies();
   my $key = $cookies{"vmware.mui.sid"};

   if (!&VMware::Management::Util::use_vmdb()) {
       return;
   }

   $key = MIME::Base64::decode_base64($key);

   return VMware::ModVmdb::GetUpdates($key);
}

############################################################
##
## get_username
##
##   Returns the username for the session.
##
## Returns:
##   Username for the session.
##
############################################################
sub get_username {
   my %cookies = &VMware::Management::Util::hash_cookies();
   my $key = $cookies{"vmware.mui.sid"};

   if (!&VMware::Management::Util::use_vmdb()) {
       return;
   }

   $key = MIME::Base64::decode_base64($key);

   return VMware::ModVmdb::GetUsername($key);
}

############################################################
##
## get_password
##
##   Returns the passwrod for the session.
##
## Returns:
##   Password for the session.
##
############################################################
sub get_password {
   my %cookies = &VMware::Management::Util::hash_cookies();
   my $key = $cookies{"vmware.mui.sid"};

   if (!&VMware::Management::Util::use_vmdb()) {
      return;
   }

   $key = MIME::Base64::decode_base64($key);

   return VMware::ModVmdb::GetPassword($key);
}

############################################################
##
## get_usertoken
##
##   Returns the user token for the session. 
##   Used to pass into commands
##
## Returns:
##   Usertoken for the session.
##
############################################################
sub get_usertoken {
   my %cookies = &VMware::Management::Util::hash_cookies();
   my $key = $cookies{"vmware.mui.sid"};

   if (!&VMware::Management::Util::use_vmdb()) {
       return;
   }

   $key = MIME::Base64::decode_base64($key);

   return VMware::ModVmdb::GetUserToken($key);
}


############################################################
##
## user_getrights
##
##   Returns the user rights that the current user has.
##
## Returns:
##   User rights for the session
##
############################################################
sub user_getrights {
   my %cookies = &VMware::Management::Util::hash_cookies();
   my $key = $cookies{"vmware.mui.sid"};

   if (!&VMware::Management::Util::use_vmdb()) {
       return 0;
   }

   $key = MIME::Base64::decode_base64($key);

   my $usertoken = VMware::ModVmdb::GetUserToken($key);
   # Need a non-user specific context to explore fully.
   my $ctx = get_full_context();
   my $rights = $ctx->GetInt("/access/vmuser/user/#$usertoken/hostRights");
   return $rights;
}

############################################################
##
## user_is_admin
##
##   Determines whether the current user is an admin.
##
## Returns:
##   1 if user is admin, 0 if not.
##
############################################################
sub user_is_admin {
   my $rights = user_getrights();
   if ($rights & VMware::ModVmdb::VMUSER_HOST_ADMINACCESS) {
     return 1;
   }
   return 0;
}

############################################################
##
## user_can_create_vms
##
##   Determines whether the current user has create vm rights
##
## Returns:
##   1 if user can create vms, 0 if not.
##
############################################################
sub user_can_create_vms {
   my $rights = user_getrights();
   if ($rights & VMware::ModVmdb::VMUSER_HOST_CREATEACCESS) {
     return 1;
   }
   return 0;
}

1;
