# ---------------------------------------------------------------------------
# Copyright 2003 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::SxSanConfiguration;
# File: VMware/Management/SxSanConfiguration.pm

use strict;

use VMware::Management::Util ();
use VMware::HConfig::SSHostRemote ();
use VMware::HConfig::PCIDevice ();
use VMware::HConfig::RawDisk;

use CGI ();
my $q;


# ---------------------------------------------------------------------------

sub handler {
   $q = new CGI(shift());
   my $data = "";

   my %credentials = VMware::Management::Util::hash_credentials();
   my $user = $credentials{"l"};
   my $pass = $credentials{"m"};
   my $host = VMware::HConfig::SSHostRemote->new(undef, 0, $user, $pass);

   if (! defined($host)) {
      my $data = error("Could not create remote host object.");
      return html($data);
   }

   if (! $host->isVMKernel()) {
      my $data = error("Could not determine SAN configuration: " .
                       "The VMkernel is not loaded.");
      return html($data);
   }

   my $op = $q->param("op");
   if ($op =~ /^rescan$/i) {
      rescan($host);
   } elsif($op =~ /^save$/i) {
     return saveMultipathData($host);
   }

   return data($host, $data);
}

# ---------------------------------------------------------------------------

sub data {
   my $host = shift();
   my $data = shift() || "";
   my ($ok, $ret);

   $data .= "var data={};$/";

   $data .= "function SanDev(dnm,drvrs,slctDrvr,bus,dev,fun,wwpn,trgts){" .
      "this.dnm=dnm;" .           # Device name (manufacturer, model, etc.)
      "this.drvrs=drvrs;" .       # List of compatible drivers...
      "this.slctDrvr=slctDrvr;" . # Index of selected driver
      "this.bus=bus;" .           # PCI bus number
      "this.dev=dev;" .           # PCI device number
      "this.fun=fun;" .           # PCI function number
      "this.wwpn=wwpn;" .         # World wide port name
      "this.trgts=trgts;}$/";     # A hash of SCSI ID/WWPN pairs.

   $data .= "function SanPth(id,brkn,enbld,actv,prfrd){" .
      "this.id=id;" .             # vmhbaX:X:X
      # "this.trgt=trgt;" .       # Target WWPN (set up by initPthTrgts)
      "this.brkn=brkn;" .         # Broken boolean
      "this.enbld=enbld;" .       # Enabled boolean
      "this.actv=actv;" .         # Active boolean
      "this.prfrd=prfrd;}$/";     # Preferred boolean

   $data .= persistentBindingsData($host) . $/;
   $data .= multipathData($host) . $/;

   # This looks like a hack, but it actually saves sending duplicate WWPNs for
   # each path. For 32 paths (16 LUNs on 4 adapters with 1 target each), the
   # size of initPthTrgts is over two thirds smaller than the WWPNs generated.
   $data .= "function initPthTrgts() {$/\t" .
              "for (var n in data.luns) {$/\t\t" .
                "for (var i = 0; i < data.luns[n].length; i++) {$/\t\t\t" .
                  "var a = data.luns[n][i].id.split(':');$/\t\t\t" .
                  "data.luns[n][i].trgt = data.adapters[a[0]].trgts[a[1]];$/\t\t\t" .
                "}$/\t" .
              "}$/" .
            "}$/";

   return html($data, "initPthTrgts(); if (parent.opCb) parent.opCb(self);");
}


sub getLun {
  my ($cn, $luns) = @_;
  for my $lun (@$luns) {
    my $cnm = $lun->getVmkName();
    if ($cnm eq $cn) {
      return $lun;
    }
  }
  return undef;
}


sub saveMultipathData {
  my $host   = shift();
  my $cnm    = $q->param("lun"); # canonical name
  my $error  = "";
 
  if (!defined($cnm)) {
    # this error should never happen. 
    $error =  error("Save Failoverpath Configuration:" .
                    "No LUN chosen for edit!.");
    return html($error);
  }
  
  my @adapters = @{$host->getVMSCSIAdapters()};
  my @luns = grep {isSerialFiberLun($_, @adapters)} @{$host->getVMSCSIDisks()};
  
  if (scalar(@luns) <= 0) {
    $error = error("Save Failoverpath Configuration:" .
                   "No serial fiber LUNs were found.");
    return html($error);
  }

  my $lun = getLun($cnm, \@luns);
  if (!defined($lun)) {
    $error = error("Save Failoverpath Configuration:" .
                   "Unable to identify the selected LUN: $lun");
    return html($error);
  }
  
  my $policy = $q->param("plcy");
  if (defined($policy)) {
    $error = setMultipathPolicy($lun, $policy);
    if (defined($error)) {
      return html($error);
    }
  }

  my $path  = $q->param("prfrd");
  if (defined($path)) {
    $error = setMultipathPrefer($lun, $path);
    if (defined($error)) {
      return html($error);
    }
  }

  $path  = $q->param("actv");
  if (defined($path)) {
    $error = setMultipathActive($lun, $path);
    if (defined($error)) {
      return html($error);
    }
  }
  
  my @paths = split(";", $q->param("idle")); # semicolon seperated paths
  foreach (@paths) {
    if (defined($_)) {
      $error = enableMultipathPath($lun, $_);
      if (defined($error)) {
        return html($error);
      }
    }
  }

  @paths = split(";", $q->param("dsbld")); # semicolon seperated paths.
  foreach (@paths) {
    if (defined($_)) {
      $error = disableMultipathPath($lun, $_);
      if (defined($error)) {
        return html($error);
      }
    }
  }

  return data($host, "");
}

#
# Supported Policies:
# Fixed: Always use the preferred path when available.
# Most Recently Used: Always use the last active path.

sub setMultipathPolicy {
  my $lun = shift();
  my $newPlcy = shift(); # A new policy requested by the user.

  if (!defined($lun)) {
    # this error should never happen. 
    return error("Save Failoverpath Configuration: No LUN chosen for edit!.");
  }
  
  $newPlcy ||= $q->param("plcy");
  my $oldPlcy = $lun->getMultipathPolicy();
  if ($newPlcy ne $oldPlcy) { # if equal then there is nothing to set.
    if ($newPlcy !~ /^fixed|mru|blncd$/i) {
      return error("Save Failoverpath Configuration:" .
                   "Unsupported policy: $newPlcy.");
    }
    # XXX - Should propagate the real error message. 
    if (!$lun->setMultipathPolicy($newPlcy)) {
      return error("Save Failoverpath Configuration:" .
                   "Unable to set policy: $newPlcy");
    }
  }
  
  return undef;
}

sub setMultipathPrefer {
  my $lun = shift();
  my $newPath = shift(); # Path that is requested by the user to be preferred.
  
  if (!defined($lun)) {
    # this error should never happen. 
    return error("Save Failoverpath Configuration: No LUN chosen for edit!.");
  }
  
  $newPath ||= $q->param("prfrd");
  my $oldPath = $lun->getMultipathPrefer();
  # No need to set a path preferred that is already the preferred path.
  if ($newPath ne $oldPath) {
    # XXX - Should propagate the real error message
    if (!$lun->setMultipathPrefer($newPath)) {
      return error("Save Failoverpath Configuration: Unable to set " .
                   "$newPath to be the preferred path.");
    }
  }
  return undef;
}

sub setMultipathActive {
  my $lun = shift();
  my $newPath = shift(); # Path that is requested by the user to be active.
  
  if (!defined($lun)) {
    # this error should never happen. 
    return error("Save Failoverpath Configuration: No LUN chosen for edit!.");
  }
  
  $newPath ||= $q->param("actv");
  my $oldPath = $lun->getMultipathActive();
  # No need to set a path active that is already the active path.
  if ($newPath ne $oldPath) {
    # XXX - Should propagate the real error message
    if (!$lun->setMultipathActive($newPath)) {
      return error("Save Failoverpath Configuration: Unable to set " .
                   "$newPath to be the active path.");
    }
  }
  return undef;
}

sub enableMultipathPath {
  my $lun = shift();
  my $newPath = shift(); # Path that is requested by the user to be enabled.
  
  if (!defined($lun)) {
    # this error should never happen. 
    return error("Save Failoverpath Configuration: No LUN chosen for edit!.");
  }
    
  $newPath ||= $q->param("idle");
  
  # More than one path may reach the same LUN. Go through the
  # list of paths and enable the requested path.
  my @paths = $lun->getMultipathPaths();
  for (@paths) {
    if ($_->{PATH} eq $newPath) {
      # Before enabling the path make sure it is alive.
      my $brkn = $_->{STATUS} eq "dead" ? 1 : 0;
      if ($brkn) {
        return error("Save Failoverpath Configuration:" .
                     "Cannot enable a broken path: $newPath.");
      } else {
        # No need to enable a path that is already enabled.
        if ($_->{STATUS} ne "on") {
          $lun->enableMultipathPath($newPath);
        } # else nothing to do...
        # intentionally falling through.
      }
      last;
    }
  }
  
  return undef;
}

sub disableMultipathPath {
  my $lun = shift();
  my $newPath = shift(); # Path that is requested by the user to be disabled.
  
  if (!defined($lun)) {
    # this error should never happen. 
    return error("Save Failoverpath Configuration: No LUN chosen for edit!.");
  }
    
  $newPath ||= $q->param("dsbld");
  
  # More than one path may reach the same LUN. Go through the
  # list of paths and disable the requested path.
  my @paths = $lun->getMultipathPaths();
  for (@paths) {
    if ($_->{PATH} eq $newPath) {
      # Before disabling the path make sure it is alive.
      my $brkn = $_->{STATUS} eq "dead" ? 1 : 0;
      if ($brkn) {
        return error("Save Failoverpath Configuration:" .
                     "Cannot disable a broken path: $newPath.");
      } else {
        # No need to disable a path that is already disabled.
        if ($_->{STATUS} eq "on") {
          $lun->disableMultipathPath($newPath);
        } # else nothing to do...
        # intentionally falling through.
      }
      last;
    }
  }
  
  return undef;
}

# ---------------------------------------------------------------------------

sub persistentBindingsData {
   my $host = shift();
   my $data = "";

   my $sanDevData = "";
   my @adapters = @{$host->getVMSCSIAdapters()};
   for my $adapter (sort {$a->getVmkName() cmp $b->getVmkName()} @adapters) {
      if ($adapter->getPCIDevice()->isSerialFiberDevice()) {
         my $vmkName = $adapter->getVmkName();

         my $trgtStr = "";
         my @targets = sort {$a->{target} <=> $b->{target}} @{$adapter->getSanDevices()};
         for (@targets) {
            my $wwpn = $_->{wwpn};
            if ($wwpn !~ /:/) {
               $wwpn =~ s/(\w{2})/$1:/g;
               chop($wwpn);
            }
            $wwpn = uc($wwpn);

            $trgtStr .= ($trgtStr ? "," : "");
            $trgtStr .= qq{"$_->{target}":"$wwpn"};
         }
         $trgtStr = "{$trgtStr}";

         my $dnm = $adapter->getPCIDevice()->getName();
         my $drvrs = join('","', @{$adapter->getPCIDevice()->getDrivers()});
         # XXX: $slctDrvr is fixed for the moment. But we need to permit selection
         # for certain SAN devices.
         my $slctDrvr = 0;
         my $bus = $adapter->getPCIDevice()->getBus();
         my $dev = $adapter->getPCIDevice()->getDevice();
         my $fun = $adapter->getPCIDevice()->getFunction();
         my $wwpn = $adapter->getWWPN();
         if ($wwpn !~ /:/) {
            $wwpn =~ s/(\w{2})/$1:/g;
            chop($wwpn);
         }
         $wwpn = uc($wwpn);
         $sanDevData .= ($sanDevData ? ",$/" : "") . qq{"$vmkName":} .
            qq{new SanDev("$dnm",["$drvrs"],$slctDrvr,$bus,$dev,$fun,"$wwpn",$trgtStr)};
      }
   }
   $data .= "data.adapters={$sanDevData};";

   return $data;
}


# ---------------------------------------------------------------------------

sub rescan {
   my $host = shift();
   my $sanDev = shift();
   my $ok = 1;

   my @adapters = @{$host->getVMSCSIAdapters()};
   for (@adapters) {
      if ($_->getPCIDevice()->isSerialFiberDevice()) {
         $host->rescanLuns($_->getVmkName());
      }
   }

   my %persisted;
   for (@adapters) {
      if ($_->getPCIDevice()->isSerialFiberDevice()) {
         # XXX: This probably isn't safe if their are multiple drivers. But
         # it's exactly what we were doing before...
         my ($sanDrvr) = @{$_->getPCIDevice()->getDrivers()};
         if (! $persisted{$sanDrvr}) {
            $host->persist_bindings($sanDrvr);
            $persisted{$sanDrvr} = 1;
         }
      }
   }
}


# ---------------------------------------------------------------------------

sub isSerialFiberLun {
   my ($adapter) = VMware::HConfig::RawDisk::unmakeLogVmkDiskName(shift()->getVmkName());

   return grep {$_->getPCIDevice()->isSerialFiberDevice() &&
      $_->getVmkName() eq $adapter} @_;
}


# ---------------------------------------------------------------------------

sub multipathData {
   my $host = shift();
   #    my %targets = %{shift()};
   my $data = "";

   my @adapters = @{$host->getVMSCSIAdapters()};
   my @luns = grep {isSerialFiberLun($_, @adapters)} @{$host->getVMSCSIDisks()};

   my $lunData = "";
   my $plcyData = "";
   for my $lun (@luns) {
      my $cnm = $lun->getVmkName();

      my $pathData = "";
      my @paths = $lun->getMultipathPaths();
      for (@paths) {
         my $id = $_->{PATH};
         my $brkn = $_->{STATUS} eq "dead" ? 1 : 0;
         my $enbld = $_->{STATUS} eq "on" ? 1 : 0;
         my $actv = $id eq $lun->getMultipathActive() ? 1 : 0;
         my $prfrd = $id eq $lun->getMultipathPrefer() ? 1 : 0;

         $pathData .= ($pathData ne "" ? "," : "");
         $pathData .= qq{new SanPth("$id",$brkn,$enbld,$actv,$prfrd)};
      }
      $pathData = "[$pathData]";

      $lunData .= ($lunData ne "" ? ",$/" : "");
      $lunData .= qq{"$cnm":$pathData};

      my $plcy = $lun->getMultipathPolicy();
      $plcyData .= ($plcyData ne "" ? ",$/" : "");
      $plcyData .= qq{"$cnm":"$plcy"};
   }

   # XXX: Policy data should be rolled into the luns hash, but doing so would
   # involve a lot of refactoring on the client. Split policy out for now.
   $data .= "data.luns={$lunData};$/";
   $data .= "data.plcy={$plcyData};";

   return $data;
}


# ---------------------------------------------------------------------------

sub error {
   my $msg = shift();
   my ($pkg, $fn, $ln) = caller();
   VMware::Management::Util::trace($msg, "error", undef, $pkg, $fn, $ln);
   return "err.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------

sub warning {
   my $msg = shift();
   my ($pkg, $fn, $ln) = caller();
   VMware::Management::Util::trace($msg, "warning", undef, $pkg, $fn, $ln);
   return "wrn.push(" . jsStringLiteral($msg) . ");$/";
}


# ---------------------------------------------------------------------------
#
# jsStringLiteral --
#
#      Make sure that $str is safe for consumption in a JavaScript program.
#
# ---------------------------------------------------------------------------

sub jsStringLiteral {
   my $str = shift();
   $str =~ s/\\/\\\\/g;
   $str =~ s/("|')/\\$1/g;
   my $lf = $/;
   $str =~ s/$lf/\\n/g;

   return qq{"$str"};
}


# ---------------------------------------------------------------------------

sub html {
   my $data = shift();
   my $cb = shift() || "if (parent.opCb) parent.opCb(self);";

   my @warnings = VMware::ExtHelpers::getWarnings();
   for (@warnings) {
      $data .= warning($_);
   }
   VMware::ExtHelpers::clearWarnings();

   my $html = qq[<html><head><title></title><script>$/] .
      qq[var err=[];var wrn=[];$/] .
      qq[$data$/</script></head><body onload="$cb"></body></html>];

   return($html,
          "Pragma"        => "no-cache",
          "Cache-Control" => "no-cache",
          "Content-Type"  => "text/html; charset=utf-8");
}

1;
