# ---------------------------------------------------------------------------
# Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
# ---------------------------------------------------------------------------
package VMware::Management::Overview;
# File: VMware/Management/Overview.pm

use strict;

use VMware::Control '1.01';
use VMware::Control::Server;
use VMware::Control::VM;
use VMware::Control::Profiler;
use VMware::Management::Util qw( trace
                                 file_to_string
                                 string_to_file
                                 hash_credentials 
				 getProductName);

use XML::DOM ();
use XML::Sablotron ();
use CGI ();

my @periods = ( 60, 300, 900 );
my @methods = ( "min", "avg", "max" );
my %methods = ( $methods[ 0 ] => "min", $methods[ 1 ] => "mean", $methods[ 2 ] => "max" );

my $doc;
my $q;
my $p = VMware::Control::Profiler::getInstance("mui");

sub handler {
  my $stub = $p->profile();
  $q = new CGI( shift( ) );
  my %credentials = hash_credentials( );
  my $server = &VMware::Control::Server::new( undef, 0,
                                              $credentials{ "l" },
                                              $credentials{ "m" } );
  my $parser = new XML::DOM::Parser;
  my $version = VMware::Control::version( );
  my $processor = new XML::Sablotron();
  my $want_version = "1.01";
  my $xsl = $ENV{ "style_DEFAULT" };
  my $root;
  my $xml;
  
  my $method = $q->param('method') || "vmserverd";
  
  if (($method eq "vmserverd") && ($want_version eq $version)) {  
    if ( defined( $server ) && $server->connect( ) ) {
      trace( "Connected to server object ($server).", "notice" );
      my $ret = $server->exec("<exec><op>VM_INFO</op><in></in></exec>");
      
      # If $server->exec failed to return any value, don't enter the following
      # block. $xml will not be defined and the block of code after this one 
      # will try to contact each vmx process in turn to ascertain the vm state
      # and the state of the system. If $ret is defined, assume that it is
      # valid XML.
      if ( defined( $ret ) ) {
        $ret = VMware::DOMAccess->newXML($ret);
        $root = $ret->get("exec.out.server");
        $doc = $ret->getDocument();

        if (!defined($root)) {
          $ret->setValue("exec.out.server", "");
          $root = $ret->get("exec.out.server");
        }

        if ($ret->getValue("exec.returncode") != 1) {
          foreach my $i ($ret->listElementNames("exec.messages")) {
            &VMware::Management::Util::append_error($doc, $root, undef,
                                                    $ret->getValue("exec.messages.$i"));
          }
        }

        &VMware::Management::Util::append_user($doc, $root, $credentials{"l"});
        &VMware::Management::Util::append_agent($doc, $root);
        &VMware::Management::Util::append_build($doc, $root);
        &VMware::Management::Util::append_adminuser($doc, $root, $server);

	my $productName = &VMware::Management::Util::getProductName($server);

        if ($productName eq "ESX") {
          trace("ESXHostInfo_Query support not implemented.", "debug");
        } elsif ($productName eq "GSX") {
          my $req = $server->exec("<exec><op>" .
                                  "GSXHostInfo_Query" .
                                  "</op><in></in></exec>");

          if (defined($req)) {
            $req = VMware::DOMAccess->newXML($req);

            if ($req->getValue("exec.returncode") == 1) {
              for ($req->listElementNames("exec.out")) {
                $ret->addSubTree("exec.out.server.hostinfo",
                                 $req->getSubTree("exec.out.$_"));
              }
            } else {
              trace("GSXHostInfo_Query: $req" , "error");
            }
            $req->dispose();
          } 
        }

        $server->disconnect( );
        trace( "server->disconnect( ) called ($server).", "notice" );

        my $sub = $ret->getSubTree("exec.out.server");
        $xml = $sub->getValue();
        $sub->dispose( );
        $ret->dispose( );
      } 
    }
  }
  
  # $xml is not definied for some reason (maybe $server->exec failed). Try to
  # contact each vmx process to ascertain the vm state and the state of the
  # system.
  unless ( defined( $xml ) ) {
    $doc = $parser->parse( "<server></server>" );
    $root = $doc->getDocumentElement( );
    my @vms;
    my $vm;
    
    &VMware::Management::Util::append_error( $doc, $root, -9999, "Virtual machine control API version mismatch. Want: $want_version. Have: $version." )
      unless $want_version eq $version;

    my $connected = $server && $server->connect();

    &VMware::Management::Util::append_host( $doc, $root );
    &VMware::Management::Util::append_agent( $doc, $root );
    &VMware::Management::Util::append_user( $doc, $root, $credentials{ "l" } );
    &VMware::Management::Util::append_build( $doc, $root );
    &VMware::Management::Util::append_product( $doc, $root, $server );
    
    if ($connected) {
      trace( "Connected to server object ($server).", "notice" );
      
      @vms = $server->enumerate( );
      
      if ( scalar( @vms ) > 0 ) {
        $vm = VMware::Control::VM::new( $server, $vms[ 0 ] );
        
        if ( ! $vm->connect( ) ) {
          # &! XXX What happens when we can't reach the first VM?
          my ( $err, $errstr ) = $vm->get_last_error( );
          trace( "Could not connect to first VM (for host stats)", "error" );
          trace( $errstr, "error" );
          &VMware::Management::Util::append_error( $doc, $root, $err, $errstr );
        } else {          
          append_stats( $root, $server );
        }

        foreach my $i ( @vms ) {
          &VMware::Management::Util::append_vm( $doc, $root, $i, $server );
        }
      }
    } else {
      # &! XXX Return an error about the inability to connect to vmauthd 
      my ( $err, $errstr ) = $server->get_last_error( );
      trace( "Could not connect to vmware-authd", "error" );
      trace( $errstr, "error" );
      &VMware::Management::Util::append_error( $doc, $root, $err, $errstr );
    }
    
    if ( $server && $server->is_connected() ) {
      $server->disconnect( );
      trace( "server->disconnect( ) called ($server).", "notice" );
    }
    
    $xml = $doc->toString( );    
    $doc->dispose( );
  } 
  
  if ( $ENV{ "vmware_DEBUG_LEVEL" } > 2 ) {
    my $xmlfile = &VMware::Management::Util::tmpdir() . "overview.xml";
    string_to_file( $xml, $xmlfile, 1 );
  }

  $processor->SetBase( "file://" . $ENV{ "vmware_BASEHREF" } );
  $p->start("XML::Sablotron::RunProcessor");
  $processor->RunProcessor( "arg:/template", "arg:/data", "arg:/result",
                            undef,
                            [ "template", 
                              file_to_string( $xsl ), 
                              "data", 
                              $xml ] );
  $p->end();
  
  return ( $processor->GetResultArg("result"),
           "Pragma"        => "no-cache",
           "Cache-Control" => "no-cache",
           "Content-Type"  => "text/html; charset=iso-8859-1" );
}

sub append_stats {
  my ( $root, $server ) = @_;
  &VMware::Management::Util::append_uptime( $doc, $root, $server );
  &VMware::Management::Util::append_cpu( $doc, $root, $server );
  &VMware::Management::Util::append_ram( $doc, $root, $server );
}

1;
